@@ -26,10 +26,11 @@ import sqlite3.dbapi2 as sqlite
 import status as pstatus
 
 import logging
-log = logging.getLogger('e3.base.Logger') # oh snap!
+log = logging.getLogger('e3.base.Logger')  # oh snap!
 
 import e3
 
+
 class Account(object):
     '''a class to store account data'''
 
@@ -64,6 +65,7 @@ class Account(object):
         return cls(contact.cid, None, contact.account,
             contact.status, contact.nick, contact.message, contact.picture)
 
+
 class Group(object):
     '''a class that represents a group of contacts'''
 
@@ -79,6 +81,7 @@ class Group(object):
         '''return a string representation of the object'''
         return "<group '%s'>" % (self.name,)
 
+
 class Logger(object):
     '''a class to log activity on an IM'''
 
@@ -341,7 +344,7 @@ class Logger(object):
         full_path = os.path.join(path, db_name)
 
         if os.path.exists(full_path + "copy"):
-            shutil.copy (full_path + "copy", full_path)
+            shutil.copy(full_path + "copy", full_path)
 
         self.connection = sqlite.connect(full_path)
         self.cursor = self.connection.cursor()
@@ -557,7 +560,8 @@ class Logger(object):
     def insert_account_by_group(self, id_account, id_group):
         '''insert the relation between an account and a group'''
         try:
-            self.execute(Logger.INSERT_ACCOUNT_BY_GROUP, (id_account, id_group))
+            self.execute(Logger.INSERT_ACCOUNT_BY_GROUP,
+                         (id_account, id_group))
         except sqlite.IntegrityError, e:
             log.error(str(e))
 
@@ -596,7 +600,10 @@ class Logger(object):
 
     def add_event(self, event, status, payload, src, dest=None, ext_time=None,
             id_time=None, cid=0):
-        '''add an event on the fact and the dimensiones using the actual time'''
+        '''
+        add an event on the fact and the dimensiones using
+        the actual time
+        '''
 
         id_event = self.insert_event(event)
         (id_src_info, id_src_acc) = self.insert_info(src.account, src.id,
@@ -645,7 +652,8 @@ class Logger(object):
         if id_event is None:
             return None
 
-        self.execute(Logger.SELECT_ACCOUNT_EVENT, (id_event, id_account, limit))
+        self.execute(Logger.SELECT_ACCOUNT_EVENT,
+                     (id_event, id_account, limit))
 
         return self._fetch_sorted()
 
@@ -821,6 +829,36 @@ class Logger(object):
                 self.delete_account_by_group(local_account.id_account,
                                              local_group.id)
 
+
+class LoggerOutputProcess(threading.Thread):
+    '''a process that processes the output calls from the logger thread'''
+
+    def __init__(self):
+        '''constructor'''
+        threading.Thread.__init__(self)
+        self.setDaemon(True)
+        self.output = Queue.Queue()
+        self._close = False
+
+    def run(self):
+        '''main method'''
+        while True:
+            if self._close:
+                return
+
+            try:
+                action, result, callback = self.output.get(True)
+                callback(result)
+            except Queue.Empty:
+                pass
+
+    def close(self):
+        self._close = True
+
+    def put(self, *args):
+        self.output.put(*args)
+
+
 class LoggerProcess(threading.Thread):
     '''a process that exposes a thread safe api to log events of a session'''
 
@@ -833,7 +871,7 @@ class LoggerProcess(threading.Thread):
         self.db_name = db_name
         self.logger = None
         self.input = Queue.Queue()
-        self.output = Queue.Queue()
+        self.output = LoggerOutputProcess()
 
         self.actions = {}
 
@@ -841,6 +879,7 @@ class LoggerProcess(threading.Thread):
         '''main method'''
         data = None
         self.logger = Logger(self.path, self.db_name)
+        self.output.start()
 
         self.actions['get_event'] = self.logger.get_event
         self.actions['get_nicks'] = self.logger.get_nicks
@@ -861,10 +900,9 @@ class LoggerProcess(threading.Thread):
                 quit = self._process(data)
 
                 if quit:
+                    self.output.close()
                     self.logger.close()
-                    #log.debug('closing logger thread')
-                    break
-
+                    return
             except Queue.Empty:
                 pass
 
@@ -910,27 +948,6 @@ class LoggerProcess(threading.Thread):
     def input_size(self):
         return self.input.qsize()
 
-    def check(self, sync=False):
-        '''call this method from the main thread if you dont want to have
-        problems with threads, it will extract the results and call the
-        callback that was passed to the get_* call
-
-        the sync parameter is used for testing, it basically waits for a
-        message'''
-
-        try:
-            if sync:
-                action, result, callback = self.output.get()
-                callback(result)
-            else:
-                while True:
-                    action, result, callback = self.output.get(False)
-                    callback(result)
-        except Queue.Empty:
-            pass
-
-        return True
-
     def log(self, event, status, payload, src, dest=None, new_time=None,
             cid=None):
         '''add an event to the log database'''
@@ -986,13 +1003,16 @@ class LoggerProcess(threading.Thread):
         '''
         self.input.put(('get_chats_between', (src, dest, from_t, to_t, limit,
                                               callback)))
-    def get_chats_by_keyword(self, src, dest, from_t, to_t, keywords, limit, callback):
+
+    def get_chats_by_keyword(self, src, dest, from_t,
+                             to_t, keywords, limit, callback):
         '''return the last # sent from src to dest or from dest to src ,
         between two timestamps from_t and to_t, containing keywords,
         where # is the limit value
         '''
-        self.input.put(('get_chats_by_keyword', (src, dest, from_t, to_t, keywords,
-                                               limit, callback)))
+        self.input.put(('get_chats_by_keyword', (src, dest, from_t, to_t,
+                                                 keywords, limit, callback)))
+
     def add_groups(self, groups):
         '''add all groups to the database'''
         self.input.put(('add_groups', (groups, None)))
@@ -1005,6 +1025,7 @@ class LoggerProcess(threading.Thread):
         '''add all contacts, groups and relations to the database'''
         self.input.put(('add_contact_by_group', (contacts, groups, None)))
 
+
 def save_logs_as_txt(results, handle):
     '''save the chats in results (from get_chats or get_chats_between) as txt
     to handle (file like object)
@@ -1016,6 +1037,53 @@ def save_logs_as_txt(results, handle):
         date_text = time.strftime('[%c]', time.gmtime(timestamp))
         handle.write("%s %s: %s\n" % (date_text, nick, message))
 
+
+def save_logs_as_xml(results, handle):
+    '''save the chats in results (from get_chats or get_chats_between) as xml
+    to handle (file like object)
+
+    the caller is responsible of closing the handle
+    '''
+    from xml.dom.minidom import Document
+
+    doc = Document()
+    doc.appendChild(doc.createProcessingInstruction("xml-stylesheet",
+                                "type=\"text/css\" href=\"conversation.css\""))
+    conversation = doc.createElement("conversation")
+    doc.appendChild(conversation)
+
+    for stat, timestamp, message, nick, account in results:
+
+        timestamp_tag = doc.createElement("timestamp")
+        date_text = time.strftime('[%c]', time.gmtime(timestamp))
+        timestamp_text = doc.createTextNode(date_text)
+        timestamp_tag.appendChild(timestamp_text)
+
+        nick_tag = doc.createElement("nick")
+        nick_text = doc.createTextNode(nick)
+        nick_tag.appendChild(nick_text)
+        timestamp_tag.appendChild(nick_tag)
+
+        status_tag = doc.createElement("status")
+        status_text = doc.createTextNode(str(stat))
+        status_tag.appendChild(status_text)
+        timestamp_tag.appendChild(status_tag)
+
+        account_tag = doc.createElement("account")
+        account_text = doc.createTextNode(account)
+        account_tag.appendChild(account_text)
+        timestamp_tag.appendChild(account_tag)
+
+        message_tag = doc.createElement("message")
+        message_text = doc.createTextNode(message)
+        message_tag.appendChild(message_text)
+        timestamp_tag.appendChild(message_tag)
+
+        conversation.appendChild(timestamp_tag)
+
+    handle.write(doc.toprettyxml(indent="  ",  encoding="UTF-8"))
+
+
 def log_message(session, members, message, sent, error=False, cid=None):
     '''log a message, session is an e3.Session object, members is a list of
     members only used if sent is True, sent is True if we sent the message,
@@ -1045,10 +1113,7 @@ def log_message(session, members, message, sent, error=False, cid=None):
 
         if len(members) == 1:
             member = members[0]
-            dst = session.contacts.get(members[0])
-
-            if dst is None:
-                dst = e3.Contact(members[0])
+            dst = session.contacts.safe_get(members[0])
 
             dest = e3.Logger.Account.from_contact(dst)
 
@@ -1058,10 +1123,7 @@ def log_message(session, members, message, sent, error=False, cid=None):
         else:
 
             for dst_account in members:
-                dst = session.contacts.get(dst_account)
-
-                if dst is None:
-                    dst = e3.Contact(dst_account)
+                dst = session.contacts.safe_get(dst_account)
 
                 dest = e3.Logger.Account.from_contact(dst)
                 logs.append((event, status, message.body, src, dest, cid))
