@@ -19,7 +19,7 @@
 class Message(object):
     '''a class that represent a msn message'''
     (TYPE_MESSAGE, TYPE_TYPING, TYPE_NUDGE, TYPE_P2P, TYPE_UNK,
-     TYPE_FLNMSG) = range(6)
+     TYPE_FLNMSG, TYPE_OLDMSG, TYPE_INFO) = range(8)
 
     def __init__(self, type_, body, account, style=None, timestamp=None,
                  display_name=None):
