@@ -27,6 +27,7 @@ from Action import Action
 
 import e3
 import Logger
+import extension
 from ContactManager import ContactManager
 
 class Session(object):
@@ -36,9 +37,12 @@ class Session(object):
     AUTHOR = 'Mariano Guerra'
     WEBSITE = 'www.emesene.org'
 
+    CAPABILITIES = range(12)
+
     (SERVICE_CONTACT_MANAGING, SERVICE_CONTACT_ALIAS, SERVICE_CONTACT_BLOCK,
      SERVICE_CONTACT_INVITE, SERVICE_GROUP_MANAGING, SERVICE_FILETRANSFER,
-     SERVICE_CALLS, SERVICE_PROFILE_PICTURE, SERVICE_STATUS) = range(9)
+     SERVICE_CALLS, SERVICE_PROFILE_PICTURE, SERVICE_STATUS, SERVICE_CONTACT_NICK,
+     SERVICE_CONTACT_PM, SERVICE_ENDPOINTS) = CAPABILITIES
 
     def __init__(self, id_=None, account=None):
         self.id_ = id_
@@ -50,7 +54,7 @@ class Session(object):
         self.contacts = None
         self.logger = None
         self.conversation_start_locked = False #if conversation start locked
-        self.conversations = None
+        self.conversations = {}
         self.conversation_managers = []
         self.extras = {}
 
@@ -66,6 +70,7 @@ class Session(object):
         self.config_dir = e3.common.ConfigDir('emesene2')
         # set the base dir of the config to the base dir plus the account
         self.signals = e3.common.Signals(EVENTS, self.events)
+        self.signals.start()
 
     def get_conversation(self, cid, members=None):
         '''
@@ -144,6 +149,23 @@ class Session(object):
             except:
                 print "Error on save configuration"
 
+    def save_extensions_config(self):
+        '''save the state of the extensions to the config'''
+        if self.config.d_extensions is None:
+            self.config.d_extensions = {}
+
+        for name, category in extension.get_categories().iteritems():
+            self.config.d_extensions[name] = \
+                category.default_id
+
+    def set_default_extensions_from_config(self):
+        '''get the ids of the default extensions stored on config
+        and set them as default on the extensions module'''
+
+        if self.config.d_extensions is not None:
+            for cat_name, ext_id in self.config.d_extensions.iteritems():
+                extension.set_default_by_id(cat_name, ext_id)
+
     def start_mail_client(self):
         pass
 
@@ -272,7 +294,9 @@ class Session(object):
 
     def session_has_service(self, service):
         '''returns True if some service is supported, False otherwise'''
-        raise NotImplementedError('Not implemented')
+        if service not in self.CAPABILITIES:
+            return False
+        return True
 
     def send_typing_notification(self, cid):
         '''send typing notification to contact'''
