@@ -70,7 +70,7 @@ EVENTS = (\
  'contact added you', 'user typing',
  'mail received', 'mail count changed',
  'broken profile', 'close', 'social request',
- 'gui message')
+ 'gui message', 'endpoint added', 'endpoint removed')
 
 ACTIONS = (\
  'login'            , 'logout'           ,
@@ -92,7 +92,7 @@ ACTIONS = (\
  'call cancel', 'call reject',
  'p2p invite'       , 'p2p accept',
  'p2p cancel'       , 'media send', # media send if got Wink and audio clips
- 'send oim')
+ 'send oim', 'disconnect other endpoints', 'disconnect endpoint')
 
 Event.set_constants(EVENTS)
 Action.set_constants(ACTIONS)
@@ -167,6 +167,9 @@ class Worker(threading.Thread):
         dah[Action.ACTION_CALL_ACCEPT] = self._handle_action_call_accept
         dah[Action.ACTION_CALL_REJECT] = self._handle_action_call_reject
         dah[Action.ACTION_CALL_CANCEL] = self._handle_action_call_cancel
+        # papylib specific
+        dah[Action.ACTION_DISCONNECT_OTHER_ENDPOINTS] = self._handle_action_disconnect_other_endpoints
+        dah[Action.ACTION_DISCONNECT_ENDPOINT] = self._handle_action_disconnect_endpoint
 
         self.action_handlers = dah
 
@@ -184,6 +187,20 @@ class Worker(threading.Thread):
                 self.session.error('Error calling action handler', action.id_)
                 traceback.print_exc()
 
+    # common functions
+    def _filedata_to_string(self, filename):
+        '''read a filename and convert it's data to String'''
+        try:
+            f = open(filename, 'rb')
+            avatar = f.read()
+            f.close()
+        except IOError:
+            return None
+
+        if not isinstance(avatar, str):
+            avatar = "".join([chr(b) for b in avatar])
+
+        return avatar
 
     # action handlers (the stubs, copy and complete them on your implementation)
     def _handle_action_add_contact(self, account):
@@ -301,8 +318,12 @@ class Worker(threading.Thread):
         '''handle Action.ACTION_SET_MEDIA
         '''
         contact = self.session.contacts.me
-        self.session.media_change_succeed(message)
-        self.session.contacts.me.media = message
+        if message is not None:
+            self.session.media_change_succeed(message[0] + " - " + message[1])
+            self.session.contacts.me.media = message[0] + " - " + message[1]
+        else:
+            self.session.contacts.me.media = None
+            self.session.media_change_succeed(None)
 
         # log the change
         account = Logger.Account.from_contact(contact)
@@ -386,3 +407,9 @@ class Worker(threading.Thread):
 
     def _handle_action_call_cancel(self, c):
         pass
+
+    def _handle_action_disconnect_other_endpoints(self):
+        pass
+
+    def _handle_action_disconnect_endpoint(self, ep_id):
+        pass
