@@ -63,6 +63,8 @@ class MessageFormatter(object):
         self.offline_incoming = \
             '<i>(offline message)</i><b>%DISPLAYNAME%</b>:%NL%    [%SHORTTIME%] %MESSAGE%%NL%'
         self.information = '<i>%MESSAGE%</i>%NL%'
+        self.history = '<div class="message-history">'\
+            '<b>%TIME% %DISPLAYNAME%</b>: %MESSAGE%%NL%</div>'
 
     def format_message(self, template, message):
         '''format a message from the template, include new line
@@ -82,7 +84,7 @@ class MessageFormatter(object):
         if msg.type is None:
             msg.type = e3.Message.TYPE_MESSAGE
 
-        if not msg.timestamp == None:
+        if not msg.timestamp is None:
             timestamp = calendar.timegm(msg.timestamp.timetuple())
         else:
             timestamp = time.time()
@@ -99,9 +101,11 @@ class MessageFormatter(object):
                 else:
                     template = self.consecutive_outgoing
 
+        if msg.type == e3.Message.TYPE_OLDMSG:
+            template = self.history
+
         if msg.type == e3.Message.TYPE_FLNMSG:
             template = self.offline_incoming
-            timestamp = msg.timestamp
 
         formated_time = time.strftime('%c', time.gmtime(timestamp))
         formated_short_time = time.strftime('%X', time.localtime(timestamp))
