@@ -20,6 +20,7 @@ import sys
 sys.path.append('..')
 import locations
 import notification
+import Collections
 
 import XmlParser
 
@@ -31,3 +32,4 @@ from ConfigDir import ConfigDir
 from RingBuffer import RingBuffer
 from MessageFormatter import MessageFormatter
 from Sounds import SoundPlayer
+from OrderedDict import OrderedDict
