@@ -38,6 +38,7 @@ class Notification():
         Class Constructor
         """
         self.session = session
+        self.session.config.get_or_set('b_mute_notification', False)
         self.session.config.get_or_set('b_notify_contact_online', True)
         self.session.config.get_or_set('b_notify_contact_offline', True)
         self.session.config.get_or_set('b_notify_receive_message', True)
@@ -65,9 +66,20 @@ class Notification():
             self.session.signals.filetransfer_invitation.subscribe(
                 self._on_filetransfer_invitation)
 
+        extension.subscribe(self._on_notification_gui_changed, 'notificationGUI')
+        extension.subscribe(self._on_notification_image_changed, 'notificationImage')
+
         self.notify_online = False
         self.last_online = None
 
+    def _on_notification_gui_changed(self, new_extension):
+        if type(self.notifier) != new_extension:
+            self.notifier = extension.get_default('notificationGUI')
+
+    def _on_notification_image_changed(self, new_extension):
+        if type(self.picture_factory) != new_extension:
+            self.picture_factory = extension.get_default('notificationImage')
+
     def remove_subscriptions(self):
         if self.session:
             self.session.signals.conv_message.unsubscribe(
@@ -84,6 +96,8 @@ class Notification():
                 self._on_filetransfer_canceled)
             self.session.signals.filetransfer_invitation.unsubscribe(
                 self._on_filetransfer_invitation)
+        extension.unsubscribe(self._on_notification_gui_changed, 'notificationGUI')
+        extension.unsubscribe(self._on_notification_image_changed, 'notificationImage')
 
     def _on_filetransfer_completed(self, args):
         uri = self.picture_factory('notification-message-email', 'mail-received')
@@ -186,6 +200,10 @@ class Notification():
         """
         This creates and shows the nofification
         """
+        if self.session.config.b_mute_notification:
+            #don't show notification if user disable all of them
+            return
+
         only_when_available = self.session.config.b_notify_only_when_available
         #Only show notifications when available
         if not only_when_available or (only_when_available and self.session.account.status == status.ONLINE):
