@@ -26,6 +26,8 @@ import papyon.gnet.errors
 import logging
 log = logging.getLogger('papylib.Events')
 
+PROFILE_URL = "http://profile.live.com/details/Edit/Pic"
+
 class ClientEvents(papyon.event.ClientEventInterface):
     def on_client_state_changed(self, state):
         if state == papyon.event.ClientState.CLOSED:
@@ -78,12 +80,18 @@ class ClientEvents(papyon.event.ClientEventInterface):
             if isinstance(error, papyon.gnet.errors.HTTPError) and \
                error.response and error.response.status and \
                error.response.status == 404:
-                self._client.session.add_event(Event.EVENT_BROKEN_PROFILE)
+                self._client.session.add_event(Event.EVENT_BROKEN_PROFILE,
+                                               PROFILE_URL)
                 return
-            if error._fault == "ItemDoesNotExist": # TODO: check it's a ContentRoamingError first
-                self._client.session.add_event(Event.EVENT_BROKEN_PROFILE)
-        elif error_type == papyon.event.ClientErrorType.ADDRESSBOOK:#TODO
+            try:
+                if error._fault == "ItemDoesNotExist":
+                    self._client.session.add_event(Event.EVENT_BROKEN_PROFILE,
+                                                   PROFILE_URL)
+            except AttributeError:
+                log.error("Client got an error: %s %s" % (error_type, error))
+        elif error_type == papyon.event.ClientErrorType.ADDRESSBOOK:
             log.error("Client got an error handling addressbook: %s %s" % (error_type, error))
+            self._client.session.add_event(Event.EVENT_DISCONNECTED, "%s %s" % (error, error_type), 0)
         elif error_type == papyon.event.ClientErrorType.OFFLINE_MESSAGES:#TODO
             log.error("Client got an error handling offline messages: %s %s" % (error_type, error))
         else:
@@ -203,6 +211,12 @@ class ProfileEvent(papyon.event.ProfileEventInterface):
     def on_profile_msn_object_changed(self):
         self._client._on_profile_msn_object_changed()
 
+    def on_profile_end_point_added(self, ep):
+        self._client._on_profile_end_point_added(ep)
+
+    def on_profile_end_point_removed(self, ep):
+        self._client._on_profile_end_point_removed(ep)
+
 class CallEvent(papyon.event.CallEventInterface):
     def __init__(self, call, client):
         papyon.event.CallEventInterface.__init__(self, call)
