@@ -93,6 +93,8 @@ class Worker(e3.base.Worker, papyon.Client):
             self.client = papyon.Client.__init__(self, server,
                 proxies=get_proxies(), version=18)
 
+        self.profile_url = PROFILE_URL
+
         self._event_handler = ClientEvents(self)
         self._contact_handler = ContactEvent(self)
         self._invite_handler = InviteEvent(self)
@@ -128,8 +130,10 @@ class Worker(e3.base.Worker, papyon.Client):
 
                 if action.id_ == Action.ACTION_QUIT:
                     log.debug('closing thread')
-                    self.logout()
+                    if not self.state == papyon.event.ClientState.CLOSED:
+                        self.logout()
                     self.session.logger.quit()
+                    self.session.signals.quit()
                     break
 
                 self._process_action(action)
@@ -154,6 +158,17 @@ class Worker(e3.base.Worker, papyon.Client):
         self.profile.client_capabilities.supports_activities = False
         self.profile.client_capabilities.supports_winks = False
         self.profile.client_capabilities.supports_shared_search = False
+        self.profile.client_capabilities.supports_voice_over_msnp = False
+        self.profile.client_capabilities.supports_uucp_sip_stack = False
+        self.profile.client_capabilities.supports_sharing_video = False
+        self.profile.client_capabilities.circle_voiceim_enabled = False
+        self.profile.client_capabilities.sharing_enabled = False
+        self.profile.client_capabilities.mobile_suspendim_fanout_disable = False
+        self.profile.client_capabilities.supports_videocall_16x9 = False
+        self.profile.client_capabilities.siptunnel_v2 = False
+        self.profile.client_capabilities.supports_wma_voiceclip = False
+        self.profile.client_capabilities.supports_circleim_voiceclip = False
+        self.profile.client_capabilities.supports_socialnews_objecttypes = False
         global PAPY_HAS_AUDIOVIDEO
         self.profile.client_capabilities.supports_voice_im = PAPY_HAS_AUDIOVIDEO
         self.profile.client_capabilities.supports_sip_invite = PAPY_HAS_AUDIOVIDEO
@@ -162,7 +177,6 @@ class Worker(e3.base.Worker, papyon.Client):
         self.profile.client_capabilities.supports_rtc_video = PAPY_HAS_AUDIOVIDEO
         self.profile.client_capabilities.has_webcam = PAPY_HAS_AUDIOVIDEO
 
-        self.profile.end_point_name = "emesene"
         self.profile.privacy = papyon.profile.Privacy.BLOCK
 
         # initialize caches
@@ -236,13 +250,9 @@ class Worker(e3.base.Worker, papyon.Client):
 
     def _add_contact(self, papycontact):
         ''' helper method to add a contact to the (gui) contact list '''
-        constants = papyon.service.description.AB.constants
-        alias = papycontact.infos.get(constants.ContactGeneral.ANNOTATIONS,
-                {}).get(constants.ContactAnnotations.NICKNAME, "")
-        alias = unicode(alias, 'utf-8')
         contact = e3.base.Contact(papycontact.account, papycontact.id,
             papycontact.display_name, papycontact.personal_message,
-            STATUS_PAPY_TO_E3[papycontact.presence], alias,
+            STATUS_PAPY_TO_E3[papycontact.presence], papycontact.alias,
             (papyon.profile.Membership.BLOCK & papycontact.memberships))
 
         self.session.contacts.contacts[papycontact.account] = contact
@@ -302,12 +312,13 @@ class Worker(e3.base.Worker, papyon.Client):
     def _on_conversation_invite(self, papyconversation):
         ''' called when we are invited in a conversation '''
         cid = time.time()
-        partecipants = list(papyconversation.participants)
-        members = [account.account for account in partecipants]
+        participants = list(papyconversation.participants)
+        members = [account.account for account in participants]
 
-        id_multichat = 'GroupChat'+str(cid)
-        self.conversations[id_multichat] = cid
-        self.rconversations[cid] = id_multichat
+        conv_id = members[0] if len(participants) == 1 else 'GroupChat'+str(cid)
+
+        self.conversations[conv_id] = cid
+        self.rconversations[cid] = conv_id
         newconversationevent = ConversationEvent(papyconversation, self)
         self._conversation_handler[cid] = newconversationevent
         self.papyconv[cid] = papyconversation
@@ -378,18 +389,8 @@ class Worker(e3.base.Worker, papyon.Client):
 
     def papy_ft_accepted(self, ftsession):
         tr = self.filetransfers[ftsession]
-        try:
-            f = open(tr.completepath, 'rb')
-            filedata = f.read()
-            f.close()
-        except Exception as e:
-            log.error("Reading of file %s failed: %s" % (tr.completepath, e))
-
-        if not isinstance(filedata, str):
-            filedata = "".join([chr(b) for b in filedata])
-
-        data=StringIO.StringIO(filedata)
-        ftsession.send(data)
+        filedata = self._filedata_to_string(tr.completepath)
+        ftsession.send(StringIO.StringIO(filedata))
 
         self.session.filetransfer_accepted(tr)
 
@@ -505,10 +506,12 @@ class Worker(e3.base.Worker, papyon.Client):
         ''' handle the reception of a message '''
         account = papycontact.account
         conv = pyconvevent.conversation
-
         if conv in self.rpapyconv:
             cid = self.rpapyconv[conv]
         else:
+            raise Exception("CID Error", "Not in rpapyconv - did not receive an invite for this message")
+        '''
+        else: # TODO:FIXME: this never happens (?)
             # emesene must create another conversation
             cid = time.time()
             self.conversations[account] = cid # add to account:cid
@@ -517,7 +520,7 @@ class Worker(e3.base.Worker, papyon.Client):
             self.papyconv[cid] = pyconvevent.conversation # add papy conv
             self.rpapyconv[pyconvevent.conversation] = cid
             self.session.conv_first_action(cid, [account])
-
+        '''
         msgobj = e3.base.Message(e3.base.Message.TYPE_MESSAGE,
                 papymessage.content, account,
                 formatting_papy_to_e3(papymessage.formatting,
@@ -561,7 +564,16 @@ class Worker(e3.base.Worker, papyon.Client):
 
         self.session.conv_message(cid, account, msgobj,
                 received_custom_emoticons)
-        e3.Logger.log_message(self.session, None, msgobj, False, cid = cid)
+
+        #XXX: when we send messages from the web iface we get those here, so log them propertly
+        if msgobj.account == self.session.contacts.me.account:
+            participants = [x.account for x in conv.participants]
+            sent = True
+        else:
+            participants = None
+            sent = False
+
+        e3.Logger.log_message(self.session, participants, msgobj, sent, cid = cid)
 
     def _on_conversation_nudge_received(self, papycontact, pyconvevent):
         ''' handle received nudges '''
@@ -587,6 +599,9 @@ class Worker(e3.base.Worker, papyon.Client):
         e3.Logger.log_message(self.session, None, msgobj, False)
 
     def _on_conversation_message_error(self, err_type, error, convevent):
+        if err_type != papyon.event.ConversationErrorType.MESSAGE:
+            return #discard silently
+
         cid = self.rpapyconv[convevent.conversation]
 
         msgobj = e3.base.Message(e3.base.Message.TYPE_MESSAGE, error,
@@ -897,6 +912,15 @@ class Worker(e3.base.Worker, papyon.Client):
         """form url"""
         return self._form_data '''
 
+    def _on_profile_end_point_added(self, ep):
+        """Called when endpoints change (added)"""
+        if ep.id != self.machine_guid:
+            self.session.endpoint_added(ep.id, ep.name)
+
+    def _on_profile_end_point_removed(self, ep):
+        """Called when endpoints change (removed)"""
+        self.session.endpoint_removed(ep.id)
+
 ################################################################################
 # BELOW THIS LINE, ONLY e3 HANDLERS
 ################################################################################
@@ -1119,16 +1143,32 @@ class Worker(e3.base.Worker, papyon.Client):
             self.address_book.rename_group(papygroup, name,
                     failed_cb=(rename_group_fail,))
 
-    def _handle_action_set_contact_alias(self, account, alias): #TODO: finish
+    def _handle_action_set_contact_alias(self, account, alias):
         ''' handle Action.ACTION_SET_CONTACT_ALIAS '''
+
+        contact = self.session.contacts.contacts.get(account, None)
+        if not contact:
+            return
+
+        account = contact.account
+        old_nick = contact.nick
+
         def set_contact_alias_fail(*args):
             log.error("Error when settings alias: %s" % args)
             # account
             self.session.contact_alias_failed('')
 
-        def set_contact_alias_succeed(*args):
-            log.info("Setting alias ok: %s" % args)
-            self.session.contact_alias_succeed(account)
+        def set_contact_alias_succeed(papycontact, *args):
+            log.info("Setting alias ok: %s" % account)
+
+            if alias != "":
+                contact.alias = papycontact.alias
+                contact.nick = papycontact.alias
+            else:
+                contact.alias = ""
+                contact.nick = papycontact.display_name
+
+            self.session.contact_attr_changed(account, 'nick', old_nick)
 
         papycontact = self.contact_by_account(account)
         new_alias = alias.encode("utf-8")
@@ -1137,27 +1177,9 @@ class Worker(e3.base.Worker, papyon.Client):
         infos = {constants.ContactGeneral.ANNOTATIONS:
                 {constants.ContactAnnotations.NICKNAME: new_alias}}
 
-        self.address_book.update_contact_infos(papycontact, infos)
-        #   TODO: Find out why these don't work
-        #    done_cb=set_contact_alias_succeed,
-        #        failed_cb=set_contact_alias_fail)
-
-        contact = self.session.contacts.contacts.get(account, None)
-
-        if not contact:
-            return
-
-        account = contact.account
-        old_nick = contact.nick
-
-        if alias != "":
-            contact.alias = new_alias
-            contact.nick = new_alias
-        else:
-            contact.alias = ""
-            contact.nick = papycontact.display_name
-
-        self.session.contact_attr_changed(account, 'nick', old_nick)
+        self.address_book.update_contact_infos(papycontact, infos,
+            done_cb=(set_contact_alias_succeed,),
+                failed_cb=set_contact_alias_fail)
 
     def _handle_action_change_status(self, status_):
         '''handle Action.ACTION_CHANGE_STATUS '''
@@ -1166,13 +1188,7 @@ class Worker(e3.base.Worker, papyon.Client):
     def _handle_action_set_media(self, message):
         '''handle Action.ACTION_SET_MEDIA
         '''
-        contact = self.session.contacts.me
-        if message is not None:
-            self.session.media_change_succeed(message[0] + " - " + message[1])
-            self.session.contacts.me.media = message[0] + " - " + message[1]
-        else:
-            self.session.contacts.me.media = None
-            self.session.media_change_succeed(None)
+        e3.base.Worker._handle_action_set_media(self, message)
         self.profile.personal_message_current_media = self.profile.personal_message, message
 
     def _handle_action_set_message(self, message):
@@ -1203,15 +1219,7 @@ class Worker(e3.base.Worker, papyon.Client):
             self.session.picture_change_succeed(self.session.account.account, None)
             return
 
-        try:
-            f = open(picture_name, 'rb')
-            avatar = f.read()
-            f.close()
-        except Exception:
-            log.error("Loading of picture %s failed" % picture_name)
-
-        if not isinstance(avatar, str):
-            avatar = "".join([chr(b) for b in avatar])
+        avatar = self._filedata_to_string(picture_name)
 
         msn_object = papyon.p2p.MSNObject(self.profile, len(avatar),
                 papyon.p2p.MSNObjectType.DISPLAY_PICTURE,
@@ -1249,35 +1257,21 @@ class Worker(e3.base.Worker, papyon.Client):
             oldcid = self.conversations[account]
             self._handle_action_close_conversation(oldcid)
 
-            self.conversations[account] = cid
-            self.rconversations[cid] = account
-            # create a papyon conversation
-            contact = self.contact_by_account(account)
-            conv = papyon.Conversation(self, [contact,])
-            self.papyconv[cid] = conv
-            self.rpapyconv[conv] = cid
-            # attach the conversation event handler
-            convhandler = ConversationEvent(conv, self)
-            self._conversation_handler[cid] = convhandler
-
-        else:
-            #print "creating a new conversation et. al"
-            # new emesene conversation
-            self.conversations[account] = cid
-            self.rconversations[cid] = account
-            contact = self.contact_by_account(account)
-            # create a papyon conversation
-            conv = papyon.Conversation(self, [contact,])
-            self.papyconv[cid] = conv
-            self.rpapyconv[conv] = cid
-            # attach the conversation event handler
-            convhandler = ConversationEvent(conv, self)
-            self._conversation_handler[cid] = convhandler
+        self.conversations[account] = cid
+        self.rconversations[cid] = account
+        # create a papyon conversation
+        contact = self.contact_by_account(account)
+        conv = papyon.Conversation(self, [contact,])
+        self.papyconv[cid] = conv
+        self.rpapyconv[conv] = cid
+        # attach the conversation event handler
+        convhandler = ConversationEvent(conv, self)
+        self._conversation_handler[cid] = convhandler
 
     def _handle_action_close_conversation(self, cid):
         '''handle Action.ACTION_CLOSE_CONVERSATION
         '''
-        #print "you close conversation %s, are you happy?" % cid
+        #print "you close conversation %f, are you happy?" % cid
         account = self.rconversations[cid]
         conv = self.papyconv[cid]
         conv.leave()
@@ -1328,7 +1322,7 @@ class Worker(e3.base.Worker, papyon.Client):
             if first_dude.presence == papyon.Presence.OFFLINE or \
                first_dude.network_id == papyon.profile.NetworkID.EXTERNAL:
                 if switchboard is None or \
-                   switchboard.state == papyon.msnp.ProtocolState.CLOSED:
+                   switchboard.state != papyon.msnp.ProtocolState.OPEN:
 
                     if message.type == e3.base.Message.TYPE_NUDGE or \
                        message.type == e3.base.Message.TYPE_TYPING:
@@ -1355,18 +1349,9 @@ class Worker(e3.base.Worker, papyon.Client):
             if l_custom_emoticons is None: l_custom_emoticons = []
 
             for custom_emoticon in l_custom_emoticons:
-                try:
-                    fpath = os.path.join(emoticon_cache.path,
-                            cedict[custom_emoticon])
-                    f = open(fpath, 'rb')
-                    d_custom_emoticon = f.read()
-                    f.close()
-                except IOError as ioerror:
-                    log.error("Loading of emoticon failed: %s" % ioerror)
 
-                if not isinstance(d_custom_emoticon, str):
-                    d_custom_emoticon = "".join(
-                            [chr(b) for b in d_custom_emoticon])
+                fpath = os.path.join(emoticon_cache.path, cedict[custom_emoticon])
+                d_custom_emoticon = self._filedata_to_string(fpath)
 
                 msn_object = papyon.p2p.MSNObject(self.session.account.account,
                         len(d_custom_emoticon),
@@ -1388,8 +1373,13 @@ class Worker(e3.base.Worker, papyon.Client):
     def _handle_action_ft_invite(self, cid, account, filename, completepath,
             preview_data):
 
+        if not os.path.getsize(completepath):
+            # size is 0 or function returned None, 
+            # don't try to send anything
+            return
+
         papycontact = self.contact_by_account(account)
-        papysession = self._ft_manager.send(papycontact, filename,
+        papysession = self.ft_manager.send(papycontact, filename,
                 os.path.getsize(completepath), preview_data)
 
         tr = e3.base.FileTransfer(papysession, papysession.filename,
@@ -1473,3 +1463,15 @@ class Worker(e3.base.Worker, papyon.Client):
 
         del self.calls[self.rcalls[c]]
         del self.rcalls[c]
+
+    def _handle_action_disconnect_other_endpoints(self):
+        """ disconnects all other msnp18 mpop instances """
+        self.disconnect_other_endpoints()
+
+    def _handle_action_disconnect_endpoint(self, ep_id):
+        """ disconnects a single msnp18 mpop instance """
+        if ep_id == "":
+            self.disconnect_other_endpoints()
+            return
+        self.disconnect_endpoint(ep_id)
+
