@@ -284,7 +284,7 @@ class Client(EventsDispatcher):
             @param password: the password needed to authenticate to the account
             @type password: utf-8 encoded string
             """
-        if (self._state != ClientState.CLOSED):
+        if self._state != ClientState.CLOSED:
             logger.warning('login already in progress')
             return
         self.__die = False
@@ -305,12 +305,56 @@ class Client(EventsDispatcher):
         self._protocol.signoff()
         self.__state = ClientState.CLOSED
 
+    def disconnect_other_endpoints(self):
+        """ Disconnects other endpoints """
+        if self._state == ClientState.CLOSED:
+            return
+        self._protocol.send_user_notification("goawyplzthxbye-nomorempop",
+           self.profile.account, "", 4)
+
+    def disconnect_endpoint(self, ep_id):
+        """ Disconnects single endpoint """
+        if self._state == ClientState.CLOSED:
+            return
+        self._protocol.send_user_notification("goawyplzthxbye",
+           self.profile.account, ep_id, 4)
+
     ### Protected API --------------------------------------------------------
+    @property
+    def client_ip(self):
+        if not self._profile:
+            return ""
+        return self._profile.profile.get("ClientIP")
+
+    @property
+    def client_port(self):
+        if not self._profile:
+            return 0
+        return self._profile.profile.get("ClientPort")
 
     @property
     def local_ip(self):
         return self._transport.sockname[0]
 
+    @property
+    def local_port(self):
+        return self._transport.sockname[1]
+
+    @property
+    def conn_type(self):
+        if not self.local_ip or not self.client_ip:
+            return "Unkown-Connect"
+        if self.local_ip == self.client_ip:
+            if self.local_port == self.client_port:
+                return "Direct-Connect"
+            else:
+                return "Port-Restrict-NAT"
+        else:
+            if self.local_port == self.client_port:
+                return "IP-Restrict-NAT"
+            else:
+                return "Symmetric-NAT"
+
     @property
     def protocol_version(self):
         return self._protocol._protocol_version
