@@ -31,7 +31,10 @@ import papyon.msnp as msnp
 from papyon.profile import Presence
 from papyon.transport import ServerType
 from papyon.util.async import run
-from papyon.util.weak import WeakSet
+try:
+    from weakref import WeakSet
+except ImportError:
+    from papyon.util.weak import WeakSet
 from papyon.event import ConversationErrorType, ContactInviteError, MessageError
 
 __all__ = ['SwitchboardManager']
@@ -209,7 +212,10 @@ class SwitchboardHandler(object):
         callback, errback = self._delivery_callbacks.pop(trid, (None, None))
         error = MessageError.DELIVERY_FAILED
         run(errback, error)
-        self._on_error(ConversationErrorType.MESSAGE, error)
+        if hasattr(self, "max_chunk_size"):
+            self._on_error(ConversationErrorType.P2P, error)
+        else:
+            self._on_error(ConversationErrorType.MESSAGE, error)
 
     # Helper functions
     def _process_pending_queues(self):
