@@ -21,7 +21,6 @@
 import os
 import sys
 
-
 # Extract any non-GStreamer arguments, and leave the GStreamer arguments for
 # processing by GStreamer. This needs to be done before GStreamer is imported,
 # so that GStreamer doesn't hijack e.g. ``--help``.
@@ -56,52 +55,32 @@ def project_path():
 
 os.chdir(os.path.abspath(project_path()))
 
+from Language import Language
+import extension
 
-# load translations
-import gettext
-import locale
-default_locale = locale.getdefaultlocale()[0]
-lang = os.getenv('LANGUAGE') or default_locale
-translation = gettext.NullTranslations()
-locales_path = 'po/' if os.path.exists('po/') else None
+extension.category_register('language', Language, 
+                            single_instance=True)
 
-if os.path.exists('default.mo'):
-    translation = gettext.GNUTranslations(open('default.mo'))
-else:
+language_management = extension.get_and_instantiate('language')
+language_management.install_default_translation()
+
+if 'USE_GI' in os.environ:
     try:
-        #try with LANGUAGE variable
-        translation = gettext.translation('emesene', localedir=locales_path, \
-                                          languages=[lang])
-    except IOError:
-        try:
-            #try with Python's locale module
-            translation = gettext.translation('emesene', \
-                                              localedir=locales_path, \
-                                              languages=[default_locale])
-        except IOError:
-            #let gettext handle the language
-            try:
-                translation = gettext.translation('emesene', \
-                                              localedir=locales_path)
-            except IOError:
-                pass
-    except AttributeError:
+        import pygicompat
+    except ImportError:
+        print "error: pygi compat not found"
         pass
 
-translation.install()
-
-
 import glib
-import optparse
 import shutil
 import signal
+import Info
 
 import debugger
 import logging
 log = logging.getLogger('emesene')
 
 import e3
-#from e3 import msn
 from e3 import dummy
 
 try:
@@ -110,9 +89,13 @@ except ImportError:
     DBusController = None
 
 try:
-    from e3.common.NetworkManagerHelper import DBusNetworkChecker
+    from e3.common.NetworkManagerHelper import DBusNetworkChecker as NetworkChecker
 except ImportError:
-    DBusNetworkChecker = None
+    #try windows network check
+    try:
+        from e3.common.NetworkManagerHelperWin32 import Win32NetworkChecker as NetworkChecker
+    except ImportError:
+        NetworkChecker = None
 
 try:
     from gui import gtkui
@@ -120,10 +103,10 @@ except ImportError, exc:
     print 'Cannot import gtkui: %s' % str(exc)
 
 try:
-    from e3 import jabber
+    from e3 import xmpp
 except ImportError, exc:
-    jabber = None
-    print 'Errors occurred while importing jabber backend: %s' % str(exc)
+    xmpp = None
+    print 'Errors occurred while importing xmpp backend: %s' % str(exc)
 
 try:
     from gui import qt4ui
@@ -136,8 +119,7 @@ except ImportError, exc:
     papylib = None
     print 'Errors occurred while importing msn backend: %s' % str(exc)
 
-from pluginmanager import get_pluginmanager
-import extension
+from e3.common.pluginmanager import get_pluginmanager
 import interfaces
 import gui
 import optionprovider
@@ -164,36 +146,35 @@ class Controller(object):
 
         self.session = None
         self.logged_in = False
-        self.timeout_id = None
         self.cur_service = None
         self.notification = None
         self.conv_manager_available = False
         self.last_session_account = None
         self.last_session_service = None
 
+        lang = self.config.get_or_set("language_config", None)
+        language_management.install_desired_translation(lang)
+
         self._parse_commandline()
         self._setup()
 
         if hasattr(signal, 'SIGINT'):
             signal.signal(signal.SIGINT,
-                lambda * args: glib.idle_add(self.close_session))
+                lambda * args: glib.idle_add(self.kill))
         if hasattr(signal, 'SIGTERM'):
             signal.signal(signal.SIGTERM,
-                lambda * args: glib.idle_add(self.close_session))
+                lambda * args: glib.idle_add(self.kill))
         if hasattr(signal, 'SIGHUP'):
             signal.signal(signal.SIGHUP,
-                lambda * args: glib.idle_add(self.close_session))
+                lambda * args: glib.idle_add(self.kill))
 
     def _setup(self):
         '''register core extensions'''
         extension.category_register('session', dummy.Session,
                 single_instance=True)
-        #extension.category_register('session', msn.Session,
-        #        single_instance=True)
-        if jabber is not None:
-            extension.register('session', jabber.Session)
+        if xmpp is not None:
+            extension.register('session', xmpp.Session)
         extension.register('session', dummy.Session)
-        #extension.register('session', msn.Session)
 
         if papylib is not None:
             extension.register('session', papylib.Session)
@@ -209,9 +190,9 @@ class Controller(object):
         else:
             self.dbus_ext = None
 
-        if DBusNetworkChecker is not None:
-            extension.category_register('network checker', DBusNetworkChecker)
-            extension.set_default('network checker', DBusNetworkChecker)
+        if NetworkChecker is not None:
+            extension.category_register('network checker', NetworkChecker)
+            extension.set_default('network checker', NetworkChecker)
             self.network_checker = extension.get_and_instantiate(
                     'network checker')
         else:
@@ -225,6 +206,8 @@ class Controller(object):
                 e3.common.notification.Notification)
         extension.category_register('history exporter',
                 e3.Logger.save_logs_as_txt)
+        extension.register('history exporter',
+                e3.Logger.save_logs_as_xml)
 
         if self.config.session is None:
             default_id = extension.get_category('session').default_id
@@ -236,10 +219,16 @@ class Controller(object):
 
     def _parse_commandline(self):
         '''parse command line options'''
-        options = PluggableOptionParser.get_parsing()[0]
+        options = optionprovider.PluggableOptionParser.get_parsing()[0]
 
         debugger.init(debuglevel=options.debuglevel)
 
+        if options.version:
+            print "Current Emesene Version: " + Info.EMESENE_VERSION
+            print "Last Stable Version: " + Info.EMESENE_LAST_STABLE
+            print "\n" + Info.EMESENE_WEBSITE
+            sys.exit(0)
+
         #needed to check for autologin
         self.emesene_is_running = False
         try:
@@ -255,7 +244,7 @@ class Controller(object):
                 if options.single_instance:
                     print "Another instance of emesene is already running."
                     self.single_instance.show()
-                    sys.exit(0)
+                    extension.get_and_instantiate('quit')
         except ImportError:
             pass
 
@@ -280,24 +269,22 @@ class Controller(object):
             self.window.iconify()
         self.window.show()
 
-    def go_login(self, proxy=None, use_http=None, cancel_clicked=False,
-            no_autologin=False):
+    def go_login(self, proxy=None, use_http=None,
+                 cancel_clicked=False, no_autologin=False):
         '''shows the login GUI'''
         if proxy is None:
             proxy = self._get_proxy_settings()
         if use_http is None:
             use_http = self.config.get_or_set('b_use_http', False)
 
-        if self.window.content_type != 'empty':
-            self.window.clear()
-
         self._save_login_dimensions()
         self._set_location(self.window)
 
         self.window.go_login(self.on_login_connect,
-            self.on_preferences_changed, self.config,
-            self.config_dir, self.config_path, proxy,
-            use_http, self.config.session, cancel_clicked, no_autologin)
+                             self.on_preferences_changed, self.config,
+                             self.config_dir, self.config_path, proxy,
+                             use_http, self.config.session,
+                             cancel_clicked, no_autologin)
         self.tray_icon.set_login()
 
     def _new_session(self, account=None):
@@ -317,7 +304,6 @@ class Controller(object):
         signals.conv_first_action.subscribe(self.on_new_conversation)
         signals.disconnected.subscribe(self.on_disconnected)
         signals.picture_change_succeed.subscribe(self.on_picture_change_succeed)
-        signals.contact_added_you.subscribe(self.on_pending_contacts)
 
         #let's start dbus and unity launcher
         if self.dbus_ext is not None:
@@ -338,14 +324,20 @@ class Controller(object):
                     self.session.conversation_managers.append(conv_manager)
             else:
                 for conv_manager in self.conversations:
-                    conv_manager.hide_all()
-                    # _on_conversation_window_close, without saving settings
                     conv_manager.close_all()
                     self.conversations.remove(conv_manager)
 
         self.last_session_account = account.account
         self.last_session_service = account.service
 
+    def kill(self):
+        '''method that tries to kill emesene in a friendly way'''
+        try:
+            self.close_session()
+        except:
+            log.exception("Error while shutting down")
+            sys.exit(1)
+
     def close_session(self, do_exit=True, server_disconnected=False):
         '''close session'''
         # prevent preference window from staying open and breaking things
@@ -353,6 +345,9 @@ class Controller(object):
         if pref:
             pref.hide()
 
+        # close all dialogs that are open
+        extension.get_default('dialog').close_all()
+
         self._remove_subscriptions()
 
         if server_disconnected:
@@ -361,23 +356,21 @@ class Controller(object):
             self.conv_manager_available = True # update with new session
         else:
             for conv_manager in self.conversations:
-                conv_manager.hide_all()
-                self._on_conversation_window_close(conv_manager)
+                conv_manager.close_all()
 
-        if self.timeout_id:
-            glib.source_remove(self.timeout_id)
-            self.timeout_id = None
-
-        if self.session is not None:
+        if self.session:
             self.session.stop_mail_client()
             self.session.quit()
 
         self.window.on_disconnect(self.close_session)
 
-        self.save_extensions_config()
+        self._save_application_language()
+        if self.session is not None:
+            self.session.save_extensions_config()
+
         self._save_login_dimensions()
 
-        if self.session is not None and self.logged_in:
+        if self.session and self.logged_in:
             self.session.save_config()
             self.session = None
             self.logged_in = False
@@ -387,10 +380,8 @@ class Controller(object):
         if do_exit:
             if self.tray_icon is not None:
                 self.tray_icon.set_visible(False)
-            self.window.hide()
-            self.window = None
 
-            sys.exit(0)
+            extension.get_and_instantiate('quit')
 
     def _remove_subscriptions(self):
         '''remove the subscriptions to signals'''
@@ -404,34 +395,23 @@ class Controller(object):
             signals.disconnected.unsubscribe(self.on_disconnected)
             signals.picture_change_succeed.unsubscribe(
                     self.on_picture_change_succeed)
-            signals.contact_added_you.unsubscribe(self.on_pending_contacts)
             if self.unity_launcher is not None:
                 self.unity_launcher.remove_session()
             # unsubscribe notifications signals
             if self.notification:
                 self.notification.remove_subscriptions()
                 self.notification = None
+            extension.unsubscribe(self._on_tray_icon_changed, 'tray icon')
 
-    def save_extensions_config(self):
-        '''save the state of the extensions to the config'''
+    def _save_application_language(self):
+        '''save global settings to application config
+           obtained from session settings.
+        '''
         if self.session is None:
             return
-
-        if self.session.config.d_extensions is None:
-            self.session.config.d_extensions = {}
-
-        for name, category in extension.get_categories().iteritems():
-            self.session.config.d_extensions[name] = \
-                category.default_id
-
-    def set_default_extensions_from_config(self):
-        '''get the ids of the default extensions stored on config
-        and set them as default on the extensions module'''
-
-        if self.session.config.d_extensions is not None:
-            for cat_name, ext_id in self.session.config\
-                    .d_extensions.iteritems():
-                extension.set_default_by_id(cat_name, ext_id)
+        if self.config is None:
+            return
+        self.config.language_config = self.session.config.language_config
 
     def _get_proxy_settings(self):
         '''return the values of the proxy settings as a e3.Proxy object
@@ -463,14 +443,6 @@ class Controller(object):
         '''
         width, height, posx, posy = self.window.get_dimensions()
 
-        # when login window is minimized, posx and posy are -32000 on Windows
-        if os.name == "nt":
-            # make sure that the saved dimensions are visible
-            if posx < (-width):
-                posx = 0
-            if posy < (-height):
-                posy = 0
-
         if not self.window.is_maximized():
             self.config.i_login_posx = posx
             self.config.i_login_posy = posy
@@ -483,16 +455,9 @@ class Controller(object):
     def draw_main_screen(self):
         '''create and populate the main screen
         '''
-        self.window.clear()
-
         last_avatar_path = self.session.config_dir.get_path("last_avatar")
         self.session.load_config()
 
-        if 'msn' in self.session.SERVICES:
-            # keepalive conversations...or not
-            b_keepalive = self.session.config.get_or_set("b_papylib_keepalive", False)
-            self.session.get_worker().keepalive_conversations = b_keepalive
-
         image_name = self.session.config.get_or_set('image_theme', 'default')
         emote_name = self.session.config.get_or_set('emote_theme', 'default')
         sound_name = self.session.config.get_or_set('sound_theme', 'default')
@@ -505,9 +470,10 @@ class Controller(object):
         self.session.config.get_or_set('last_avatar', last_avatar_path)
 
         self.config.save(self.config_path)
-        self.set_default_extensions_from_config()
+        self.session.set_default_extensions_from_config()
 
         self._draw_tray_icon()
+        self.tray_icon.unsubscribe()
         self.tray_icon.set_main(self.session)
 
         self.window.go_main(self.session, self.on_new_conversation,
@@ -524,6 +490,12 @@ class Controller(object):
 
         handler = gui.base.TrayIconHandler(self.session, gui.theme, self.close_session)
         self.tray_icon = trayiconcls(handler, self.window)
+        extension.subscribe(self._on_tray_icon_changed, 'tray icon')
+
+    def _on_tray_icon_changed(self, new_extension):
+        self._draw_tray_icon()
+        self.tray_icon.unsubscribe()
+        self.tray_icon.set_main(self.session)
 
     def _sync_emesene1(self):
         syn = extension.get_default('synch tool')
@@ -535,7 +507,7 @@ class Controller(object):
             syn = syn(self.session, current_service)
             syn.show()
 
-    def _set_location(self, window, is_conv=False):
+    def _set_location(self, window, is_conv=False, single_window=False):
         '''get and set the location of the window'''
         if is_conv:
             posx = self.session.config.get_or_set('i_conv_posx', 100)
@@ -551,16 +523,10 @@ class Controller(object):
             height = self.config.get_or_set('i_login_height', 410)
             maximized = self.config.get_or_set('b_login_maximized', False)
 
-        screen = window.get_screen()
-        pwidth, pheight = screen.get_width(), screen.get_height()
-        if posx > pwidth:
-            posx = (pwidth - width) // 2
-        if posy > pheight:
-            posy = (pheight - height) // 2
         if maximized:
             window.maximize()
 
-        window.set_location(width, height, posx, posy)
+        window.set_location(width, height, posx, posy, single_window and is_conv)
 
     def on_preferences_changed(self, use_http, proxy, session_id, service):
         '''called when the preferences on login change'''
@@ -576,8 +542,8 @@ class Controller(object):
         self._remove_subscriptions()
         self._new_session()
         self.go_login(cancel_clicked=True)
-        self.window.content.clear_all()
-        self.window.content.show_error(reason)
+        self.window.content_main.clear_all()
+        self.window.content_main.show_error(reason, login_failed=True)
 
     def _setup_plugins(self):
         plugin_manager = get_pluginmanager()
@@ -591,8 +557,7 @@ class Controller(object):
 
         for plugin in self.session.config.l_active_plugins:
             plugin_manager.plugin_start(plugin, self.session)
-        self.set_default_extensions_from_config()
-        self.window.content.replace_extensions()
+        self.session.set_default_extensions_from_config()
 
     def on_login_succeed(self):
         '''callback called on login succeed'''
@@ -621,17 +586,16 @@ class Controller(object):
         if not on_reconnect:
             self.on_preferences_changed(use_http, proxy, session_id,
                     self.config.service)
-            self.window.clear()
             self.avatar_path = self.config_dir.join(host, account.account,
                     'avatars', 'last')
             self.window.go_connect(self.on_cancel_login, self.avatar_path,
                     self.config)
             self.window.show()
         else:
-            self.window.content.clear_connect()
+            self.window.content_main.clear_connect()
 
         self._new_session(account)
-        
+
         # set default values if not already set
         self.session.config.get_or_set('b_conv_minimized', True)
         self.session.config.get_or_set('b_conv_maximized', False)
@@ -648,20 +612,21 @@ class Controller(object):
         self.session.config.get_or_set('b_notify_receive_message', True)
         self.session.config.get_or_set('b_notify_only_when_available', True)
         self.session.config.get_or_set('b_show_userpanel', True)
+        self.session.config.get_or_set('b_show_mail_inbox', True)
         self.session.config.get_or_set('b_show_emoticons', True)
         self.session.config.get_or_set('b_show_header', True)
         self.session.config.get_or_set('b_show_info', True)
         self.session.config.get_or_set('b_show_toolbar', True)
         self.session.config.get_or_set('b_allow_auto_scroll', True)
-        self.session.config.get_or_set('adium_theme',
-                'renkoo.AdiumMessageStyle')
+        self.session.config.get_or_set('adium_theme', 'renkoo.AdiumMessageStyle')
         self.session.config.get_or_set('b_enable_spell_check', False)
         self.session.config.get_or_set('b_download_folder_per_account', False)
         self.session.config.get_or_set('b_override_text_color', False)
         self.session.config.get_or_set('override_text_color', '#000000')
+        self.session.config.get_or_set('b_conversation_tabs', True)
+        self.session.config.get_or_set('b_single_window', True)
+        self.session.config.get_or_set('b_open_mail_in_desktop', False)
 
-        self.timeout_id = glib.timeout_add(500,
-                self.session.signals._handle_events)
         self.session.login(account.account, account.password, account.status,
             proxy, host, port, use_http)
 
@@ -673,34 +638,8 @@ class Controller(object):
             self.session.quit()
         self.go_login(cancel_clicked=True, no_autologin=True)
 
-    def on_pending_contacts(self):
-        '''callback called when some contact is pending'''
-        def on_contact_added_you(responses):
-            '''
-            callback called when the dialog is closed
-            '''
-            for account in responses['accepted']:
-                self.session.add_contact(account)
-
-            for account in responses['rejected']:
-                self.session.reject_contact(account)
-
-        if self.session.contacts.pending:
-            accounts = []
-            for contact in self.session.contacts.pending.values():
-                accounts.append((contact.account, contact.display_name))
-
-            dialog = extension.get_default('dialog')
-            dialog.contact_added_you(accounts, on_contact_added_you)
-
     def on_contact_list_ready(self):
         '''callback called when the contact list is ready to be used'''
-        self.window.content.contact_list.fill()
-
-        self.on_pending_contacts()
-
-        glib.timeout_add(500, self.session.logger.check)
-
         notificationcls = extension.get_default('notification')
         self.notification = notificationcls(self.session)
         self.soundPlayer = extension.get_and_instantiate('sound', self.session)
@@ -708,8 +647,8 @@ class Controller(object):
     def on_new_conversation(self, cid, members, other_started=True):
         '''callback called when the other user does an action that justify
         opening a conversation'''
-        conversation_tabs = self.session.config.get_or_set(
-                'b_conversation_tabs', True)
+        conv_tabs = self.session.config.b_conversation_tabs
+        sing_wind = self.session.config.b_single_window
 
         conv_manager = None
 
@@ -720,37 +659,35 @@ class Controller(object):
                 break
 
         if conv_manager is None:
-            if not self.conversations or not conversation_tabs:
+            if not self.conversations or not conv_tabs:
+                if sing_wind and conv_tabs:
+                    window = self.window
+                else:
+                    windowcls = extension.get_default('window frame')
+                    window = windowcls(self._on_conversation_window_close)
 
-                windowcls = extension.get_default('window frame')
-                window = windowcls(self._on_conversation_window_close)
-
-                window.go_conversation(self.session)
-                self._set_location(window, True)
-                conv_manager = window.content
+                window.go_conversation(self.session, self._on_conversation_window_close)
+                self._set_location(window, True, sing_wind)
+                conv_manager = window.content_conv
                 self.conversations.append(conv_manager)
                 self.session.conversation_managers.append(conv_manager)
 
-                if self.session.config.b_conv_minimized and other_started:
-                    window.iconify()
-                    window.show()
-                    window.iconify()
-                else:
-                    window.show()
-
+                if not (sing_wind and conv_tabs):
+                    if self.session.config.b_conv_minimized and other_started:
+                        window.iconify()
+                        window.show()
+                        window.iconify()
+                    else:
+                        window.show()
             else:
                 conv_manager = self.conversations[0]
 
         conversation = conv_manager.new_conversation(cid, members)
 
-        conversation.update_data()
-        conversation.show(other_started) # puts widget visible
-
         # raises the container and grabs the focus
         # handles cases where window is minimized and ctrl+tab focus stealing
         if not other_started:
-            conv_manager.present(conversation)
-
+            conv_manager.present(conversation, sing_wind and conv_tabs)
 
         if self.session.config.b_play_first_send and not \
            self.session.config.b_play_type:
@@ -759,16 +696,7 @@ class Controller(object):
     def _on_conversation_window_close(self, conv_manager):
         '''method called when the conversation window is closed'''
         if self.session:
-            width, height, posx, posy = \
-                    conv_manager.get_dimensions()
-
-            # when window is minimized, posx and posy are -32000 on Windows
-            if os.name == "nt":
-                # make sure that the saved dimensions are visible
-                if posx < (-width):
-                    posx = 0
-                if posy < (-height):
-                    posy = 0
+            width, height, posx, posy = conv_manager.get_dimensions()
 
             if not conv_manager.is_maximized():
                 self.session.config.i_conv_width = width
@@ -778,9 +706,9 @@ class Controller(object):
                 self.session.config.b_conv_maximized = False
             else:
                 self.session.config.b_conv_maximized = True
+                self.session.config.i_conv_width = width #needed for single window
             self.session.conversation_managers.remove(conv_manager)
 
-        conv_manager.close_all()
         self.conversations.remove(conv_manager)
 
     def on_close(self, close=False):
@@ -798,18 +726,17 @@ class Controller(object):
         else:
             self.go_login(cancel_clicked=True, no_autologin=True)
             if reason is not None:
-                self.window.content.clear_all()
-                self.window.content.show_error(reason)
+                self.window.content_main.clear_all()
+                self.window.content_main.show_error(reason)
 
     def on_reconnect(self, account):
         '''makes the reconnect after 30 seconds'''
-        self.window.clear()
         self.window.go_connect(self.on_cancel_login, self.avatar_path,
                 self.config)
 
         proxy = self._get_proxy_settings()
         use_http = self.config.get_or_set('b_use_http', False)
-        self.window.content.on_reconnect(self.on_login_connect, account, \
+        self.window.content_main.on_reconnect(self.on_login_connect, account, \
                                          self.config.session, proxy, use_http, \
                                          self.cur_service)
 
@@ -825,27 +752,6 @@ class Controller(object):
                 except OSError, e:
                     log.warning("Cannot remove file: %s" % e)
 
-class PluggableOptionParser(object):
-
-    results = ()
-
-    def __init__(self, args):
-        self.parser = optparse.OptionParser(conflict_handler="resolve")
-        self.args = args
-        custom_options = extension.get_category('option provider').use()()\
-                .option_register().get_result()
-        for opt in custom_options.values():
-            self.parser.add_option(opt)
-
-    def read_options(self):
-        if not self.__class__.results:
-            self.__class__.results = self.parser.parse_args(self.args)
-        return self.__class__.results
-
-    @classmethod
-    def get_parsing(cls):
-        return cls.results
-
 def main():
     """
     the main method of emesene
@@ -853,9 +759,10 @@ def main():
     extension.category_register('session', dummy.Session, single_instance=True)
     extension.category_register('option provider', None,
             interfaces=interfaces.IOptionProvider)
+    extension.register('quit', sys.exit)
     extension.get_category('option provider').multi_extension = True
     extension.get_category('option provider').activate(optionprovider.ExtensionDefault)
-    options = PluggableOptionParser(args=emesene_args)
+    options = optionprovider.PluggableOptionParser(args=emesene_args)
     options.read_options()
     main_method = extension.get_default('main')
     main_method(Controller)
