@@ -71,8 +71,10 @@ import os
 import sys
 import weakref
 import logging
-log = logging.getLogger('extension')
 
+from NotificationObject import NotificationObject
+
+log = logging.getLogger('extension')
 
 class MultipleObjects(object):
     '''
@@ -204,6 +206,24 @@ class Category(object):
         self.ids[cls] = class_name
         return True
 
+    def unregister(self, cls):
+        '''This will "remove" a class to the possible extension.
+        @param cls: A Class, NOT an instance
+        '''
+        class_name = _get_class_name(cls)
+        try:
+            del self.classes[class_name]
+            del self.ids[cls]
+        except KeyError:
+            return False
+
+        _id = _get_class_name(cls)
+        if self.default_id == _id:
+            self.default_id = None
+            self.instance = None
+
+        return True
+
     def activate(self, cls):
         '''This will make an extension "active", that means you can use it
         for your multi-extension'''
@@ -254,7 +274,7 @@ class Category(object):
             if not self.get_extensions():
                 return None
             self.default = self.get_extensions().values()[0]
-            log.warning('Choosing a default extension for %s RANDOMLY! --> %s'\
+            log.info('Choosing a default extension for %s RANDOMLY! --> %s'\
                     % (self.name, self.default))
 
         return self.classes[self.default_id]
@@ -356,73 +376,225 @@ class Category(object):
         else:
             return False
 
+class ExtensionManager(NotificationObject):
+    def __init__(self):
+        NotificationObject.__init__(self)
+        self._categories = {} #'CategoryName': Category('ClassName')
 
-_categories = {} #'CategoryName': Category('ClassName')
+    def category_register(self, category, system_default, interfaces=(), \
+            single_instance=False):
+        '''Add a category'''
+        try:
+            iter(interfaces)
+        except TypeError:
+            interfaces = (interfaces,)
+        if category not in self._categories: #doesn't exist
+            self._categories[category] = Category(category, system_default, \
+                    interfaces, single_instance)
+        else: #already exist
+            self._categories[category].set_interface(interfaces)
 
+    def register(self, category_name, cls, force_default=False):
+        '''Register cls as an Extension for category.
+        If the class doesn't agree to the required interfaces, raises ValueError.
+        If the category doesn't exist, return False
+        If exists register the cls and return True
+        '''
+        category = self.get_category(category_name)
+        if category is None: #doesn't exist
+            self.category_register(category_name, cls)
+            category = self._categories[category_name]
+            ret = False
+        else: #already exists
+            ret = category.register(cls)
+
+        if force_default and len(category.get_extensions()) == 1:
+            self.set_default(category_name, cls)
+
+        return False
+
+    def unregister(self, category_name, cls):
+        '''Unregister cls as an Extension for category.
+        If the category doesn't exist, return True
+        If exists unregister the cls and return True
+        '''
+        category = self.get_category(category_name)
+        if category is None: #doesn't exist
+            return True
+        #FIXME: remove category if no extensions remain?
+        ret = category.unregister(cls)
+        self.notify_change(category_name, self.get_default(category_name))
+        return ret
+
+    def get_category(self, category_name):
+        '''Get a Category object, return the category if exists, None otherwise'''
+        return self._categories.get(category_name, None)
+
+    def get_categories(self):
+        '''return a dict with all the categories'''
+        return self._categories
+
+    def get_multiextension_categories(self):
+        ''' get available categories'''
+        categories_blacklisted = ['option provider', 'session', 'external api', 'quit', 'main']
+        categories = [ctg for ctg in self.get_categories().keys()
+                        if len(self.get_extensions(ctg)) > 1 and not ctg in categories_blacklisted]
+        categories.sort()
+        return categories
+
+    def get_extensions(self, category_name):
+        '''return a dict of the available extensions id:class'''
+        category = self.get_category(category_name)
+        if category is not None:
+            return category.get_extensions()
+
+        return None
+
+    def get_default(self, category_name):
+        '''This will return a "default" extension if the category is registered
+        if not, return None'''
+        category = self.get_category(category_name)
+        if category is not None:
+            return category.default
+
+        return None
+
+    def get_instance(self, category_name):
+        '''
+        If the category is a "single interface" one, and we have an instance,
+        return it.
+        Otherwise None
+        '''
+        category = self.get_category(category_name)
+
+        if category is not None:
+            return category.get_instance()
+
+        return None
+
+
+    def get_and_instantiate(self, category_name, *args, **kwargs):
+        '''
+        Get an instance of the default extension.
+        If this category is a "single interface" one, it will also save
+        a reference to that instance.
+        If this method is called when a reference is already saved, it will
+        return that one, NOT a new one.
+        '''
+        category = self.get_category(category_name)
+
+        if category is not None:
+            return category.get_and_instantiate(*args, **kwargs)
+
+        return None
+
+    def delete_instance(self, category_name):
+        '''
+        Delete an instance of a "single interface" category.
+        '''
+        category = self.get_category(category_name)
+
+        category.instance = None
+
+    def set_default(self, category_name, cls):
+        '''set the cls as default for the category category_name, if cls is not
+        on the list of registered extensions, then if will be registered,
+        if the category exists and the extension is registered return True,
+        if the category doesn't exists, return False'''
+        category = self.get_category(category_name)
+        if category is not None:
+            category.default = cls
+            self.notify_change(category_name, cls)
+            return True
+
+        return False
+
+    def set_default_by_id(self, category_name, id_):
+        '''set the default extension of a category through its id (generated
+        by _get_class_name method), if the id is not available it will raise
+        ValueError
+        if the category exists and the extension is registered return True,
+        if the category doesn't exists, return False'''
+        category = self.get_category(category_name)
+        if category is not None:
+            category.set_default_by_id(id_)
+            new_ext = self.get_default(category_name)
+            self.notify_change(category_name, new_ext)
+            return True
+
+        return False
+
+    def get_system_default(self, category_name):
+        '''return the default category registered by core, it can be used as
+        fallback if the default extension on the category raises
+        an Exception when instantiated, if the category is not registerd
+        return None'''
+        category = self.get_category(category_name)
+        if category is not None:
+            return category.system_default
+
+        return None
+
+    def implements(self, *categories):
+        '''decorator to nicely show which interfaces we are implementing'''
+
+        def _impl(ext):
+            '''decorating function'''
+            ext.implements = categories
+            for ctg in categories:
+                if not self.get_category(ctg):
+                    self.category_register(ctg, None)
+                self.get_category(ctg).register(ext)
+            return ext
+
+        return _impl
+
+#XXX: singleton
+extension_manager = ExtensionManager()
 
 def category_register(category, system_default, interfaces=(), \
         single_instance=False):
     '''Add a category'''
-    try:
-        iter(interfaces)
-    except TypeError:
-        interfaces = (interfaces,)
-    if category not in _categories: #doesn't exist
-        _categories[category] = Category(category, system_default, \
-                interfaces, single_instance)
-    else: #already exist
-        _categories[category].set_interface(interfaces)
+    extension_manager.category_register(category, system_default, interfaces, \
+        single_instance)
 
-
-def register(category_name, cls):
+def register(category_name, cls, force_default=False):
     '''Register cls as an Extension for category.
     If the class doesn't agree to the required interfaces, raises ValueError.
     If the category doesn't exist, return False
     If exists register the cls and return True
     '''
-    category = get_category(category_name)
-    if category is None: #doesn't exist
-        return category_register(category_name, cls)
-    else: #already exists
-        return category.register(cls)
-
-    return False
+    return extension_manager.register(category_name, cls, force_default)
 
+def unregister(category_name, cls):
+    '''Unregister cls as an Extension for category.
+    If the category doesn't exist, return True
+    If exists unregister the cls and return True
+    '''
+    extension_manager.unregister(category_name, cls)
 
 def get_category(category_name):
     '''Get a Category object, return the category if exists, None otherwise'''
-    return _categories.get(category_name, None)
+    return extension_manager.get_category(category_name)
 
 
 def get_categories():
     '''return a dict with all the categories'''
-    return _categories
+    return extension_manager.get_categories()
 
 def get_multiextension_categories():
     ''' get available categories'''
-    categories_blacklisted = ['option provider', 'session']
-    categories = [ctg for ctg in get_categories().keys()
-                    if len(get_extensions(ctg)) > 1 and not ctg in categories_blacklisted]
-    categories.sort()
-    return categories
+    return extension_manager.get_multiextension_categories()
 
 def get_extensions(category_name):
     '''return a dict of the available extensions id:class'''
-    category = get_category(category_name)
-    if category is not None:
-        return category.get_extensions()
-
-    return None
+    return extension_manager.get_extensions(category_name)
 
 
 def get_default(category_name):
     '''This will return a "default" extension if the category is registered
     if not, return None'''
-    category = get_category(category_name)
-    if category is not None:
-        return category.default
-
-    return None
+    return extension_manager.get_default(category_name)
 
 
 def get_instance(category_name):
@@ -431,13 +603,7 @@ def get_instance(category_name):
     return it.
     Otherwise None
     '''
-    category = get_category(category_name)
-
-    if category is not None:
-        return category.get_instance()
-
-    return None
-
+    return extension_manager.get_instance(category_name)
 
 def get_and_instantiate(category_name, *args, **kwargs):
     '''
@@ -447,32 +613,22 @@ def get_and_instantiate(category_name, *args, **kwargs):
     If this method is called when a reference is already saved, it will
     return that one, NOT a new one.
     '''
-    category = get_category(category_name)
+    return extension_manager.get_and_instantiate(category_name, *args, **kwargs)
 
-    if category is not None:
-        return category.get_and_instantiate(*args, **kwargs)
-
-    return None
 
 def delete_instance(category_name):
     '''
     Delete an instance of a "single interface" category.
     '''
-    category = get_category(category_name)
+    extension_manager.delete_instance(category_name)
 
-    category.instance = None
 
 def set_default(category_name, cls):
     '''set the cls as default for the category category_name, if cls is not
     on the list of registered extensions, then if will be registered,
     if the category exists and the extension is registered return True,
     if the category doesn't exists, return False'''
-    category = get_category(category_name)
-    if category is not None:
-        category.default = cls
-        return True
-
-    return False
+    return extension_manager.set_default(category_name, cls)
 
 
 def set_default_by_id(category_name, id_):
@@ -481,25 +637,18 @@ def set_default_by_id(category_name, id_):
     ValueError
     if the category exists and the extension is registered return True,
     if the category doesn't exists, return False'''
-    category = get_category(category_name)
-    if category is not None:
-        category.set_default_by_id(id_)
-        return True
-
-    return False
-
+    return extension_manager.set_default_by_id(category_name, id_)
 
 def get_system_default(category_name):
     '''return the default category registered by core, it can be used as
     fallback if the default extension on the category raises
     an Exception when instantiated, if the category is not registerd
     return None'''
-    category = get_category(category_name)
-    if category is not None:
-        return category.system_default
-
-    return None
+    return extension_manager.get_system_default(category_name)
 
+def implements(*categories):
+    '''decorator to nicely show which interfaces we are implementing'''
+    return extension_manager.implements(*categories)
 
 def is_implementation(cls, interface_cls):
     '''Check if cls implements all the methods provided by interface_cls.
@@ -527,17 +676,8 @@ def _get_class_name(cls):
 
     return path
 
+def subscribe(callback, item=None):
+    extension_manager.subscribe(callback, item)
 
-def implements(*categories):
-    '''decorator to nicely show which interfaces we are implementing'''
-
-    def _impl(ext):
-        '''decorating function'''
-        ext.implements = categories
-        for ctg in categories:
-            if not get_category(ctg):
-                category_register(ctg, None)
-            get_category(ctg).register(ext)
-        return ext
-
-    return _impl
+def unsubscribe(callback, item=None):
+    extension_manager.unsubscribe(callback, item)
