@@ -67,8 +67,9 @@ class AdiumTheme(MetaData):
 
         info_file = file(os.path.join(path, 'Contents', 'Info.plist'))
         info = parsers.Plist(info_file).info
+        theme_variants = sorted(self.get_theme_variants())
         self.default_variant = info.get('DefaultVariant', None) or \
-                               sorted(self.get_theme_variants())[0]
+                               ('' if not theme_variants else theme_variants[0])
         self.variant = variant
         self.theme_version = info.get('MessageViewVersion', 0)
 
@@ -123,7 +124,8 @@ class AdiumTheme(MetaData):
             template_path = urljoin("gui", "base", "template.html")
         self.template = read_file(template_path)
 
-        self.header = read_file(self.resources_path, 'Header.html') or ""
+        #~ self.header = read_file(self.resources_path, 'Header.html') or ""
+        self.header = ""
         self.footer = read_file(self.resources_path, 'Footer.html') or ""
 
     def _format_incoming(self, msg):
@@ -225,6 +227,8 @@ class AdiumTheme(MetaData):
 
         template = template.replace('%message%', escape_no_xml(msg.message))
 
+        template = re.sub("%time{(.*?)}%", replace_time, template)
+
         if msg.timestamp is None:
             template = template.replace('%time%',
                 escape(time.strftime("%H:%M")))
@@ -237,7 +241,6 @@ class AdiumTheme(MetaData):
             template = template.replace('%time%',
                 escape(d_time.strftime('%x %X')))
 
-        template = re.sub("%time{(.*?)}%", replace_time, template)
         template = template.replace('%shortTime%',
             escape(time.strftime("%H:%M")))
         template = template.replace('%service%', escape(msg.service))
@@ -372,7 +375,14 @@ def unescape(string_):
 
 def replace_time(match):
     '''replace the format of the time to it's value'''
-    return time.strftime(match.groups()[0])
+    #Python time function differs from adium time
+    #so we try to convert it but if that fails,
+    #we fallback to simple time
+    format = match.groups()[0]
+    result = time.strftime(match.groups()[0])
+    if result == format:
+        result = '%time%'
+    return result
 
 def style_message(msgtext, style):
     '''add html markupt to msgtext to format the style of the message'''
