@@ -50,9 +50,10 @@ class Conversation(object):
 
         self._message_waiting = False
 
-        buffer_size = session.config.get_or_set("i_msg_history_size", 5)
+        buffer_size = 10
         self.messages = RingBuffer(max=buffer_size)
         self.message_offset = 0
+        self.message_input = ""
 
         if members is None:
             self.members = []
@@ -204,10 +205,6 @@ class Conversation(object):
         self.session.config.font_color = '#' + color.to_hex()
         self.input.update_style(self.cstyle)
 
-    def on_style_selected(self, style):
-        '''called when a new font is selected'''
-        self.cstyle = style
-
     def on_invite(self, account):
         '''called when a contact is selected to be invited'''
         self.session.conversation_invite(self.cid, account)
@@ -217,13 +214,19 @@ class Conversation(object):
         self.session.filetransfer_invite(self.cid, self.members[0],
                 filename, completepath, self.get_preview(completepath))
 
+    def check_visible(self):
+        ''' called by conversation manager to prevent hidden conversations
+            from receiving messages '''
+        raise NotImplementedError
+
     def on_video_call(self):
         '''called when the user is requesting a video-only call'''
         raise NotImplementedError
 
     def on_toggle_avatar(self):
         '''hide or show the avatar bar'''
-        raise NotImplementedError
+        # widget visibility is handled in _on_show_info_changed
+        self.session.config.b_show_info = not self.session.config.b_show_info
 
     def on_voice_call(self):
         '''called when the user is requesting an audio-only call'''
@@ -238,15 +241,24 @@ class Conversation(object):
         self.output.clear()
         self.conv_status.clear()
 
+    def _on_block_user_accept(self, accept, account):
+        if accept == gui.stock.YES:
+            self.session.block(account)
+
     def on_block_user(self):
         '''blocks the first user of the conversation'''
         account = self.members[0]
-        contact = self.session.contacts.contacts[account]
+        contact = self.session.contacts.get(account)
+
+        if contact is None:
+            return
 
         if contact.blocked:
             self.session.unblock(account)
         else:
-            self.session.block(account)
+            dialog = extension.get_default('dialog')
+            dialog.yes_no(_("Are you sure you want to block this contact?"),
+                          self._on_block_user_accept, account)
 
     def on_contact_block_succeed(self, account):
         if account == self.members[0]:
@@ -399,6 +411,7 @@ class Conversation(object):
         self.output_message(message, cedict)
 
         self.messages.push(text)
+        self.reset_message_offset()
         self.play_send()
         self.conv_status.update_status()
 
@@ -415,10 +428,7 @@ class Conversation(object):
 
     def on_receive_message(self, message, account, received_custom_emoticons):
         '''method called when a message arrives to the conversation'''
-        contact = self.session.contacts.get(account)
-
-        if contact is None:
-            contact = e3.Contact(account)
+        contact = self.session.contacts.safe_get(account)
 
         if message.type == e3.Message.TYPE_MESSAGE or \
            message.type == e3.Message.TYPE_FLNMSG:
@@ -429,6 +439,11 @@ class Conversation(object):
 
             user_emcache = self.caches.get_emoticon_cache(account)
 
+            #XXX: when we send messages from the web iface we get those here, so show them propertly
+            if contact.account == self.session.contacts.me.account:
+                self.output_message(message, None)
+                return
+
             self.input_message(message, contact, 
                                received_custom_emoticons, user_emcache.path)
 
@@ -499,14 +514,8 @@ class Conversation(object):
         if self.is_group_chat:
             text = _('Group chat')
         elif len(self.members) == 1:
-            contact = self.session.contacts.get(self.members[0])
-
-            # can be false if we are un a group chat with someone we dont
-            # have and the last contact leaves..
-            if contact:
-                text = contact.display_name
-            else:
-                text = self.members[0]
+            contact = self.session.contacts.safe_get(self.members[0])
+            text = contact.display_name
         else:
             log.debug('unknown state on Conversation._get_text')
             text = '(?)'
@@ -530,8 +539,8 @@ class Conversation(object):
         '''called when a contact joins the conversation'''
         if account not in self.members:
             self.members.append(account)
-            contact = self.session.contacts.get(account)
-            if contact and len(self.members) > 1:
+            contact = self.session.contacts.safe_get(account)
+            if len(self.members) > 1:
                 message = e3.base.Message(e3.base.Message.TYPE_MESSAGE, \
                 _('%s has joined the conversation') % (contact.display_name), \
                 account)
@@ -548,15 +557,14 @@ class Conversation(object):
         if len(self.members) > 1 and account in self.members:
             self.members.remove(account)
             self.update_data()
-            contact = self.session.contacts.get(account)
-            if contact:
-                message = e3.base.Message(e3.base.Message.TYPE_MESSAGE, \
-                _('%s has left the conversation') % (contact.display_name), \
-                account)
-                msg = gui.Message.from_information(contact, message)
+            contact = self.session.contacts.safe_get(account)
+            message = e3.base.Message(e3.base.Message.TYPE_MESSAGE, \
+            _('%s has left the conversation') % (contact.display_name), \
+            account)
+            msg = gui.Message.from_information(contact, message)
 
-                self.output.information(self.formatter, msg)
-                self.conv_status.post_process_message(msg)
+            self.output.information(self.formatter, msg)
+            self.conv_status.post_process_message(msg)
 
     def on_group_started(self):
         '''called when a group conversation starts'''
@@ -568,29 +576,20 @@ class Conversation(object):
 
     def _update_single_information(self, account):
         '''set the data of the conversation to the data of the account'''
-        contact = self.session.contacts.get(account)
+        contact = self.session.contacts.safe_get(account)
 
-        if contact:
-            if contact.media == '':
-                message = MarkupParser.escape(contact.message)
-            else:
-                message = MarkupParser.escape(contact.media)
-
-            nick = MarkupParser.escape(contact.display_name)
+        if contact.media == '':
+            message = MarkupParser.escape(contact.message)
         else:
-            message = ''
-            nick = account
+            message = MarkupParser.escape(contact.media)
+
+        nick = MarkupParser.escape(contact.display_name)
 
         self.update_single_information(nick, message, account)
 
     def _set_image_visible(self, value):
         '''hide or show the widget according to value'''
         self.set_image_visible(value)
-        if value:
-            self.info.show()
-        else:
-            self.info.hide()
-
         self._image_visible = value
 
     def _get_image_visible(self):
@@ -606,7 +605,7 @@ class Conversation(object):
         self._header_visible = value
 
     def _get_header_visible(self):
-        '''return the value of image_visible'''
+        '''return the value of _header_visible'''
         return self._header_visible
 
     header_visible = property(fget=_get_header_visible,
@@ -657,11 +656,23 @@ class Conversation(object):
         -1 will go backwards
 
         if no message in the buffer return an empty string
-        """
-        self.message_offset += change
 
+        if there is message before cycling, it will be stored
+        till the message sent
+        """
+        if change < 0 and self.message_offset == 0:
+            self.message_input = self.input.text
+
+        self.message_offset += change
+        if abs(self.message_offset) + 1 > len(self.messages):
+            self.message_offset=1 - len(self.messages)
+        if self.message_offset > 0:
+            self.reset_message_offset()
         try:
-            self.input.text = self.messages.peak(self.message_offset)
+            if self.message_offset == 0:
+                self.input.text = self.message_input
+            else:
+                self.input.text = self.messages.peak(self.message_offset)
         except IndexError:
             pass
 
@@ -669,7 +680,7 @@ class Conversation(object):
         self.message_offset = 0
 
     def _member_to_contact(self,member):
-        return self.session.contacts.contacts[member]
+        return self.session.contacts.safe_get(member)
 
     def steal_emoticon(self, uri):
         """
