@@ -31,16 +31,12 @@ class ConversationManager(object):
         self.on_last_close = on_last_close
 
         self.conversations = {}
-        if self.session:
-            self.subscribe_signals()
-
-        conversation_tabs = self.session.config.get_or_set(
-                'b_conversation_tabs', True)
-
-        if self.session.conversations is None or conversation_tabs:
-            self.session.conversations = {}
+        self.subscribe_signals()
 
     def subscribe_signals(self):
+        if not self.session:
+            return
+
         self.session.signals.conv_message.subscribe(
             self._on_message)
         self.session.signals.gui_message.subscribe(
@@ -63,6 +59,9 @@ class ConversationManager(object):
             self._on_p2p_finished)
 
     def unsubscribe_signals(self):
+        if not self.session:
+            return
+
         self.session.signals.conv_message.unsubscribe(
             self._on_message)
         self.session.signals.gui_message.unsubscribe(
@@ -92,16 +91,26 @@ class ConversationManager(object):
 
     def _on_message(self, cid, account, message, cedict=None):
         '''called when a message is received'''
+
+        log.debug('Message received: %f, %s' % (cid, account))
+
         conversation = self.has_similar_conversation(cid, [account])
         conversation_tabs = self.session.config.get_or_set(
                 'b_conversation_tabs', True)
 
-        if conversation is None and conversation_tabs:
+        if conversation is None and not conversation_tabs:
+            return
+
+        if conversation is None:
             conversation = self.new_conversation(cid, [account])
 
         if conversation is not None:
             self.set_message_waiting(conversation, True)
             conversation.on_receive_message(message, account, cedict)
+        else:
+            log.error('No conversation found. Available cids are:')
+            for conv in self.conversations.itervalues():
+                log.error('%f, %s' % (conv.cid, conv.members))
 
     def _on_user_typing(self, cid, account, *args):
         """
@@ -181,19 +190,27 @@ class ConversationManager(object):
         if not found return None
         '''
         cid = float(cid)
+        log.debug('Looking for conversations with the same cid: %f' % cid)
 
         if cid in self.conversations:
+            log.debug('A similar conversation was found with the same cid: %f' % self.conversations[cid].icid)
             return self.conversations[cid]
 
         if members is not None:
             for conversation in self.conversations.itervalues():
                 if conversation.members == members:
+                    log.debug('A similar conversation was found with the '
+                              'same members and cid: %f' % conversation.cid)
                     return conversation
 
         for conversation in self.conversations.itervalues():
             if conversation.icid == cid:
+                log.debug('A similar conversation was found with the '
+                          'same icid: %f' % conversation.icid)
                 return conversation
 
+        log.debug('No similar conversation was found')
+
         return None
 
     def reuse_conversation(self, cid, members):
@@ -213,6 +230,8 @@ class ConversationManager(object):
 
         old_cid = conversation.cid
 
+        log.debug('Reusing conversation. Old cid: %f, new cid: %f' % (old_cid, cid))
+
         if old_cid in self.conversations:
             del self.conversations[old_cid]
         if old_cid in self.session.conversations:
@@ -221,6 +240,7 @@ class ConversationManager(object):
         conversation.cid = cid
         self.conversations[cid] = conversation
         self.session.conversations[cid] = conversation
+
         return conversation
 
     def new_conversation(self, cid, members=None):
@@ -230,25 +250,28 @@ class ConversationManager(object):
         this method returns a tuple containing a boolean and a conversation
         object. If the conversation already exists, return True on as first
         value'''
+
+        log.debug('Constructing a new conversation: %f' % cid)
+
         conversation = self.reuse_conversation(cid, members)
 
         if conversation:
+            log.debug('Returning an existing conversation')
             return conversation
 
         conversation = self.add_new_conversation(self.session, cid, members)
         self.conversations[cid] = conversation
         self.session.conversations[cid] = conversation
 
-        self.after_new_conversation(conversation)
         #notify a new conversation has started
         self.session.conv_started(cid, members)
-        return conversation
 
-    def after_new_conversation(self, conversation):
-        '''
-         What to do after create a conversation
-        '''
-        pass
+        # Update and show the conversation
+        conversation.update_data()
+        conversation.show() # puts widget visible
+
+        log.debug('Returning a new conversation')
+        return conversation
 
     def renew_session(self, session):
         '''reopen all conversations when the user reconnects'''
@@ -259,8 +282,8 @@ class ConversationManager(object):
             conversation.subscribe_signals()
             account = conversation.members[0]
 
-            contact = self.session.contacts.contacts[account]
-            conversation.set_sensitive(not contact.blocked,  True)
+            contact = self.session.contacts.safe_get(account)
+            conversation.set_sensitive(not contact.blocked, True)
  
             self.reuse_conversation(cid, [account])
             self.session.new_conversation(account, cid)
@@ -286,31 +309,18 @@ class ConversationManager(object):
             if account in conversation.members:
                 conversation.update_p2p(account, _type, *what)
 
-    def on_conversation_close(self, conversation):
-        """
-        called when the user wants to close a conversation widget
-        """
-        # TODO: there is a strange case when changing the tabbed to no tabbed
-        # config, for some reason that conversations don't seem to be removed
-        self.close(conversation)
-
-        if len(self.conversations) == 0:
-            self.on_last_close()
-
     def close(self, conversation):
         '''close a conversation'''
+        log.debug('Closing conversation: %f' % conversation.cid)
         self.session.close_conversation(conversation.cid)
-        self.remove_conversation(conversation)
         del self.conversations[conversation.cid]
         del self.session.conversations[conversation.cid]
+        self.remove_conversation(conversation)
         conversation.on_close()
-        self.after_close()
 
-    def after_close(self):
-        '''
-        Override what to do after close a conversation
-        '''
-        pass
+        if len(self.conversations) == 0:
+            log.debug('Closing the conversation window')
+            self.on_last_close()
 
     def close_all(self):
         '''close and finish all conversations'''
@@ -331,12 +341,6 @@ class ConversationManager(object):
         '''
         raise NotImplementedError("not implemented")
 
-    def hide_all(self):
-        '''
-        hide all conversations
-        '''
-        raise NotImplementedError("not implemented")
-
     def is_active(self):
         '''
         return True if the conversation manager is active
