@@ -25,42 +25,15 @@ import e3.base
 import gui
 import extension
 import Desktop
+import Info
 
 import logging
 log = logging.getLogger('gui.base.Handler')
 
-EMESENE_LICENSE = '''    emesene is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 3 of the License, or
-    (at your option) any later version.
-
-    emesene is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with emesene; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-'''
-
-EMESENE_VERSION = "2.12.3"
-EMESENE_LAST_STABLE = "2.12.3"
-EMESENE_WEBSITE = "http://www.emesene.org"
-EMESENE_AUTHORS = "marianoguerra & c10ud"
-EMESENE_CONTRIBUTORS = [ # TODO: Load this from CONTRIBUTORS
-'Riccardo (c10ud) <c10ud.dev@gmail.com>',
-'Mariano Guerra <luismarianoguerra@gmail.com>',
-'arielj <arieljuod@gmail.com>',
-'Stefano Candori <stefanocandori@gmail.com>',
-'4ndreaSt4gi <stagi.andrea@gmail.com>',
-'Davide Lo Re <boyska@gmail.com>',
-'dequis <dx@dxzone.com.ar>',
-'Sven (Sbte) <svenb.linux@gmail.com>',
-'Jose Rostagno <joserostagno@vijona.com.ar>',
-'nenemfromhell <sniper3000@gmail.com>',
-'Gabriele "Whisky" Visconti <visconti.gabriele@gmail.com>',
-]
+#TODO: Remove this someday in future because we still need it
+#      for pre-2.12.4-dev versions searching for updates
+EMESENE_VERSION = Info.EMESENE_VERSION
+EMESENE_LAST_STABLE = Info.EMESENE_LAST_STABLE
 
 class MenuHandler(object):
     '''this handler contains all the handlers needed to handle all the
@@ -99,6 +72,9 @@ class FileHandler(object):
         else:
             self.session.close(True)
 
+    def on_disconnect_endpoint_selected(self, ep_id):
+        self.session.disconnect_endpoint(ep_id)
+
 class ActionsHandler(object):
     '''this handler contains all the handlers needed to handle the actions
     menu items
@@ -119,6 +95,7 @@ class OptionsHandler(object):
         '''constructor'''
         self.session = session
         self.contact_list = contact_list
+        self.preferences = None
 
     def on_order_by_group_toggled(self, active):
         '''called when the order by group radio button is toggled'''
@@ -152,13 +129,17 @@ class OptionsHandler(object):
 
     def on_preferences_selected(self):
         '''called when the preference button is selected'''
-        instance = extension.get_and_instantiate('preferences', self.session)
-        if self.session is not instance.session:
-            instance.remove_subscriptions()
+        self.preferences = extension.get_and_instantiate('preferences',
+                                                         self.session)
+
+        if self.session is not self.preferences.session:
+            self.preferences.remove_subscriptions()
             extension.delete_instance('preferences')
-            instance = extension.get_and_instantiate('preferences', self.session)
-        instance.show()
-        instance.present()
+            self.preferences = extension.get_and_instantiate('preferences',
+                                                             self.session)
+
+        self.preferences.show()
+        self.preferences.present()
 
     def on_plugins_selected(self):
         '''called when the plugins button is selected'''
@@ -178,14 +159,14 @@ class HelpHandler(object):
 
     def on_about_selected(self):
         '''called when the about item is selected'''
-        self.dialog.about_dialog('emesene', EMESENE_VERSION, EMESENE_AUTHORS,
-            _('A simple yet powerful instant messaging client'), EMESENE_LICENSE,
-            EMESENE_WEBSITE, EMESENE_CONTRIBUTORS, _('translator-credits'),
+        self.dialog.about_dialog('emesene', Info.EMESENE_VERSION, Info.EMESENE_AUTHORS,
+            _('A simple yet powerful instant messaging client'), Info.EMESENE_LICENSE,
+            Info.EMESENE_WEBSITE, Info.EMESENE_CONTRIBUTORS, _('translator-credits'),
             gui.theme.image_theme.logo)
 
     def on_website_selected(self):
         '''called when the website item is selected'''
-        Desktop.open(EMESENE_WEBSITE)
+        Desktop.open(Info.EMESENE_WEBSITE)
 
     def on_debug_selected(self):
         '''called when the preference button is selected'''
@@ -200,16 +181,16 @@ class HelpHandler(object):
         self.dialog.information(_("New version available: %(ver)s\nVisit the "
                     "<a href=\"%(web)s\">emesene website</a> for the "
                     "latest informations about it.") % {"ver" : ver,
-                    "web" : EMESENE_WEBSITE})
+                    "web" : Info.EMESENE_WEBSITE})
 
     def on_check_update_selected(self):
         ''' checks if a new stable version of emesene is available '''
-        f = urllib.urlopen("https://github.com/emesene/emesene/raw/master/emesene/gui/base/Handler.py")
+        f = urllib.urlopen("https://github.com/emesene/emesene/raw/master/emesene/Info.py")
         s = f.read()
         f.close()
         s = re.findall(r'EMESENE_LAST_STABLE = "(([^"\\]+|\\.)*)"', s)
         try:
-            loc_ver = EMESENE_VERSION.split(".")
+            loc_ver = Info.EMESENE_VERSION.split(".")
             rem_ver = s[0][0].split(".") # [("version","version")]
             if len(rem_ver[2]) > 2:
                 return # no RCs, BETAs or DEVELs
@@ -290,10 +271,15 @@ class ContactHandler(object):
 
     def on_block_contact_selected(self):
         '''called when block contact is selected'''
+        def on_block_contact_accept(accept, account):
+            if accept == gui.stock.YES:
+                self.session.block(account)
+
         contact = self.contact_list.get_contact_selected()
 
         if contact:
-            self.session.block(contact.account)
+            self.dialog.yes_no(_("Are you sure you want to block this contact?"),
+                          on_block_contact_accept, contact.account)
         else:
             self.dialog.error(_('No contact selected'))
 
@@ -352,7 +338,7 @@ class ContactHandler(object):
         group_src = self.contact_list.get_contact_selected_group()
 
         if contact and group_src and group_dst:
-            self.session.move_to_group(contact.account, 
+            self.session.move_to_group(contact.account,
                                     group_src.identifier, group_dst.identifier)
         else:
             self.dialog.error(_('No contact selected'))
@@ -443,11 +429,11 @@ class GroupHandler(object):
             self.contact_list.fill()
         else:
             self.dialog.error(_('No group selected'))
-    
+
     def on_unset_favorite_group_selected(self):
         ''' called when unset as favorite is selected in a favorite group '''
         group = self.contact_list.get_group_selected()
-        
+
         if group:
             # reset group weight
             self.session.config.d_weights[group.identifier] = 0
@@ -482,10 +468,9 @@ class MyAccountHandler(object):
         self.session.set_nick(nick)
         self.session.set_message(pm)
 
-
     def on_set_picture_selected(self, widget, data=None):
         '''called when set picture is selected'''
-        _av_chooser = extension.get_default('avatar chooser')(self.session)
+        _av_chooser = extension.get_and_instantiate('avatar chooser', self.session)
         _av_chooser.show()
 
 class ConversationToolbarHandler(object):
@@ -513,15 +498,11 @@ class ConversationToolbarHandler(object):
         self.dialog.select_color(self.conversation.cstyle.color,
             self.conversation.on_color_selected)
 
-    def on_style_selected(self):
-        '''called when the Style button is selected'''
-        self.dialog.select_style(self.conversation.cstyle,
-            self.conversation.on_style_selected)
-
     def on_invite_selected(self):
         '''called when the Invite button is selected'''
+        l_buddy_exclude = list(self.conversation.members)
         self.dialog.invite_dialog(self.session,
-            self.conversation.on_invite)
+                                  self.conversation.on_invite, l_buddy_exclude)
 
     def on_clean_selected(self):
         '''called when the Clean button is selected'''
@@ -540,9 +521,16 @@ class ConversationToolbarHandler(object):
         '''called when the client requestes to a remote user to
         start a file transfer'''
         def open_file_cb(response, filepath):
-            if response is not gui.stock.CANCEL:
-                filename = os.path.basename(filepath)
-                self.conversation.on_filetransfer_invite(filename, filepath)
+            if response is not gui.stock.CANCEL and \
+                response is not gui.stock.CLOSE:
+
+                if filepath is None:
+                    extension.get_default('dialog').error(
+                                                _("No file selected"))
+                else:
+                    filename = os.path.basename(filepath)
+                    self.conversation.on_filetransfer_invite(filename,
+                                                             filepath)
 
         self.dialog.choose_file(os.path.expanduser("~"), open_file_cb)
 
@@ -553,7 +541,7 @@ class ConversationToolbarHandler(object):
     def on_toggle_avatar_selected(self):
         '''hide or show the avatar bar'''
         self.conversation.on_toggle_avatar()
-		
+
     def on_invite_video_call_selected(self):
         '''called when the user is requesting a video-only call'''
         self.conversation.on_video_call()
@@ -596,23 +584,28 @@ class TrayIconHandler(FileHandler):
         """
         FileHandler.__init__(self, session, on_quit)
         self.theme = theme
-        
+        self.preferences = None
+
     def on_hide_show_mainwindow(self, main_window=None):
         if main_window is not None:
             if main_window.get_visible():
                 main_window.hide()
             else:
                 main_window.present()
-                
+
     def on_preferences_selected(self):
         '''called when the preference button is selected'''
-        instance = extension.get_and_instantiate('preferences', self.session)
-        if self.session is not instance.session:
-            instance.remove_subscriptions()
+        self.preferences = extension.get_and_instantiate('preferences',
+                                                         self.session)
+
+        if self.session is not self.preferences.session:
+            self.preferences.remove_subscriptions()
             extension.delete_instance('preferences')
-            instance = extension.get_and_instantiate('preferences', self.session)
-        instance.show()
-        instance.present()
+            self.preferences = extension.get_and_instantiate('preferences',
+                                                             self.session)
+
+        self.preferences.show()
+        self.preferences.present()
 
 class FileTransferHandler(object):
     ''' this handler handles a file transfer object '''
@@ -641,7 +634,7 @@ class FileTransferHandler(object):
         ''' when a file transfer is accepted by the other party'''
         self.transfer.time_start = time.time()
         self.transfer.state = e3.base.FileTransfer.TRANSFERRING
-        
+
     def reject(self):
         ''' rejects a file transfer '''
         self.transfer.state = e3.base.FileTransfer.FAILED
@@ -670,7 +663,7 @@ class CallHandler(object):
     def accepted(self):
         ''' when a call is accepted by the other party'''
         self.call.state = e3.base.Call.ESTABLISHED
-        
+
     def reject(self):
         ''' rejects a call '''
         self.call.state = e3.base.Call.FAILED
