@@ -23,11 +23,14 @@ from Handler import *
 from Message import Message
 from ContactList import ContactList
 from Conversation import Conversation
+from LoginBase import LoginBase
 from ConversationManager import ConversationManager
 from AvatarManager  import AvatarManager
 from PictureHandler import PictureHandler
 from ContactInformation import ContactInformation
 from BaseTray import BaseTray
+from MainWindowBase import MainWindowBase
+from Hotmail import Hotmail
 
 theme = Theme()
 
