@@ -21,6 +21,7 @@ import gtk
 import gui
 import extension
 import glib
+from gui.gtkui import check_gtk3
 
 class ContactInfoRotate(gtk.VBox):
     '''a widget that contains the display pictures of the contacts and our
@@ -72,23 +73,31 @@ class ContactInfoRotate(gtk.VBox):
         self.his_avatarBox.set_tooltip_text(_('Click to see informations'))
         self.his_avatarBox.set_border_width(4)
 
-        last_avatar = self.session.config.last_avatar
-        if self.session.config_dir.file_readable(last_avatar):
-            my_picture = last_avatar
-        else:
-            my_picture = gui.theme.image_theme.user
+        if check_gtk3():
+            #In gtk3 eventbox renders background, so make it transparent
+            prov = gtk.CssProvider()
+            prov.load_from_data("* {\n"
+                "background-color: transparent;\n"
+                "}");
 
-        his_picture = gui.theme.image_theme.user
+            context = self.his_avatarBox.get_style_context()
+            context.add_provider(prov, 600) #GTK_STYLE_PROVIDER_PRIORITY_APPLICATION
+            context.save()
+
+            context = self.avatarBox.get_style_context()
+            context.add_provider(prov, 600) #GTK_STYLE_PROVIDER_PRIORITY_APPLICATION
+            context.save()
+
+        my_picture = self.session.config.last_avatar
+
+        # Obtains his picture and details.
+        contact = self.session.contacts.safe_get(None)
         if members is not None:
             account = members[0]
-            contact = self.session.contacts.get(account)
-
-            if contact:
-                if contact.picture:
-                    his_picture = contact.picture
+            contact = self.session.contacts.safe_get(account)
 
         self.first = self.his_avatarBox
-        self.his_avatar.set_from_file(his_picture)
+        self.his_avatar.set_from_file(contact.picture, contact.blocked)
 
         self.last = self.avatarBox
         self.avatar.set_from_file(my_picture)
@@ -132,7 +141,7 @@ class ContactInfoRotate(gtk.VBox):
 
     def _on_avatar_click(self, widget, data):
         '''method called when user click on his avatar '''
-        av_chooser = extension.get_default('avatar chooser')(self.session)
+        av_chooser = extension.get_and_instantiate('avatar chooser',  self.session)
         av_chooser.set_modal(True)
         av_chooser.show()
 
@@ -167,7 +176,7 @@ class ContactInfoRotate(gtk.VBox):
 
         path = contact.picture
         if path != '':
-            self.his_avatar.set_from_file(path)
+            self.his_avatar.set_from_file(path, contact.blocked)
 
         self.index = (self.index+1)%len(self.members)
         return True
@@ -194,10 +203,10 @@ class ContactInfoRotate(gtk.VBox):
             self.timer = None
 
         account = members[0]
-        contact = self.session.contacts.get(account)
-        if contact and contact.picture:
+        contact = self.session.contacts.safe_get(account)
+        if contact.picture:
             his_picture = contact.picture
-            self.his_avatar.set_from_file(his_picture)
+            self.his_avatar.set_from_file(his_picture, contact.blocked)
 
     def update_group(self, members):
         self.members = members
