@@ -27,7 +27,12 @@ import gui
 import utils
 import extension
 import gobject
-import Renderers
+from gui.gtkui import check_gtk3
+
+if check_gtk3():
+    import RenderersNew as Renderers
+else:
+    import Renderers
 
 import gui.base.ConversationStatus as ConversationStatus
 
@@ -43,6 +48,7 @@ class ContactInformation(gtk.Window, gui.base.ContactInformation):
         '''constructor'''
         gui.base.ContactInformation.__init__(self, session, account)
         gtk.Window.__init__(self)
+        self.session = session
         self.set_default_size(640, 350)
         self.set_title(_('Contact information (%s)') % (account,))
         self.set_position(gtk.WIN_POS_CENTER)
@@ -61,7 +67,6 @@ class ContactInformation(gtk.Window, gui.base.ContactInformation):
     def _create_tabs(self):
         '''create all the tabs on the window'''
         self.info = InformationWidget(self.session, self.account)
-        self.nicks = ListWidget(self.session, self.account)
 
         self.avatar_manager = gui.base.AvatarManager(self.session)
 
@@ -69,14 +74,17 @@ class ContactInformation(gtk.Window, gui.base.ContactInformation):
 
         self.avatars = IconView(_('Avatar history'), [account_path],
                         None, None, IconView.TYPE_SELF_PICS, None)
-        self.messages = ListWidget(self.session, self.account)
         self.status = ListWidget(self.session, self.account)
         self.chats = ChatWidget(self.session, self.account)
 
         self.tabs.append_page(self.info, gtk.Label(_('Information')))
-        self.tabs.append_page(self.nicks, gtk.Label(_('Nick history')))
+        if self.session.session_has_service(e3.Session.SERVICE_CONTACT_NICK):
+            self.nicks = ListWidget(self.session, self.account)
+            self.tabs.append_page(self.nicks, gtk.Label(_('Nick history')))
         self.tabs.append_page(self.avatars, gtk.Label(_('Avatar history')))
-        self.tabs.append_page(self.messages, gtk.Label(_('Message history')))
+        if self.session.session_has_service(e3.Session.SERVICE_CONTACT_PM):
+            self.messages = ListWidget(self.session, self.account)
+            self.tabs.append_page(self.messages, gtk.Label(_('Message history')))
         self.tabs.append_page(self.status, gtk.Label(_('Status history')))
         self.tabs.append_page(self.chats, gtk.Label(_('Chat history')))
 
@@ -119,7 +127,8 @@ class InformationWidget(gtk.VBox):
         self.message.set_alignment(0.0, 0.5)
         self.status = gtk.Image()
         self.status.set_alignment(0.0, 0.5)
-        self.image = gtk.Image()
+        Avatar = extension.get_default('avatar')
+        self.image = Avatar()
         image_align = gtk.Alignment(0.5,0.5)
         image_align.add(self.image)
         self.blocked = gtk.Label()
@@ -168,22 +177,23 @@ class InformationWidget(gtk.VBox):
         '''update the information of the contact'''
         if self.contact:
             if self.contact.display_name == self.contact.nick:
-                self.nick.set_markup(Renderers.msnplus_to_list(
-                        gobject.markup_escape_text(self.contact.display_name)))
+                self.nick.set_markup(
+                        gobject.markup_escape_text(self.contact.display_name))
             else:
-                self.nick.set_markup(Renderers.msnplus_to_list(
+                self.nick.set_markup(
                         gobject.markup_escape_text(self.contact.nick
-                            + ' (' + self.contact.display_name + ')')))
+                            + ' (' + self.contact.display_name + ')'))
+
             self.mail.set_markup(self.contact.account)
-            self.message.set_markup(Renderers.msnplus_to_list(
-                    gobject.markup_escape_text(self.contact.message)))
+            self.message.set_markup(
+                gobject.markup_escape_text(self.contact.message))
             self.status.set_from_file(
-                gui.theme.image_theme.status_icons[self.contact.status])
-            if (self.contact.picture):
-                self.image.set_from_file(self.contact.picture)
-            else:
-                self.image.set_from_file(gui.theme.image_theme.user)
-            if (self.contact.blocked):
+            gui.theme.image_theme.status_icons[self.contact.status])
+            
+            self.image.set_from_file(self.contact.picture,
+                                     self.contact.blocked)
+
+            if self.contact.blocked:
                 self.blocked.set_markup(_('Yes'))
             else:
                 self.blocked.set_markup(_('No'))
@@ -437,6 +447,7 @@ class ChatWidget(gtk.VBox):
                 return
 
             exporter = extension.get_default('history exporter')
+
             exporter(results, open(path, "w"))
 
         self.request_chats_between(limit, _on_save_chats_ready)
