@@ -98,40 +98,55 @@ class ContactList(gui.ContactList, gtk.TreeView):
         self.connect('motion-notify-event', self.tooltips.on_motion)
         self.connect('leave-notify-event', self.tooltips.on_leave)
 
-        crt = extension.get_and_instantiate('nick renderer')
-        crt.set_property('ellipsize', pango.ELLIPSIZE_END)
-        pbr_status = gtk.CellRendererPixbuf()
+        self.crt = extension.get_and_instantiate('nick renderer')
 
-        column = gtk.TreeViewColumn()
-        column.set_expand(True)
+        self.column = gtk.TreeViewColumn()
+        self.column.set_expand(True)
 
         self.exp_column = gtk.TreeViewColumn()
         self.exp_column.set_max_width(16)
 
         self.append_column(self.exp_column)
-        self.append_column(column)
+        self.append_column(self.column)
         self.set_expander_column(self.exp_column)
 
-        column.pack_start(self.pbr, False)
-        column.pack_start(crt, True)
-        column.pack_start(pbr_status, False)
-
-        column.add_attribute(self.pbr, 'image', 0)
-        column.add_attribute(crt, 'markup', 2)
-        column.add_attribute(self.pbr, 'visible', 3)
-        column.add_attribute(pbr_status, 'visible', 3)
-        column.add_attribute(pbr_status, 'pixbuf', 4)
-        column.add_attribute(self.pbr, 'offline', 7)
+        self.prepare_contactlist_column()
 
         self.set_search_column(2)
         self.set_headers_visible(False)
 
-        self.connect('row-activated', self._on_row_activated)
-        self.connect('button-release-event' , self._on_button_press_event)
-        self.connect('row-expanded' , self._on_expand)
-        self.connect('row-collapsed' , self._on_collapse)
-        self.connect('drag-data-get', self._on_drag_data_get)
-        self.connect('drag-drop', self._on_drag_drop)
+        self.connect('row-activated',  self._on_row_activated)
+        self.connect('button-release-event',  self._on_button_press_event)
+        self.connect('row-expanded',  self._on_expand)
+        self.connect('row-collapsed',  self._on_collapse)
+        self.connect('drag-data-get',  self._on_drag_data_get)
+        self.connect('drag-drop',  self._on_drag_drop)
+        extension.subscribe(self.on_nick_renderer_changed, 'nick renderer')
+
+    def prepare_contactlist_column(self):
+        '''update column properties'''
+        self.crt.set_property('ellipsize', pango.ELLIPSIZE_END)
+        pbr_status = gtk.CellRendererPixbuf()
+        self.column.pack_start(self.pbr, False)
+        self.column.pack_start(self.crt, True)
+        self.column.pack_start(pbr_status, False)
+        self.column.add_attribute(self.pbr, 'image', 0)
+        self.column.add_attribute(self.crt, 'markup', 2)
+        self.column.add_attribute(self.pbr, 'visible', 3)
+        self.column.add_attribute(pbr_status, 'visible', 3)
+        self.column.add_attribute(pbr_status, 'pixbuf', 4)
+        self.column.add_attribute(self.pbr, 'offline', 7)
+
+    def on_nick_renderer_changed(self, new_renderer):
+        if type(self.crt) != new_renderer:
+            #remove and and again
+            self.column.clear()
+            self.crt = new_renderer()
+            self.prepare_contactlist_column()
+
+    def remove_subscriptions(self):
+        gui.ContactList.remove_subscriptions(self)
+        extension.unsubscribe(self.on_nick_renderer_changed, 'nick renderer')
 
     def _set_accels(self, mainwindow):
         """set the keyboard shortcuts
@@ -141,15 +156,19 @@ class ContactList(gui.ContactList, gtk.TreeView):
         self.accel_group = accel_group
         accel_group.connect_group(gtk.keysyms.Delete, 0, gtk.ACCEL_LOCKED,
                 self._on_key_delete_item)
+        accel_group.connect_group(gtk.keysyms.T,
+                                  gtk.gdk.CONTROL_MASK,
+                                  gtk.ACCEL_LOCKED,
+                                  self.open_conversation)
 
     def _on_key_delete_item(self, accel_group, window, keyval, mod):
         if self.is_focus():
             if self.is_contact_selected():
                 self.contact_handler.on_remove_contact_selected()
-            
+
             elif self.is_group_selected() and \
                  self.group_handler.is_by_group_view():
-                
+
                 self.group_handler.on_remove_group_selected()
 
     def _on_expand(self, treeview, iter_, path):
@@ -182,7 +201,7 @@ class ContactList(gui.ContactList, gtk.TreeView):
                 if bool(contact.blocked):
                     pixbufblock=utils.gtk_pixbuf_load(gui.theme.image_theme.blocked_overlay)
                     utils.simple_images_overlap(pix, pixbufblock,
-                                                -pixbufblock.props.width, 
+                                                -pixbufblock.props.width,
                                                 -pixbufblock.props.width)
 
                 picture = gtk.image_new_from_pixbuf(pix)
@@ -212,14 +231,14 @@ class ContactList(gui.ContactList, gtk.TreeView):
             if bool(contact.blocked):
                 pixbufblock=utils.gtk_pixbuf_load(gui.theme.image_theme.blocked_overlay)
                 utils.simple_images_overlap(pix, pixbufblock,
-                                            -pixbufblock.props.width, 
+                                            -pixbufblock.props.width,
                                             -pixbufblock.props.width)
 
             picture = gtk.image_new_from_pixbuf(pix)
 
         return picture
 
-    def _visible_func(self, model, _iter):
+    def _visible_func(self, model, _iter, *args):
         '''return True if the row should be displayed according to the
         value of the config'''
         obj = self._model[_iter][1]
@@ -298,13 +317,24 @@ class ContactList(gui.ContactList, gtk.TreeView):
 
     def _get_selected(self):
         '''return the selected row or None'''
-        iter_ = self.get_selection().get_selected()[1]
+        try:
+            iter_ = self.get_selection().get_selected()[1]
+        except AttributeError:
+            return None
 
         if iter_ is None:
             return None
 
         return self.model.convert_iter_to_child_iter(iter_)
 
+    def open_conversation(self, *args):
+        """
+        Opens a new conversation if a contact is selected
+        """
+        contact = self.get_contact_selected()
+        if contact:
+            self.contact_selected.emit(contact)
+
     def _on_row_activated(self, treeview, path, view_column):
         '''callback called when the user selects a row'''
         group = self.get_group_selected()
@@ -477,7 +507,7 @@ class ContactList(gui.ContactList, gtk.TreeView):
             utils.safe_gtk_pixbuf_load(gui.theme.image_theme.status_icons[contact.status]),
             weight, False, offline)
 
-        # if group_offline is set and the contact is offline 
+        # if group_offline is set and the contact is offline
         # then put it on the special offline group
         if self.group_offline and offline:
             duplicate = self._duplicate_check(contact)
@@ -485,11 +515,11 @@ class ContactList(gui.ContactList, gtk.TreeView):
                 return duplicate
             if not self.offline_group:
                 self.session.config.d_weights['1'] = 0
-                self.offline_group = e3.Group(_("Offline"), 
-                                              identifier='1', 
+                self.offline_group = e3.Group(_("Offline"),
+                                              identifier='1',
                                               type_=e3.Group.OFFLINE)
 
-                self.offline_group_iter = self.add_group(self.offline_group, 
+                self.offline_group_iter = self.add_group(self.offline_group,
                                                          True)
 
             self.offline_group.contacts.append(contact.account)
@@ -506,11 +536,11 @@ class ContactList(gui.ContactList, gtk.TreeView):
             if not self.online_group:
                 self.session.config.d_weights['0'] = 1
 
-                self.online_group = e3.Group(_("Online"), 
+                self.online_group = e3.Group(_("Online"),
                                             identifier='0',
                                             type_=e3.Group.ONLINE)
 
-                self.online_group_iter = self.add_group(self.online_group, 
+                self.online_group_iter = self.add_group(self.online_group,
                                                         True)
 
             self.online_group.contacts.append(contact.account)
@@ -527,8 +557,8 @@ class ContactList(gui.ContactList, gtk.TreeView):
 
                 return self._model.append(self.no_group_iter, contact_data)
             else:
-                self.no_group = e3.Group(_("No group"), 
-                                         identifier='0', 
+                self.no_group = e3.Group(_("No group"),
+                                         identifier='0',
                                          type_ = e3.Group.NONE)
 
                 self.no_group_iter = self.add_group(self.no_group, True)
@@ -551,7 +581,7 @@ class ContactList(gui.ContactList, gtk.TreeView):
                 # check on the root
                 elif isinstance(obj, e3.Contact) and \
                         obj.account == contact.account:
-                    
+
                     return row.iter
 
             return self._model.append(None, contact_data)
@@ -589,7 +619,7 @@ class ContactList(gui.ContactList, gtk.TreeView):
     def remove_contact(self, contact, group=None):
         '''remove a contact from the specified group, if group is None
         then remove him from all groups'''
-        if not group:
+        if group is None:
             # go though the groups and the contacts without group
             for row in self._model:
                 obj = row[1]
@@ -600,7 +630,7 @@ class ContactList(gui.ContactList, gtk.TreeView):
                         # if we find it, we remove it
                         if con.account == contact.account:
                             # we remove it from tree and from group.
-                            del self._model[contact_row.iter]
+                            self._model.remove(contact_row.iter)
                             del con
                             if obj.contacts.count(contact.account) > 0:
                                 obj.contacts.remove(contact.account)
@@ -609,9 +639,8 @@ class ContactList(gui.ContactList, gtk.TreeView):
                 # if it's a contact without group (at the root)
                 elif isinstance(obj, e3.Contact) and \
                       obj.account == contact.account:
-                    
                     # TODO: Is removing only the tree object?
-                    del self._model[row.iter]
+                    self._model.remove(row.iter)
                     del obj # CHECK!!!!!
 
             return
@@ -626,7 +655,7 @@ class ContactList(gui.ContactList, gtk.TreeView):
                     con = contact_row[1]
                     # if we find it, we remove it, from group and model
                     if con.account == contact.account:
-                        del self._model[contact_row.iter]
+                        self._model.remove(contact_row.iter)
 
                         if group.contacts.count(contact.account) > 0:
                             group.contacts.remove(contact.account)
@@ -757,8 +786,15 @@ class ContactList(gui.ContactList, gtk.TreeView):
         self._model[self.offline_group_iter] = group_data
         self.update_group(self.offline_group)
 
+    def expand_groups(self):
+        ''' expand groups while searching'''
+        if self.is_searching:
+            self.expand_all()
+            return
+
     def un_expand_groups(self):
         ''' restore groups after a search'''
+        self.get_selection().unselect_all()
         for row in self._model:
             obj = row[1]
             if isinstance(obj, e3.Group):
@@ -779,7 +815,7 @@ class ContactList(gui.ContactList, gtk.TreeView):
             obj = row[1]
             if isinstance(obj, e3.Group) and \
                 obj.identifier == group.identifier:
-                
+
                 path = None
                 childpath = None
                 if group.name in self.group_state:
@@ -795,7 +831,7 @@ class ContactList(gui.ContactList, gtk.TreeView):
                 group.contacts = obj.contacts
                 group_data = (None, group,
                               self.format_group(group),
-                              False, None, weight, 
+                              False, None, weight,
                               (group.type != e3.base.Group.STANDARD), False)
 
                 self._model[row.iter] = group_data
@@ -823,8 +859,14 @@ class ContactList(gui.ContactList, gtk.TreeView):
             return override
 
         if self.order_by_name:
-            return cmp(Plus.msnplus_strip(contact1.display_name),
-                       Plus.msnplus_strip(contact2.display_name))
+            # first order by status, online contacts first
+            result = cmp(e3.status.ORDERED.index(contact1.status),
+                e3.status.ORDERED.index(contact2.status))
+            if result == 0:
+                #same status, order by name
+                result = cmp(Plus.msnplus_strip(contact1.display_name),
+                            Plus.msnplus_strip(contact2.display_name))
+            return result
 
         result = cmp(e3.status.ORDERED.index(contact1.status),
             e3.status.ORDERED.index(contact2.status))
@@ -877,11 +919,11 @@ class ContactList(gui.ContactList, gtk.TreeView):
 
     def _on_drag_drop(self, widget, drag_context, x, y, time):
         drag_context.finish(True, False, time)
-        if self.session.config.b_order_by_group:    
+        if self.session.config.b_order_by_group:
             group_src = self.get_contact_selected_group()
 
             try:
-                pos = widget.get_dest_row_at_pos(x,y)[0][0]
+                pos = widget.get_dest_row_at_pos(x, y)[0][0]
             except TypeError: # yes, this can happen if you drag into a blank area
                 return True
             group_des = self.model[pos][1]
@@ -889,7 +931,20 @@ class ContactList(gui.ContactList, gtk.TreeView):
             if group_src and group_src != group_des and not self._model[pos][6]:
 
                 self.session.move_to_group(self.get_contact_selected().account,
-                                           group_src.identifier, 
+                                           group_src.identifier,
                                            group_des.identifier)
 
         return True
+
+    def select_top_contact(self):
+        ''' Select the topmost contact '''
+        selected = self.get_selection()
+        for row in self.model:
+            obj = row[1]
+            if isinstance(obj, e3.Group):
+                for contact_row in row.iterchildren():
+                    selected.select_path(contact_row.path)
+                    return
+            elif isinstance(obj, e3.Contact):
+                selected.select_path(row.path)
+                return
