@@ -25,6 +25,7 @@ import urllib
 import stock
 import utils
 import gui
+from gui.gtkui import check_gtk3
 import extension
 import e3
 
@@ -50,7 +51,9 @@ class Conversation(gtk.VBox, gui.Conversation):
 
         self.panel = gtk.VPaned()
 
-        self.show_avatar_in_taskbar = self.session.config.get_or_set('b_show_avatar_in_taskbar', True)
+        self.show_avatar_in_taskbar = self.session.config.get_or_set(
+                                                    'b_show_avatar_in_taskbar',
+                                                    True)
 
         Header = extension.get_default('conversation header')
         OutputText = extension.get_default('conversation output')
@@ -98,8 +101,12 @@ class Conversation(gtk.VBox, gui.Conversation):
         self.panel.pack1(self.output, True, False)
         self.panel.pack2(frame_input, False, False)
 
-        self.panel_signal_id = self.panel.connect_after('expose-event',
-                self.update_panel_position)
+        if not check_gtk3():
+            self.panel_signal_id = self.panel.connect_after('expose-event',
+                    self.update_panel_position)
+        else:
+            self.panel_signal_id = self.panel.connect_after('draw',
+                    self.update_panel_position)
         self.panel.connect('button-release-event', self.on_input_panel_resize)
 
         self.hbox = gtk.HBox()
@@ -121,25 +128,53 @@ class Conversation(gtk.VBox, gui.Conversation):
         else:
             #update adium theme header/footer
             account = self.members[0]
-            contact = self.session.contacts.get(account)
-            if contact:
-                his_picture = contact.picture or utils.path_to_url(os.path.abspath(gui.theme.image_theme.user))
-                nick = contact.nick
-                display_name = contact.display_name
-                self.set_sensitive(not contact.blocked, True)
-            else:
-                his_picture = utils.path_to_url(os.path.abspath(gui.theme.image_theme.user))
-                nick = ""
-                display_name = ""
+            contact = self.session.contacts.safe_get(account)
+            his_picture = contact.picture or utils.path_to_url(os.path.abspath(gui.theme.image_theme.user))
+            nick = contact.nick
+            display_name = contact.display_name
+            self.set_sensitive(not contact.blocked, True)
             my_picture = self.session.contacts.me.picture or utils.path_to_url(os.path.abspath(gui.theme.image_theme.user))
             self.output.clear(account, nick, display_name, 
                               my_picture, his_picture)
 
         self._load_style()
         self.subscribe_signals()
+        extension.subscribe(self.on_below_conversation_changed, 'below conversation')
+        extension.subscribe(self.on_conversation_info_extension_changed, 'conversation info')
 
         self.tab_index = -1 # used to select an existing conversation
 
+    def on_conversation_info_extension_changed(self, new_extension):
+        if type(self.info) != new_extension:
+            self.hbox.remove(self.info)
+            self.info = None
+
+            if new_extension:
+                self.info = new_extension(self.session, self.members)
+                if self.session.config.get_or_set('b_avatar_on_left', False):
+                    self.hbox.pack_start(self.info, False)
+                else:
+                    self.hbox.pack_end(self.info, False)
+
+            if self.session.config.b_show_info:
+                self.info.show()
+
+    def on_below_conversation_changed(self, newvalue):
+        if type(self.below_conversation) != newvalue:
+            #replace the below conversation
+            if not self.below_conversation is None:
+                self.remove(self.below_conversation)
+                self.below_conversation = None
+
+            if newvalue:
+                self.below_conversation = newvalue(self, self.session)
+                self.pack_end(self.below_conversation, False)
+                self.below_conversation.show()
+
+
+    def check_visible(self):
+        return self.get_visible()
+
     def steal_emoticon(self, path_uri):
         '''receives the path or the uri for the emoticon to be added'''
         if path_uri.startswith("file://"):
@@ -180,43 +215,35 @@ class Conversation(gtk.VBox, gui.Conversation):
 
     def _on_show_toolbar_changed(self, value):
         '''callback called when config.b_show_toolbar changes'''
-        if value:
-            self.toolbar.show()
-        else:
-            self.toolbar.hide()
+        self.toolbar.set_visible(value)
 
     def _on_show_header_changed(self, value):
         '''callback called when config.b_show_header changes'''
-        if value:
-            self.header.show()
-        else:
-            self.header.hide()
+        self.header.set_visible(value)
 
     def _on_show_info_changed(self, value):
         '''callback called when config.b_show_info changes'''
-        if value:
-            self.info.show()
-        else:
-            self.info.hide()
+        self.info.set_visible(value)
+        self.toolbar.update_toggle_avatar_icon(value)
 
     def _on_show_avatar_onleft(self, value):
         '''callback called when config.b_avatar_on_left changes'''
-        if value:
-            self.hbox.reorder_child(self.panel, 1)
-            self.hbox.reorder_child(self.info, 0)
-        else:
-            self.hbox.reorder_child(self.panel, 0)
-            self.hbox.reorder_child(self.info, 1)
+        self.hbox.reorder_child(self.panel, value)
+        self.hbox.reorder_child(self.info, not value)
 
     def on_close(self):
         '''called when the conversation is closed'''
         self.unsubscribe_signals()
+        extension.unsubscribe(self.on_below_conversation_changed, 'below conversation')
+        extension.unsubscribe(self.on_conversation_info_extension_changed, 'conversation info')
+        if self.below_conversation:
+            self.below_conversation = None
 
         self.destroy()
         self.info.destroy()
         self.header.destroy()
 
-    def show(self, other_started=False):
+    def show(self):
         '''override the show method'''
         gtk.VBox.show(self)
 
@@ -230,9 +257,6 @@ class Conversation(gtk.VBox, gui.Conversation):
         self.hbox.show()
         self.panel.show_all()
 
-        if not other_started:
-            self.input_grab_focus()
-
         if not self.session.config.b_show_toolbar:
             self.toolbar.hide()
 
@@ -300,6 +324,10 @@ class Conversation(gtk.VBox, gui.Conversation):
         """
         update the information for a conversation with multiple users
         """
+        if self.session.account.account in self.members:
+            # this can happen sometimes (e.g. when you're invisible. see #1297)
+            self.members.remove(self.session.account.account)
+
         #TODO add plus support for nick to the tab label!
         members_nick = []
         i = 0
@@ -332,13 +360,17 @@ class Conversation(gtk.VBox, gui.Conversation):
         self.info.set_sensitive(is_sensitive or force_sensitive_block_button)
         self.toolbar.set_sensitive(is_sensitive, force_sensitive_block_button)
 
+        # redraws block button to its corresponding icon and tooltip
+        contact_unblocked = not force_sensitive_block_button or is_sensitive
+        self.toolbar.redraw_ublock_button(contact_unblocked)
+
     def set_image_visible(self, is_visible):
         """
         set the visibility of the widget that displays the images of the members
 
         is_visible -- boolean that says if the widget should be shown or hidden
         """
-        self.info.show() if is_visible else self.info.hide()
+        self.info.set_visible(is_visible)
 
     def set_header_visible(self, is_visible):
         '''
@@ -346,7 +378,7 @@ class Conversation(gtk.VBox, gui.Conversation):
 
         is_visible -- boolean that says if the widget should be shown or hidden
         '''
-        self.header.show() if is_visible else self.header.hide()
+        self.header.set_visible(is_visible)
 
     def set_toolbar_visible(self, is_visible):
         '''
@@ -354,27 +386,11 @@ class Conversation(gtk.VBox, gui.Conversation):
 
         is_visible -- boolean that says if the widget should be shown or hidden
         '''
-        self.toolbar.show() if is_visible else self.toolbar.hide()
+        self.toolbar.set_visible(is_visible)
 
     def get_preview(self, completepath):
         return utils.makePreview(completepath)
 
-    def _on_block_user_accept(self, accept, account):
-        if accept == stock.YES:
-            self.session.block(account)
-
-    def on_block_user(self):
-        '''blocks the first user of the conversation'''
-        account = self.members[0]
-        contact = self.session.contacts.contacts[account]
-
-        if contact.blocked:
-            self.session.unblock(account)
-        else:
-            dialog = extension.get_default('dialog')
-            dialog.yes_no(_("Are you sure you want to block this contact?"),
-                          self._on_block_user_accept, account)
-
     def on_user_typing(self, account):
         """
         inform that the other user has started typing
@@ -397,24 +413,6 @@ class Conversation(gtk.VBox, gui.Conversation):
         elif account in self.members:
             self.info.his_avatar.set_from_file(path)
 
-    def on_toggle_avatar(self):
-        '''hide or show the avatar bar'''
-        if not self.session.config.b_show_info:
-            self.toolbar.toggle_avatar.set_tooltip_text(_('Hide avatar'))
-        else:
-            self.toolbar.toggle_avatar.set_tooltip_text(_('Show avatar'))
-            
-        self.info.set_visible(not self.info.get_visible())
-        self.toolbar.toggle_avatar.set_stock_id(self.get_toggle_avatar_icon())
-        self.session.config.b_show_info = not self.session.config.b_show_info
-
-    def get_toggle_avatar_icon(self):
-        current_stock_id = self.toolbar.toggle_avatar.get_stock_id()
-        if current_stock_id == gtk.STOCK_GO_FORWARD:
-            return gtk.STOCK_GO_BACK
-        else:
-            return gtk.STOCK_GO_FORWARD
-
     def _on_avatarsize_changed(self, value):
         self.info._on_avatarsize_changed(value)
 
