@@ -20,8 +20,10 @@ import gtk
 
 import e3
 import gui
+from gui.gtkui import check_gtk3
 import utils
 
+
 class ConversationToolbar(gtk.Toolbar):
     """
     A class that represents the toolbar on the conversation window
@@ -46,6 +48,44 @@ class ConversationToolbar(gtk.Toolbar):
 
         self.draw()
 
+    def redraw_ublock_button(self, contact_unblocked):
+        """
+        redraws the block button,
+        setting the appropiated stock_id and tooltip
+        """
+        if contact_unblocked:
+            ublock_icon = self.theme_tool_block
+            tooltip_text = _('Block contact')
+        else:
+            ublock_icon = self.theme_tool_unblock
+            tooltip_text = _('Unblock contact')
+
+        self.ublock.set_tooltip_text(tooltip_text)
+
+        if isinstance(ublock_icon, gtk.Widget):
+            self.ublock.set_icon_widget(ublock_icon)
+        else:
+            self.ublock.set_stock_id(ublock_icon)
+
+    def update_toggle_avatar_icon(self, toggle_avatar):
+        """
+        redraws the toggle avatar button,
+        setting the appropiated stock_id and tooltip
+        """
+        if toggle_avatar:
+            tooltip_text = _('Hide avatar')
+            toggle_avatar_icon = self.theme_tool_hide_avatar
+        else:
+            tooltip_text =_('Show avatar')
+            toggle_avatar_icon = self.theme_tool_show_avatar
+
+        self.toggle_avatar.set_tooltip_text(tooltip_text)
+
+        if isinstance(toggle_avatar_icon, gtk.Widget):
+            self.toggle_avatar.set_icon_widget(toggle_avatar_icon)
+        else:
+            self.toggle_avatar.set_stock_id(toggle_avatar_icon)
+
     def set_sensitive(self, is_sensitive, force_sensitive_block_button=False):
         self.ublock.set_sensitive(force_sensitive_block_button or is_sensitive)
         self.toggle_avatar.set_sensitive(force_sensitive_block_button or is_sensitive)
@@ -60,10 +100,11 @@ class ConversationToolbar(gtk.Toolbar):
         self.invite_audio_call.set_sensitive(is_sensitive)
         self.invite_file_transfer.set_sensitive(is_sensitive)
 
-
     def draw(self):
         '''draw the toolbar'''
-        toolbar_small = self.handler.session.config.get_or_set('b_toolbar_small', False)
+        toolbar_small = self.handler.session.config.get_or_set(
+                                                'b_toolbar_small',
+                                                False)
 
         if toolbar_small:
             size = gtk.ICON_SIZE_MENU
@@ -71,8 +112,8 @@ class ConversationToolbar(gtk.Toolbar):
             size = gtk.ICON_SIZE_LARGE_TOOLBAR
 
         self.settings = self.get_settings()
-        self.settings.set_long_property('gtk-toolbar-icon-size', size, \
-            'ConversationToolbar.py:37')
+        self.settings.set_long_property('gtk-toolbar-icon-size', size,
+                                        'ConversationToolbar.py:37')
 
         # check if we have theme-specific toolbar-icons
 
@@ -85,7 +126,8 @@ class ConversationToolbar(gtk.Toolbar):
             theme_tool_invite = utils.gtk_ico_image_load(image_theme.tool_invite, size)
             theme_tool_clean = utils.gtk_ico_image_load(image_theme.tool_clean, size)
             theme_tool_file_transfer = utils.gtk_ico_image_load(image_theme.tool_file_transfer, size)
-            theme_tool_ublock = utils.gtk_ico_image_load(image_theme.tool_ublock, size)
+            self.theme_tool_block = utils.gtk_ico_image_load(image_theme.tool_block, size)
+            self.theme_tool_unblock = utils.gtk_ico_image_load(image_theme.tool_unblock, size)
         else:
             theme_tool_font = gtk.STOCK_SELECT_FONT
             theme_tool_font_color = gtk.STOCK_SELECT_COLOR
@@ -97,12 +139,15 @@ class ConversationToolbar(gtk.Toolbar):
             theme_tool_invite = gtk.STOCK_ADD
             theme_tool_clean = gtk.STOCK_CLEAR
             theme_tool_file_transfer = gtk.STOCK_GO_UP
-            theme_tool_ublock = gtk.STOCK_STOP
+            self.theme_tool_block = gtk.STOCK_STOP
+            self.theme_tool_unblock = gtk.STOCK_YES
 
-        if self.session.config.b_avatar_on_left == self.session.config.b_show_info:
-            theme_tool_toggle_avatar = gtk.STOCK_GO_BACK
+        if self.session.config.b_avatar_on_left:
+            self.theme_tool_hide_avatar = gtk.STOCK_GO_BACK
+            self.theme_tool_show_avatar = gtk.STOCK_GO_FORWARD
         else:
-            theme_tool_toggle_avatar = gtk.STOCK_GO_FORWARD
+            self.theme_tool_hide_avatar = gtk.STOCK_GO_FORWARD
+            self.theme_tool_show_avatar = gtk.STOCK_GO_BACK
 
         theme_tool_call = utils.gtk_ico_image_load(image_theme.call, size)
         theme_tool_video = utils.gtk_ico_image_load(image_theme.video, size)
@@ -120,13 +165,15 @@ class ConversationToolbar(gtk.Toolbar):
         self.color.connect('clicked',
             lambda *args: self.handler.on_color_selected())
 
-        self.emotes = gtk.ToolButton(theme_tool_emotes)
+        self.emotes = gtk.ToolButton()
+        self.emotes.set_icon_widget(theme_tool_emotes)
         self.emotes.set_label(_('Send an emoticon'))
         self.emotes.set_tooltip_text(_('Send an emoticon'))
         self.emotes.connect('clicked',
             lambda *args: self.handler.on_emotes_selected())
 
-        self.nudge = gtk.ToolButton(theme_tool_nudge)
+        self.nudge = gtk.ToolButton()
+        self.nudge.set_icon_widget(theme_tool_nudge)
         self.nudge.set_label(_('Request attention'))
         self.nudge.set_tooltip_text(_('Request attention'))
         self.nudge.connect('clicked',
@@ -150,18 +197,15 @@ class ConversationToolbar(gtk.Toolbar):
         self.invite_file_transfer.connect('clicked',
             lambda *args: self.handler.on_invite_file_transfer_selected())
 
-        self.ublock = gtk.ToolButton(theme_tool_ublock)
+        self.ublock = gtk.ToolButton(self.theme_tool_block)
         self.ublock.set_label(_('Block/Unblock contact'))
         self.ublock.set_tooltip_text(_('Block/Unblock contact'))
         self.ublock.connect('clicked',
             lambda *args: self.handler.on_ublock_selected())
 
-        self.toggle_avatar = gtk.ToolButton(theme_tool_toggle_avatar)
+        self.toggle_avatar = gtk.ToolButton(self.theme_tool_show_avatar)
         self.toggle_avatar.set_label(_('Hide/Show avatar'))
-        if self.handler.session.config.b_show_info:
-            self.toggle_avatar.set_tooltip_text(_('Hide avatar'))
-        else:
-            self.toggle_avatar.set_tooltip_text(_('Show avatar'))
+        self.update_toggle_avatar_icon(self.session.config.b_show_info)
         self.toggle_avatar.connect('clicked',
             lambda *args: self.handler.on_toggle_avatar_selected())
 
@@ -208,4 +252,3 @@ class ConversationToolbar(gtk.Toolbar):
             self.add(self.ublock)
         self.add(gtk.SeparatorToolItem())
         self.add(self.toggle_avatar)
-
