@@ -28,8 +28,16 @@ import extension
 import e3
 import gobject
 import gui
+import Info
 
-from pluginmanager import get_pluginmanager
+from gui.gtkui import check_gtk3
+
+if check_gtk3():
+    import TinyButtonNew as TinyButton
+else:
+    import TinyButton
+
+from e3.common.pluginmanager import get_pluginmanager
 
 class ExtensionListView(gtk.TreeView):
     def __init__(self, store, radio=False):
@@ -129,8 +137,39 @@ class ExtensionListTab(gtk.VBox):
         self.buttonbox.set_layout(gtk.BUTTONBOX_EDGE)
         self.buttonbox.set_border_width(2)
 
+        #progressbar widgets
+        self.progressbox = gtk.VBox()
+        self.progressbox.set_border_width(2)
+        box = gtk.HBox()
+        box.set_border_width(2)
+        self.progresslabel = gtk.Label()
+        self.progressbar = gtk.ProgressBar()
+        self.progress_cancel_button = TinyButton.TinyButton(gtk.STOCK_CANCEL)
+        self.progress_cancel_button.set_tooltip_text(_('Cancel'))
+        self.end_progress_handler = None
+
+        box.pack_start(self.progresslabel, False, False, 0)
+        box.pack_start(self.progressbar, False, False, 5)
+        box.pack_start(self.progress_cancel_button, False, False, 0)
+
+        separator = gtk.HSeparator()
+        self.progressbox.pack_start(separator, False, False, 0)
+        self.progressbox.pack_start(box, False, False, 2)
+
         self.pack_start(self.lists)
         self.pack_start(self.buttonbox, False)
+        self.pack_start(self.progressbox, False)
+
+    def set_action(self, action):
+        '''called when the action changes'''
+        self.progresslabel.set_text(action)
+        self.progressbar.set_fraction(0)
+        self.progressbar.set_text("0 %")
+
+    def update_action(self, progress):
+        '''called when the progress is updated'''
+        self.progressbar.set_fraction(progress / 100.0)
+        self.progressbar.set_text("%d %s" % (progress, "%"))
 
     def _make_list(self, type_, radio):
         '''make a new extension list'''
@@ -209,6 +248,9 @@ class DownloadListBase(ExtensionListTab):
         '''constructor'''
         ExtensionListTab.__init__(self, session, radio, use_tabs)
         self.updated_amount = 0
+        self.default_action = _('Refresh extensions')
+        self.progress = False
+
 
         self.config_dir = e3.common.ConfigDir('emesene2')
 
@@ -238,12 +280,14 @@ class DownloadListBase(ExtensionListTab):
 
     def on_update(self, widget=None, refresh=False):
         '''called when the liststore need to be changed'''
-        dialog = extension.get_default('dialog')
-        self.progress = dialog.progress_window(
-                        _('Refresh extensions'), self._end_progress_cb)
-        self.progress.set_action(_("Refreshing extensions"))
-        self.progress.show_all()
-        gobject.timeout_add(100, self.update_progress)
+        if self.progress:
+            return
+
+        self.progress = True
+        self.set_action(self.default_action)
+        self.progressbox.show_all()
+        self.end_progress_handler = self.progress_cancel_button.connect('clicked', self._end_progress_cb)
+        gobject.timeout_add_seconds(1, self.update_progress)
         utils.GtkRunner(self.show_update_callback, self.update, refresh)
 
     def update(self, refresh):
@@ -270,7 +314,7 @@ class DownloadListBase(ExtensionListTab):
             for current_collection in collection.itervalues():
                 progress += current_collection.progress / (len(self.list_types) * 2.0)
 
-        self.progress.update(100.0 * progress)
+        self.update_action(100.0 * progress)
 
         return True
 
@@ -279,11 +323,13 @@ class DownloadListBase(ExtensionListTab):
         need the first argument on the on_update method'''
         if result is not None and not result[0]:
             log.error(str(result[1]))
-        if self.progress is None:
+        if not self.progress:
             return
 
-        self.progress.destroy()
-        self.progress = None
+        #hide progressbox
+        self.progressbox.hide()
+        self.progress_cancel_button.disconnect(self.end_progress_handler)
+        self.progress = False
         self.show_update()
 
     def _end_progress_cb(self, *args):
@@ -326,7 +372,9 @@ class DownloadList(DownloadListBase):
         combo_store.append((_('Supported'), 'supported'))
         combo_store.append((_('Community'), 'community'))
 
-        source_combo = gtk.ComboBox(combo_store)
+        source_combo = gtk.ComboBox()
+        source_combo.set_model(combo_store)
+
         cell = gtk.CellRendererText()
         source_combo.pack_start(cell, True)
         source_combo.add_attribute(cell, 'text', 0)
@@ -364,12 +412,13 @@ class DownloadList(DownloadListBase):
 
     def get_selected_name(self, list_view):
         '''Gets the current selected name in the list view.'''
-        model, iter = list_view.get_selection().get_selected()
-        if iter is not None:
-            name = model.get_value(iter, 2)
-            return name
-        else:
-            return None
+        selection = list_view.get_selection()
+        if not selection is None:
+            model, iter = selection.get_selected()
+            if iter is not None:
+                name = model.get_value(iter, 2)
+                return name
+        return None
 
     def on_change_source(self, combobox):
         '''called when the source is changed'''
@@ -380,12 +429,16 @@ class DownloadList(DownloadListBase):
 
     def start_download(self, widget=None):
         '''start the download of an extension'''
-        dialog = extension.get_default('dialog')
-        self.progress = dialog.progress_window(
-                        _('Downloading extensions'), self._end_progress_cb)
-        self.progress.set_action(_("Downloading extensions"))
-        self.progress.show_all()
-        gobject.timeout_add(100, self.update_progress)
+        if self.progress:
+            return
+
+        self.progress = True
+        self.set_action(_('Downloading extensions'))
+        self.progressbox.show_all()
+        self.progress_cancel_button.connect('clicked', self._end_progress_cb)
+        self.end_progress_handler = self.progress_cancel_button.connect('clicked',
+                                        self._end_progress_cb)
+        gobject.timeout_add_seconds(1, self.update_progress)
         utils.GtkRunner(self.show_update_callback, self.download)
 
     def download(self):
@@ -411,7 +464,7 @@ class DownloadList(DownloadListBase):
             return True
 
         if self.version_value(meta.get('required emesene version')) > \
-           self.version_value(gui.base.EMESENE_VERSION):
+           self.version_value(Info.EMESENE_VERSION):
             return False
 
         return True
@@ -445,6 +498,8 @@ class ThemeList(DownloadList):
             self, session, 'themes',
             self.config_dir.join('themes'), True, True)
 
+        self.default_action = _('Refresh themes')
+
         self.themes = {}
         self.theme_configs = {}
         self.callbacks = {}
@@ -548,12 +603,15 @@ class UpdateList(DownloadListBase):
 
     def start_download(self, widget=None):
         '''start the download of an extension'''
-        dialog = extension.get_default('dialog')
-        self.progress = dialog.progress_window(
-                        _('Updating extensions'), self._end_progress_cb)
-        self.progress.set_action(_("Updating extensions"))
-        self.progress.show_all()
-        gobject.timeout_add(100, self.update_download_progress)
+        if self.progress:
+            return
+
+        self.progress = True
+        self.set_action(_('Updating extensions'))
+        self.progressbox.show_all()
+        self.end_progress_handler = self.progress_cancel_button.connect('clicked',
+                                        self._end_progress_cb)
+        gobject.timeout_add_seconds(1, self.update_download_progress)
         utils.GtkRunner(self.show_update_callback, self.download_updates)
 
     def download_updates(self):
@@ -565,7 +623,6 @@ class UpdateList(DownloadListBase):
                         return
 
                     current_extension = self.get_collection(collection_name, ext_type, ext)
-                    self.remove(path, current_extension)
                     self.download(ext, current_extension)
                     self.updated_amount += 1
                 ext_list = {}
@@ -583,7 +640,7 @@ class UpdateList(DownloadListBase):
                 for current_collection in collection.itervalues():
                     progress += current_collection.progress / float(self.update_amount)
 
-        self.progress.update(100.0 * progress)
+        self.update_action(100.0 * progress)
 
         return True
 
@@ -605,7 +662,7 @@ class UpdateList(DownloadListBase):
             return True
 
         if self.version_value(meta.get('required emesene version')) > \
-           self.version_value(gui.base.EMESENE_VERSION):
+           self.version_value(Info.EMESENE_VERSION):
             return False
 
         if not local_meta.get('version'):
