@@ -18,7 +18,12 @@
 
 import gtk
 
-import Renderers
+from gui.gtkui import check_gtk3
+
+if check_gtk3():
+    import RenderersNew as Renderers
+else:
+    import Renderers
 
 class Header(gtk.HBox):
     '''a widget used to display some information about the conversation'''
@@ -66,7 +71,7 @@ class Header(gtk.HBox):
     def _set_information(self, lines):
         '''set the text on the information, lines is a tuple of size 3 with 3
         strings that will be replaced on the template'''
-        self._information.set_markup(Renderers.msnplus_to_list(Header.INFO_TPL % lines))
+        self._information.set_markup(Header.INFO_TPL % lines)
         
     def _get_information(self):
         '''return the text on the information'''
@@ -75,7 +80,7 @@ class Header(gtk.HBox):
     def copy_nick(self, data, widget=None):
         nick_list = []
         for member in self.members:
-            contact = self.session.contacts.get(member)
+            contact = self.session.contacts.safe_get(member)
             nick_list.append(contact.nick)
 
         clipboard = gtk.clipboard_get(gtk.gdk.SELECTION_CLIPBOARD)
@@ -84,7 +89,7 @@ class Header(gtk.HBox):
     def copy_pm(self, data, widget=None):
         pm_list = []
         for member in self.members:
-            contact = self.session.contacts.get(member)
+            contact = self.session.contacts.safe_get(member)
             pm_list.append(contact.message)
 
         clipboard = gtk.clipboard_get(gtk.gdk.SELECTION_CLIPBOARD)
