@@ -59,8 +59,8 @@ class LoginBaseUI(gtk.Alignment):
             self._on_account_key_press)
         self.cmb_account.connect('changed',
             self._on_account_changed)
-        self.cmb_account.connect('key-release-event',
-            self._on_account_key_release)
+        self.acc_key_rel_handler = self.cmb_account.connect('key-release-event',
+                                                   self._on_account_key_release)
 
         self.btn_status = StatusButton.StatusButton()
         self.btn_status.set_tooltip_text(_('Status'))
@@ -288,8 +288,21 @@ class Login(LoginBaseUI, gui.LoginBase):
             self.cmb_account.get_children()[0].set_text(
                 account.rpartition('|')[0])
 
-        if not self.cancel_clicked:
-            self._check_autologin()
+    def check_autologin(self):
+        '''check if autologin is set and can be started'''
+        if self.cancel_clicked:
+            return
+
+        account = self.config.get_or_set('last_logged_account', '')
+
+        if account != '' and int(self.config.d_remembers.get(account, 0)) == 3:
+            password = self.decode_password(account)
+
+            self.cmb_account.get_children()[0].set_text(account.rpartition('|')[0])
+            self.txt_password.set_text(password)
+
+            if not self.no_autologin:
+                self.do_connect()
 
     def _get_active_service(self):
         '''fetch the active service from the session combo box'''
@@ -351,19 +364,6 @@ class Login(LoginBaseUI, gui.LoginBase):
 
         self._combo_session_list.append(self.session_combo)
 
-    def _check_autologin(self):
-        '''check if autologin is set and can be started'''
-        account = self.config.get_or_set('last_logged_account', '')
-
-        if account != '' and int(self.config.d_remembers.get(account, 0)) == 3:
-            password = self.decode_password(account)
-
-            self.cmb_account.get_children()[0].set_text(account.rpartition('|')[0])
-            self.txt_password.set_text(password)
-
-            if not self.no_autologin:
-                self.do_connect()
-
     def do_connect(self):
         '''
         do all the stuffs needed to connect
@@ -430,6 +430,8 @@ class Login(LoginBaseUI, gui.LoginBase):
             service = self._get_active_service()
 
         self.update_service(service)
+        ext_id, ext = self.service2id[service]
+        self.btn_status.set_sensitive(ext.SERVICE_STATUS in ext.CAPABILITIES)
 
         if account + '|' + service in self.accounts:
             account_and_service = account + '|' + service
@@ -437,7 +439,6 @@ class Login(LoginBaseUI, gui.LoginBase):
             self.remember_account.set_sensitive(False)
             self.forget_me.set_sensitive(True)
             self.btn_status.set_status(int(self.status[account_and_service]))
-            self.config.d_user_service[account] = service
 
             passw = self.decode_password(account_and_service)
             avatar_path = self.current_avatar_path(account)
@@ -483,10 +484,12 @@ class Login(LoginBaseUI, gui.LoginBase):
         self._clear_all()
         self.cmb_account.get_children()[0].set_text('')
 
-    def show_error(self, reason):
+    def show_error(self, reason, login_failed=False):
         '''
         show an error on the top of the window using nicebar
         '''
+        if login_failed:
+            self.auto_login.set_active(False)
         self.nicebar.new_message(reason, gtk.STOCK_DIALOG_ERROR)
 
     def _reload_account_list(self, *args):
@@ -544,7 +547,7 @@ class Login(LoginBaseUI, gui.LoginBase):
             if response == stock.YES:
                 account = self.cmb_account.get_active_text()
                 service = self._get_active_service()
-                self.forgive_user(account, service)
+                self.forget_user(account, service)
                 self._reload_account_list()
                 self.cmb_account.get_children()[0].set_text('')
 
@@ -668,6 +671,9 @@ class ConnectingWindow(Login):
 
         #for reconnecting
         self.reconnect_timer_id = None
+        
+        # Disconnect inherited and unneeded event
+        self.cmb_account.disconnect(self.acc_key_rel_handler)
 
         account = config.get_or_set('last_logged_account', '')
         remembers = config.get_or_set('d_remembers', {})
@@ -780,4 +786,3 @@ class ConnectingWindow(Login):
             return False
         else:
             return True
-
