@@ -19,7 +19,9 @@
 import gtk
 import e3
 import gui
+from gui.gtkui import check_gtk3
 import utils
+import sys
 
 import extension
 
@@ -68,6 +70,64 @@ class MainMenu(gtk.MenuBar):
         self.append(self.options)
         self.append(self.help)
 
+    def set_accels(self, accel_group):
+        """
+        Set accelerators for menu items
+        """
+        if sys.platform == 'darwin':
+            self.file_menu.quit.add_accelerator(
+                    'activate', accel_group, gtk.keysyms.Q,
+                    gtk.gdk.META_MASK, gtk.ACCEL_VISIBLE)
+            self.file_menu.disconnect.add_accelerator(
+                    'activate', accel_group, gtk.keysyms.D,
+                    gtk.gdk.META_MASK, gtk.ACCEL_VISIBLE)
+        else:
+            self.file_menu.quit.add_accelerator(
+                    'activate', accel_group, gtk.keysyms.Q,
+                    gtk.gdk.CONTROL_MASK, gtk.ACCEL_VISIBLE)
+            self.file_menu.disconnect.add_accelerator(
+                    'activate', accel_group, gtk.keysyms.D,
+                    gtk.gdk.CONTROL_MASK, gtk.ACCEL_VISIBLE)
+
+class EndPointsMenu(gtk.Menu):
+    """
+    A widget that contains all the endpoints
+    """
+    def __init__(self, handler, session):
+        """
+        constructor
+
+        handler -- e3common.Handler.FileHandler
+        """
+        gtk.Menu.__init__(self)
+        self.handler = handler
+        self.session = session
+
+        self.ep_dict = {}
+        name = 'All other endpoints'
+        ep_item = gtk.MenuItem(_(name))
+        ep_item.connect('activate',
+                lambda *args : self.handler.on_disconnect_endpoint_selected(""))
+        self.append(ep_item)
+        self.append(gtk.SeparatorMenuItem())
+
+        self.session.signals.endpoint_added.subscribe(self.endpoint_added)
+        self.session.signals.endpoint_removed.subscribe(self.endpoint_removed)
+
+    def endpoint_added(self, ep_id, ep_name):
+        self.endpoint_removed(ep_id)
+        ep_item = gtk.MenuItem(ep_name)
+        ep_item.connect('activate',
+            lambda *args: self.handler.on_disconnect_endpoint_selected(ep_id))
+        ep_item.show()
+        self.append(ep_item)
+        self.ep_dict[ep_id] = ep_item
+
+    def endpoint_removed(self, ep_id):
+        if ep_id in self.ep_dict:
+            self.ep_dict[ep_id].hide()
+            del self.ep_dict[ep_id]
+
 class FileMenu(gtk.Menu):
     """
     A widget that represents the File popup menu located on the main menu
@@ -91,6 +151,12 @@ class FileMenu(gtk.Menu):
             self.status.set_submenu(self.status_menu)
             self.append(self.status)
 
+        if session and session.session_has_service(e3.Session.SERVICE_ENDPOINTS):
+            self.ep = gtk.MenuItem(_('Disconnect endpoints'))
+            self.ep_menu = EndPointsMenu(handler, session)
+            self.ep.set_submenu(self.ep_menu)
+            self.append(self.ep)
+
         self.disconnect = gtk.ImageMenuItem(gtk.STOCK_DISCONNECT)
         self.disconnect.connect('activate',
             lambda *args: self.handler.on_disconnect_selected())
@@ -117,17 +183,12 @@ class ActionsMenu(gtk.Menu):
         self.handler = handler
 
         ContactsMenu = extension.get_default('menu contact')
-        GroupsMenu = extension.get_default('menu group')
         AccountMenu = extension.get_default('menu account')
 
         self.contact = gtk.ImageMenuItem(_('_Contact'))
         self.contact.set_image(utils.safe_gtk_image_load(gui.theme.image_theme.chat))
         self.contact_menu = ContactsMenu(self.handler.contact_handler, session)
         self.contact.set_submenu(self.contact_menu)
-        self.group = gtk.ImageMenuItem(_('_Group'))
-        self.group.set_image(utils.safe_gtk_image_load(gui.theme.image_theme.group_chat))
-        self.group_menu = GroupsMenu(self.handler.group_handler, session)
-        self.group.set_submenu(self.group_menu)
         self.account = gtk.ImageMenuItem(_('_Account'))
         self.account.set_image(utils.safe_gtk_image_load(gui.theme.image_theme.chat))
 
@@ -137,7 +198,15 @@ class ActionsMenu(gtk.Menu):
         self.myaccount.set_submenu(self.account_menu)
 
         self.append(self.contact)
-        self.append(self.group)
+
+        if session.session_has_service(e3.Session.SERVICE_GROUP_MANAGING):
+            GroupsMenu = extension.get_default('menu group')
+            self.group = gtk.ImageMenuItem(_('_Group'))
+            self.group.set_image(utils.safe_gtk_image_load(gui.theme.image_theme.group_chat))
+            self.group_menu = GroupsMenu(self.handler.group_handler)
+            self.group.set_submenu(self.group_menu)
+            self.append(self.group)
+
         self.append(self.myaccount)
 
 class OptionsMenu(gtk.Menu):
@@ -154,8 +223,13 @@ class OptionsMenu(gtk.Menu):
         gtk.Menu.__init__(self)
         self.handler = handler
 
-        self.by_status = gtk.RadioMenuItem(None, _('Order by _status'))
-        self.by_group = gtk.RadioMenuItem(self.by_status, _('Order by _group'))
+        if not check_gtk3():
+            self.by_status = gtk.RadioMenuItem(None, _('Order by _status'))
+            self.by_group = gtk.RadioMenuItem(self.by_status, _('Order by _group'))
+        else:
+            self.by_status = gtk.RadioMenuItem(_('Order by _status'))
+            self.by_status.set_use_underline(True)
+            self.by_group = gtk.RadioMenuItem.new_with_mnemonic_from_widget(self.by_status, _('Order by _group'))
         self.by_group.set_active(config.b_order_by_group)
         self.by_status.set_active(not config.b_order_by_group)
 
@@ -250,4 +324,3 @@ class HelpMenu(gtk.Menu):
         self.append(self.debug)
         self.append(gtk.SeparatorMenuItem())
         self.append(self.updatecheck)
-
