@@ -20,10 +20,12 @@ import gtk
 
 import e3.common
 import gui
+from gui.gtkui import check_gtk3
 import subprocess
 import sys
 import extension
 import stock
+from Language import Language
 
 from gui.base import MarkupParser
 
@@ -41,17 +43,7 @@ except:
         return []
 
 # TODO: consider moving to nicer icons than stock ones.
-LIST = [
-    {'stock_id' : gtk.STOCK_PAGE_SETUP,'text' : _('Main Window')},
-    {'stock_id' : gtk.STOCK_PAGE_SETUP,'text' : _('Conversation Window')},
-    {'stock_id' : gtk.STOCK_FLOPPY,'text' : _('Desktop')},
-    {'stock_id' : gtk.STOCK_MEDIA_PLAY,'text' : _('Sounds')},
-    {'stock_id' : gtk.STOCK_LEAVE_FULLSCREEN,'text' : _('Notifications')},
-    {'stock_id' : gtk.STOCK_SELECT_COLOR,'text' : _('Theme')},
-    {'stock_id' : gtk.STOCK_EXECUTE,'text' : _('Extensions')},
-    {'stock_id' : gtk.STOCK_DISCONNECT,'text' : _('Plugins')},
-    {'stock_id' : gtk.STOCK_REFRESH,'text' : _('Updates')},
-]
+
 
 class Preferences(gtk.Window):
     """A window to display/modify the preferences
@@ -65,6 +57,18 @@ class Preferences(gtk.Window):
         self.set_title(_("Preferences"))
         self.session = session
 
+        self.LIST = [
+            {'stock_id' : gtk.STOCK_PAGE_SETUP,'text' : _('Main Window')},
+            {'stock_id' : gtk.STOCK_PAGE_SETUP,'text' : _('Conversation Window')},
+            {'stock_id' : gtk.STOCK_FLOPPY,'text' : _('General')},
+            {'stock_id' : gtk.STOCK_MEDIA_PLAY,'text' : _('Sounds')},
+            {'stock_id' : gtk.STOCK_LEAVE_FULLSCREEN,'text' : _('Notifications')},
+            {'stock_id' : gtk.STOCK_SELECT_COLOR,'text' : _('Theme')},
+            {'stock_id' : gtk.STOCK_EXECUTE,'text' : _('Extensions')},
+            {'stock_id' : gtk.STOCK_DISCONNECT,'text' : _('Plugins')},
+            {'stock_id' : gtk.STOCK_REFRESH,'text' : _('Updates')},
+        ]
+
         self.set_default_size(600, 400)
         self.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_DIALOG)
 
@@ -122,6 +126,7 @@ class Preferences(gtk.Window):
         self.close = gtk.Button(stock=gtk.STOCK_CLOSE)
         self.close.connect('clicked', self.save_and_hide)
         self.buttons.pack_start(self.close)
+        self.set_accels()
 
         vbox.pack_start(self.buttons, False, False)
 
@@ -145,19 +150,42 @@ class Preferences(gtk.Window):
         self.add(vbox)
         vbox.show_all()
 
+    if gui.gtkui.check_gtk3():
+        def hide_on_delete(self, event, data):
+            self.hide()
+            return True
+
     def save_and_hide(self, widget):
         self.hide()
         self.session.save_config()
+        
+    def set_accels(self):
+        """
+        set the keyboard shortcuts
+        """
+        accel_group = gtk.AccelGroup()
+        self.add_accel_group(accel_group)
+        self.accel_group = accel_group
+        accel_group.connect_group(gtk.keysyms.Escape,
+                                  0, gtk.ACCEL_LOCKED,
+                                  self.on_key_close_preferences)
+
+    def on_key_close_preferences(self, accel_group, window, keyval, modifier):
+        '''Catches ESC key event and closes preferences window'''
+        self.save_and_hide(None)
+
 
     def remove_subscriptions(self):
         self.conversation.remove_subscriptions()
         self.sound.remove_subscriptions()
+        self.notifications_page.remove_subscriptions()
+        self.desktop.remove_subscriptions()
         if hasattr(self, 'msn_papylib'):
             self.msn_papylib.privacy.remove_subscriptions()
 
     def remove_from_list(self, icon, text, page):
 
-        LIST.remove({'stock_id' : icon,'text' : text})
+        self.LIST.remove({'stock_id' : icon,'text' : text})
         self.page_dict.remove(page)
         num = self.notebook.page_num(page)
         self.notebook.remove_page(num)
@@ -165,7 +193,7 @@ class Preferences(gtk.Window):
 
     def add_to_list(self, icon, text, page):
 
-        LIST.append({'stock_id' : icon,'text' : text})
+        self.LIST.append({'stock_id' : icon,'text' : text})
         self.page_dict.append(page)
         self.notebook.append_page(page)
         self.__refresh_list()
@@ -184,7 +212,7 @@ class Preferences(gtk.Window):
         self.page_dict.append(self.plugins_page)
         self.page_dict.append(self.updates_page)
 
-        for i in LIST:
+        for i in self.LIST:
             # we should use always the same icon size,
             # we can remove that field in LIST
             self.listStore.append([self.render_icon(i['stock_id'],
@@ -192,14 +220,14 @@ class Preferences(gtk.Window):
 
         msn_in_list = False
         fb_in_list = False
-        for pair in LIST:
+        for pair in self.LIST:
             msn_in_list = msn_in_list or pair['text'] == _('Live Messenger')
             fb_in_list = fb_in_list or pair['text'] == _('Facebook')
 
         if 'msn' in self.session.SERVICES:
         # only when session is papylib.
             if not msn_in_list:
-                LIST.append({'stock_id' : gtk.STOCK_NETWORK,
+                self.LIST.append({'stock_id' : gtk.STOCK_NETWORK,
                                  'text' : _('Live Messenger')})
                 self.listStore.append([self.render_icon(gtk.STOCK_NETWORK,
                                       gtk.ICON_SIZE_LARGE_TOOLBAR),
@@ -208,14 +236,14 @@ class Preferences(gtk.Window):
             self.msn_papylib_page = self.msn_papylib
             self.page_dict.append(self.msn_papylib_page)
         elif msn_in_list:
-            LIST.remove({'stock_id' : gtk.STOCK_NETWORK,
+            self.LIST.remove({'stock_id' : gtk.STOCK_NETWORK,
                          'text' : _('Live Messenger')})
             self.__refresh_list()
 
-        if 'facebook' in self.session.SERVICES:
+        if 'facebook' in self.session.SERVICES and self.session._is_facebook:
         # only when session is papylib.
             if not fb_in_list:
-                LIST.append({'stock_id' : gtk.STOCK_NETWORK,
+                self.LIST.append({'stock_id' : gtk.STOCK_NETWORK,
                                  'text' : _('Facebook')})
                 self.listStore.append([self.render_icon(gtk.STOCK_NETWORK,
                                       gtk.ICON_SIZE_LARGE_TOOLBAR),
@@ -224,7 +252,7 @@ class Preferences(gtk.Window):
             self.facebook_page = self.facebook
             self.page_dict.append(self.facebook_page)
         elif fb_in_list:
-            LIST.remove({'stock_id' : gtk.STOCK_NETWORK,
+            self.LIST.remove({'stock_id' : gtk.STOCK_NETWORK,
                          'text' : _('Facebook')})
             self.__refresh_list()
 
@@ -235,7 +263,7 @@ class Preferences(gtk.Window):
 
         self.listStore.clear()
 
-        for i in LIST:
+        for i in self.LIST:
             # we should use always the same icon size,
             # we can remove that field in LIST
             self.listStore.append([self.render_icon(i['stock_id'],
@@ -244,7 +272,11 @@ class Preferences(gtk.Window):
     def _on_row_activated(self, treeview):
         # Get information about the row that has been selected
         cursor, obj = treeview.get_cursor()
-        self.showPage(cursor[0])
+        if not gui.gtkui.check_gtk3():
+            self.showPage(cursor[0])
+        else:
+            if cursor:
+                self.showPage(cursor.get_indices()[0])
 
     def showPage(self, index):
         self.notebook.set_current_page(index)
@@ -283,6 +315,8 @@ class BaseTable(gtk.Table):
 
         label.set_line_wrap(line_wrap)
         self.attach(label, column, column + 1, row, row + 1, yoptions=0)
+        self.current_row += 1
+
 
     def add_button(self, text, column, row, 
                    on_click, xoptions=gtk.EXPAND|gtk.FILL, 
@@ -307,8 +341,9 @@ class BaseTable(gtk.Table):
         if increment_current_row:
             self.current_row += 1
 
-    def append_entry_default(self, text, format_type, property_name, default):
-        """append a row with a label and a entry, set the value to the
+    def create_entry_default(self, text, format_type,
+                             property_name, default, tooltip_text, has_help=True):
+        """creates a row with a label and a entry, set the value to the
         value of property_name if exists, if not set it to default.
          Add a reset button that sets the value to the default"""
 
@@ -335,12 +370,18 @@ class BaseTable(gtk.Table):
         entry.set_text(text)
 
         reset = gtk.Button()
-        entry_help = gtk.Button()
 
         hbox.pack_start(label)
         hbox.pack_start(entry, False)
         hbox.pack_start(reset, False)
-        hbox.pack_start(entry_help, False)
+        if has_help:
+            entry_help = gtk.Button()
+            hbox.pack_start(entry_help, False)
+            help_image = gtk.image_new_from_stock(gtk.STOCK_HELP,
+                                                  gtk.ICON_SIZE_MENU)
+            entry_help.set_image(help_image)
+            entry_help.connect('clicked', on_help_clicked, format_type)
+            entry_help.set_tooltip_text(tooltip_text)
 
         reset_image = gtk.image_new_from_stock(gtk.STOCK_CLEAR,
                                                gtk.ICON_SIZE_MENU)
@@ -349,10 +390,15 @@ class BaseTable(gtk.Table):
         reset.connect('clicked', on_reset_clicked, entry, default)
         entry.connect('changed', on_entry_changed, property_name)
 
-        help_image = gtk.image_new_from_stock(gtk.STOCK_HELP,
-                                              gtk.ICON_SIZE_MENU)
-        entry_help.set_image(help_image)
-        entry_help.connect('clicked', on_help_clicked, format_type)
+        return hbox
+
+    def append_entry_default(self, text, format_type,
+                             property_name, default, tooltip_text, has_help=True):
+        """append a row with a label and a entry, set the value to the
+        value of property_name if exists, if not set it to default.
+         Add a reset button that sets the value to the default"""
+        hbox = self.create_entry_default(text, format_type,
+                             property_name, default, tooltip_text, has_help)
 
         self.append_row(hbox, None)
 
@@ -375,7 +421,7 @@ class BaseTable(gtk.Table):
         return widget
 
     def append_range(self, text, property_name, 
-                     min_val, max_val,is_int=True):
+                     min_val, max_val,is_int=True, marks=[]):
         """append a row with a scale to select an integer value between
         min and max
         """
@@ -395,6 +441,9 @@ class BaseTable(gtk.Table):
         if is_int:
             scale.set_digits(0)
 
+        for mark in marks:
+            scale.add_mark(mark, gtk.POS_BOTTOM, None)
+
         hbox.pack_start(label, True, True)
         hbox.pack_start(scale, False)
 
@@ -402,6 +451,7 @@ class BaseTable(gtk.Table):
                       property_name, is_int)
 
         self.append_row(hbox, None)
+        return scale
 
     def fill_combo(self, combo, getter, property_name, values=None):
         if values:
@@ -432,7 +482,7 @@ class BaseTable(gtk.Table):
                           property_name, values)
         return combo
 
-    def create_combo_with_label(self, text, 
+    def create_combo_with_label(self, text,
                                 getter, property_name, 
                                 values=None, changed_cb = None):
         """creates and return a new ComboBox with a label and append 
@@ -531,12 +581,6 @@ class BaseTable(gtk.Table):
     def on_update(self):
         pass
 
-    def on_redraw_main_screen(self, button):
-        """called when the Redraw main screen button is clicked"""
-        self.session.save_config()
-        self.session.signals.login_succeed.emit()
-        self.session.signals.contact_list_ready.emit()
-
     def on_import_emesene1(self, button):
         """called when the Import button is clicked"""
         syn = extension.get_default('synch tool')
@@ -558,18 +602,20 @@ class MainWindow(BaseTable):
         ContactList = extension.get_default('contact list')
 
         self.append_markup('<b>'+_('User panel:')+'</b>')
-
         self.append_check(_('Show user panel'),
             'session.config.b_show_userpanel')
-
+        self.append_check(_('Show unread mail count'),
+            'session.config.b_show_mail_inbox')
         self.append_markup('<b>'+_('Contact list:')+'</b>')
-        self.append_range(_('Contact list avatar size'),
-            'session.config.i_avatar_size', 18, 64)
+        avatar_size = self.append_range(_('Contact list avatar size'),
+            'session.config.i_avatar_size', 18, 64, marks=[32, 48])
 
         self.append_entry_default(_('Nick format'), 'nick',
-                'session.config.nick_template_clist', ContactList.NICK_TPL)
+                                  'session.config.nick_template_clist',
+                                  ContactList.NICK_TPL, _('Nick Format Help'))
         self.append_entry_default(_('Group format'), 'group',
-                'session.config.group_template', ContactList.GROUP_TPL)
+                                  'session.config.group_template',
+                                  ContactList.GROUP_TPL, _('Group Format Help'))
 
         if sys.platform == 'darwin':
 
@@ -590,7 +636,7 @@ class MainWindow(BaseTable):
                                        '(requires restart of emesene)'),
                                        'session.config.b_show_dock_icon')
             button.connect("toggled", do_hideshow)
-            self.session.config.get_or_set('b_hide_menu', True)    
+            self.session.config.get_or_set('b_hide_menu', False)    
             button = self.append_check(_('Hide menu'),
                                        'session.config.b_hide_menu')
 
@@ -608,30 +654,6 @@ class ConversationWindow(BaseTable):
         self.set_border_width(5)
         self.session = session
 
-        #language option
-
-        langs = list_dicts()
-
-        self.spell_lang = self.session.config.get_or_set("spell_lang", "en")
-        self.lang_menu = gtk.combo_box_new_text()
-        self.lang_menu.connect("changed", self._on_lang_combo_change)
-
-        index = 0
-        for lang in langs:
-            self.lang_menu.append_text(lang)
-            if lang == self.spell_lang:
-                self.lang_menu.set_active(index)
-            index += 1
-
-        cb_check_spelling = self.create_check(
-            _('Enable spell check if available (requires %s)') 
-            % 'python-gtkspell',
-            'session.config.b_enable_spell_check')
-
-        h_lang_box = gtk.HBox()
-        h_lang_box.pack_start(cb_check_spelling)
-        h_lang_box.pack_start(self.lang_menu)
-
         # override text color option
 
         cb_override_text_color = self.create_check(
@@ -661,99 +683,72 @@ class ConversationWindow(BaseTable):
         self.tab_pos_cb = self.create_combo_with_label(_('Tab position'),
             self.get_tab_positions, 'session.config.i_tab_position',range(4))
 
+        self.int_mode = 2
+        if session.config.b_single_window:
+            self.int_mode = 0
+        elif session.config.b_conversation_tabs:
+            self.int_mode = 1
+            
+        self.integrated_mode_cb = self.create_combo_with_label(_('Integrated mode'),
+            self.get_integrated_mode, 'int_mode', range(3),
+            self._on_integrated_mode_change)
+
         self.session.config.get_or_set('b_avatar_on_left', False)
         self.session.config.get_or_set('b_toolbar_small', False)
-        self.session.config.get_or_set('b_conversation_tabs', True)
         self.session.config.get_or_set('b_escape_hotkey', True)
         self.session.config.get_or_set('b_close_button_on_tabs', True)
-        self.append_check(_('Tabbed Conversations'),
-                'session.config.b_conversation_tabs')
-        self.append_row(self.tab_pos_cb)
         self.session.config.get_or_set('b_show_avatar_in_taskbar', True)
-        self.append_check(_('Start minimized/iconified'),
-                          'session.config.b_conv_minimized')
-        self.append_check(_('Show emoticons'),
-                          'session.config.b_show_emoticons')
+
+        self.append_markup('<b>'+_('Layout')+'</b>')
+        self.append_row(self.integrated_mode_cb)
+        self.append_row(self.tab_pos_cb)
         self.append_check(_('Show conversation header'),
             'session.config.b_show_header')
-        self.append_check(_('Show conversation side panel'),
-            'session.config.b_show_info')
         self.append_check(_('Show conversation toolbar'),
             'session.config.b_show_toolbar')
         self.append_check(_('Show close button on tabs'),
             'session.config.b_close_button_on_tabs')
-        self.append_check(_('Enable escape hotkey to close tabs'),
-            'session.config.b_escape_hotkey')
+        # Avatar-on-left sensitivity depends on side panel visibility
+        self.cb_avatar_left = self.create_check(_('Avatar on conversation left side'),
+            'session.config.b_avatar_on_left')
+        self.append_row(self.cb_avatar_left)
+
+        self.append_markup('<b>'+_('Appareance')+'</b>')
+        self.append_check(_('Show emoticons'),
+                          'session.config.b_show_emoticons')
         # small-toolbar sensitivity depends on conversation toolbar visibility
         self.cb_small_toolbar = self.create_check(_('Small conversation toolbar'),
             'session.config.b_toolbar_small')
         self.session.config.subscribe(self._on_cb_show_toolbar_changed,
             'b_show_toolbar')
         self.append_row(self.cb_small_toolbar)
-
-        # Avatar-on-left sensitivity depends on side panel visibility
-        self.cb_avatar_left = self.create_check(_('Avatar on conversation left side'),
-            'session.config.b_avatar_on_left')
-        self.session.config.subscribe(self._on_cb_side_panel_changed,
-            'b_show_info')
-        self.append_row(self.cb_avatar_left)
-        self.append_check(_('Allow auto scroll in conversation'),
-            'session.config.b_allow_auto_scroll')
-        self.append_row(h_lang_box)
         self.append_check(_('Show avatars in taskbar instead of status icons'),
             'session.config.b_show_avatar_in_taskbar')
 
         self.append_row(h_color_box)
+
         #update ColorButton sensitive
         self._on_cb_override_text_color_toggled(
                 self.session.config.get_or_set('b_override_text_color', False))
 
-        self.append_range(_('Conversation avatar size'),
-            'session.config.i_conv_avatar_size', 18, 128)
+        avatar_size = self.append_range(_('Conversation avatar size'),
+            'session.config.i_conv_avatar_size', 18, 128, marks=[32,64,96])
 
-        self.session.config.subscribe(self._on_spell_change,
-            'b_enable_spell_check')
-
-        self.session.config.subscribe(self._on_conversation_tabs_change,
-            'b_conversation_tabs')
-
-        #update tab_pos combo sensitivity
-        self._on_conversation_tabs_change(self.session.config.get_or_set('b_conversation_tabs', True))
-        #update spell lang combo sensitivity
-        self._on_spell_change(self.session.config.get_or_set('b_enable_spell_check', False))
-        #update side-panel dependent options sensitivity
-        self._on_cb_side_panel_changed(self.session.config.get_or_set('b_show_info', True))
         #update small-toolbar sensitivity
         self._on_cb_show_toolbar_changed(self.session.config.get_or_set('b_show_toolbar', True))
 
+        self.append_markup('<b>'+_('Behavior')+'</b>')
+        self.append_check(_('Start minimized/iconified'),
+                          'session.config.b_conv_minimized')
+        self.append_check(_('Enable escape hotkey to close tabs'),
+            'session.config.b_escape_hotkey')
+        self.append_check(_('Allow auto scroll in conversation'),
+            'session.config.b_allow_auto_scroll')
+
         self.show_all()
 
     def _on_cb_show_toolbar_changed(self, value):
-        if value:
-            self.cb_small_toolbar.set_sensitive(True)
-        else:
-            self.cb_small_toolbar.set_sensitive(False)
-
-    def _on_cb_side_panel_changed(self, value):
-        if value:
-            self.cb_avatar_left.set_sensitive(True)
-        else:
-            self.cb_avatar_left.set_sensitive(False)
-
-    def _on_spell_change(self, value):
-        if value:
-            self.lang_menu.set_sensitive(True)
-        else:
-            self.lang_menu.set_sensitive(False)
-
-    def _on_conversation_tabs_change(self, value):
-        if value:
-            self.tab_pos_cb.set_sensitive(True)
-        else:
-            self.tab_pos_cb.set_sensitive(False)
-
-    def _on_lang_combo_change(self, combo):
-        self.session.config.spell_lang = combo.get_active_text()
+        self.cb_small_toolbar.set_sensitive(value)
 
     def get_tab_positions(self):
         return [_("Top"), _("Bottom"), _("Left"), _("Right")]
@@ -761,20 +756,29 @@ class ConversationWindow(BaseTable):
     def remove_subscriptions(self):
         self.session.config.unsubscribe(self._on_cb_show_toolbar_changed,
             'b_show_toolbar')
-        self.session.config.unsubscribe(self._on_cb_side_panel_changed,
-            'b_show_info')
-        self.session.config.unsubscribe(self._on_spell_change,
-            'b_enable_spell_check')
-        self.session.config.unsubscribe(self._on_conversation_tabs_change,
-            'b_conversation_tabs')
         self.session.config.unsubscribe(self._on_cb_override_text_color_toggled,
             'b_override_text_color')
 
     def _on_cb_override_text_color_toggled(self, value):
-        if value:
-            self.b_text_color.set_sensitive(True)
+        self.b_text_color.set_sensitive(value)
+
+    def _on_integrated_mode_change(self, combo, property_name, value):
+        self.int_mode = combo.get_active()
+        if self.int_mode == 0:
+            self.session.config.b_single_window = True
+            self.session.config.b_conversation_tabs = True
+        elif self.int_mode == 1:
+            self.session.config.b_single_window = False
+            self.session.config.b_conversation_tabs = True
         else:
-            self.b_text_color.set_sensitive(False)
+            self.session.config.b_single_window = False
+            self.session.config.b_conversation_tabs = False
+        self.tab_pos_cb.set_sensitive(self.session.config.b_conversation_tabs)
+
+    def get_integrated_mode(self):
+        return [_("Single Window"),
+                _("Tabbed Conversations"),
+                _("Multiple Conversations")]
 
 class Sound(BaseTable):
     """the panel to display/modify the config related to the sounds
@@ -816,10 +820,7 @@ class Sound(BaseTable):
 
     def _on_mute_sounds_changed(self, value):
         for i in self.array:
-            if value:
-                i.set_sensitive(False)
-            else:
-                i.set_sensitive(True)
+            i.set_sensitive(not value)
 
     def remove_subscriptions(self):
         self.session.config.unsubscribe(self._on_mute_sounds_changed,
@@ -835,22 +836,41 @@ class Notification(BaseTable):
         BaseTable.__init__(self, 4, 1)
         self.set_border_width(5)
         self.session = session
+        self.array = []
+
+        self.append_markup('<b>'+_('General:')+'</b>')
+        self.append_check(_('Mute notification'),
+            'session.config.b_mute_notification')
         self.append_markup('<b>'+_('Users events:')+'</b>')
-        self.append_check(_('Notify on contact online'),
-            'session.config.b_notify_contact_online')
-        self.append_check(_('Notify on contact offline'),
-            'session.config.b_notify_contact_offline')
+        self.array.append(self.append_check(_('Notify on contact online'),
+            'session.config.b_notify_contact_online'))
+        self.array.append(self.append_check(_('Notify on contact offline'),
+            'session.config.b_notify_contact_offline'))
         self.append_markup('<b>'+_('Messages events:')+'</b>')
-        self.append_check(_('Notify on received message'),
-            'session.config.b_notify_receive_message')
-        self.append_check(_('Notify when a contact is typing'),
-            'session.config.b_notify_typing')
-        self.append_check(_('Notify also when the conversation has focus'),
-            'session.config.b_notify_when_focussed')
-        self.append_check(_('Only when available'),
-            'session.config.b_notify_only_when_available')
+        self.array.append(self.append_check(_('Notify on received message'),
+            'session.config.b_notify_receive_message'))
+        self.array.append(self.append_check(_('Notify when a contact is typing'),
+            'session.config.b_notify_typing'))
+        self.array.append(self.append_check(_('Notify also when the conversation has focus'),
+            'session.config.b_notify_when_focussed'))
+        self.array.append(self.append_check(_('Only when available'),
+            'session.config.b_notify_only_when_available'))
+
+        self._on_mute_notification_changed(self.session.config.b_mute_notification)
+
+        self.session.config.subscribe(self._on_mute_notification_changed,
+            'b_mute_notification')
+
         self.show_all()
 
+    def _on_mute_notification_changed(self, value):
+        for i in self.array:
+            i.set_sensitive(not value)
+
+    def remove_subscriptions(self):
+        self.session.config.unsubscribe(self._on_mute_notification_changed,
+            'b_mute_notification')
+
 class Theme(BaseTable):
     """the panel to display/modify the config related to the theme
     """
@@ -1005,10 +1025,7 @@ class Extension(BaseTable):
 
         self.add_text('', 0, 7, True)
 
-        self.add_button(_('Redraw main screen'), 0, 8,
-                self.on_redraw_main_screen, 0, 0)
-
-        self.add_button(_('Import from emesene1'), 1, 8,
+        self.add_button(_('Import from emesene1'), 0, 8,
                 self.on_import_emesene1, 0, 0)
 
     def _add_categories_and_extensions_combos(self):
@@ -1028,10 +1045,13 @@ class Extension(BaseTable):
 
     def _on_category_changed(self, combo):
         """callback called when the category on the combo changes"""
-        self.extensions.disconnect(self.ext_id)
+        if self.extensions.handler_is_connected(self.ext_id):
+            self.extensions.disconnect(self.ext_id)
         self.extensions.get_model().clear()
         self.extension_list = []
         category = combo.get_active_text()
+        if category is None:
+            return
         default = extension.get_default(category)
         extensions = extension.get_extensions(category)
 
@@ -1089,17 +1109,25 @@ class Extension(BaseTable):
     def on_update(self):
         '''called when changed to this page'''
         # empty categories combo
-        model = self.categories.get_model()
-        self.categories.set_model(None)
-        model.clear()
+        if not gui.gtkui.check_gtk3():
+            model = self.categories.get_model()
+            self.categories.set_model(None)
+            model.clear()
+        else:
+            self.categories.remove_all()
+
         # fill it again with available categories
         # this is done because a plugin may have changed them
         categories = extension.get_multiextension_categories()
 
         for item in categories:
-            model.append([item])
+            if not gui.gtkui.check_gtk3():
+                model.append([item])
+            else:
+                self.categories.append_text(item)
 
-        self.categories.set_model(model)
+        if not gui.gtkui.check_gtk3():
+            self.categories.set_model(model)
         self.categories.set_active(0)
 
 class DesktopTab(BaseTable):
@@ -1108,17 +1136,82 @@ class DesktopTab(BaseTable):
     def __init__(self, session):
         """constructor
         """
-        BaseTable.__init__(self, 3, 2)
+        BaseTable.__init__(self, 4, 2)
         self.set_border_width(5)
         self.session = session
 
         self.append_markup('<b>'+_('Logger')+'</b>')
         self.append_check(_('Enable logger'),
                           'session.config.b_log_enabled')
+                          
+        # language settings
+        self.append_markup('<b>'+_('Language')+'</b>')
+        # languages combobox
+        self._language_management = extension.get_and_instantiate('language')
+
+        self.session.config.subscribe(self._on_language_changed,
+                                      'language_config')
+
+        self.add_text(_("Select language:"), 0, 3,  True)
+
+        default = 0
+        index = 1
+        combo_store = gtk.ListStore(str, str)
+        combo_store.append((None, _('Automatic detection')))
+
+        lang_dict = self._language_management.LANGUAGES_DICT
+
+        for lang_key in sorted(lang_dict.keys()):
+            combo_store.append((lang_key, lang_dict[lang_key]))
+            if lang_key == self.session.config.language_config:
+                default = index
+            index += 1
+
+        #in case we have some new language and it's not already on the DICT
+        for lang_key in self._language_management.get_available_languages():
+            if lang_key not in self._language_management.LANGUAGES_DICT.keys():
+                combo_store.append((lang_key, lang_key))
+                if lang_key == self.session.config.language_config:
+                    default = index
+                index += 1
+
+        if gui.gtkui.check_gtk3():
+            self.language_combo = gtk.ComboBox.new_with_model(combo_store)
+        else:
+            self.language_combo = gtk.ComboBox(combo_store)
+
+        cell = gtk.CellRendererText()
+        self.language_combo.pack_start(cell, True)
+        self.language_combo.add_attribute(cell, 'text', 1)
+        self.language_combo.set_active(default)
+
+        self.language_combo.connect('changed', self.on_language_combo_changed,
+                                    'session.config.language_config')
+
+        self.attach(self.language_combo, 2, 3, 3, 4, gtk.FILL, 0)
+
+        #language option
+        self.session.config.get_or_set("spell_lang", "en")
+        self.lang_menu = self.create_combo(self.get_spell_langs, 'session.config.spell_lang')
+
+        cb_check_spelling = self.create_check(
+            _('Enable spell check if available \n(requires %s)')
+            % 'python-gtkspell',
+            'session.config.b_enable_spell_check')
+
+        self.append_row(cb_check_spelling)
+        self.attach(self.lang_menu, 2, 3, 4, 5, gtk.FILL, 0)
+
+        self.session.config.subscribe(self._on_spell_change,
+            'b_enable_spell_check')
+
+        #update spell lang combo sensitivity
+        self._on_spell_change(self.session.config.get_or_set('b_enable_spell_check', False))
+
         self.append_markup('<b>'+_('File transfers')+'</b>')
         self.append_check(_('Sort received files by sender'),
                           'session.config.b_download_folder_per_account')
-        self.add_text(_('Save files to:'), 0, 4, True)
+        self.add_text(_('Save files to:'), 0, 7, True)
 
         def on_path_selected(f_chooser):
             ''' updates the download dir config value '''
@@ -1131,10 +1224,36 @@ class DesktopTab(BaseTable):
             buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                      gtk.STOCK_OPEN, gtk.RESPONSE_OK))
         fc_button = gtk.FileChooserButton(path_chooser)
-        fc_button.set_current_folder(self.session.config.get_or_set("download_folder", \
+        fc_button.set_current_folder(self.session.config.get_or_set("download_folder",
                 e3.common.locations.downloads()))
+        if gui.gtkui.check_gtk3():
+            #setting on path_chooser didn't work
+            fc_button.set_action (gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
         fc_button.connect('selection-changed', on_path_selected)
-        self.attach(fc_button, 2, 3, 4, 5, gtk.EXPAND|gtk.FILL, 0)
+        self.attach(fc_button, 2, 3, 7, 8, gtk.EXPAND|gtk.FILL, 0)
+
+        # mail settings
+        self.append_markup('<b>'+_('Mail')+'</b>')
+        self.append_check(_('Open mail in default desktop client'),
+                          'session.config.b_open_mail_in_desktop')
+
+    def remove_subscriptions(self):
+        self.session.config.unsubscribe(self._on_spell_change,
+            'b_enable_spell_check')
+
+    def _on_language_changed(self,  lang):
+        self._language_management.install_desired_translation(lang)
+    
+    def on_language_combo_changed(self, combo, property_name):
+        index = combo.get_active()
+        lang_key = combo.get_model()[index][0]
+        self.set_attr(property_name, lang_key)
+
+    def _on_spell_change(self, value):
+        self.lang_menu.set_sensitive(value)
+
+    def get_spell_langs(self):
+        return sorted(set(list_dicts()))
 
 class MSNPapylib(BaseTable):
     """ This panel contains some msn-papylib specific settings """
@@ -1156,6 +1275,15 @@ class MSNPapylib(BaseTable):
         c_keep.connect('toggled', self._on_keepalive_toggled)
         vbox.pack_start(c_keep, False, False)
 
+        c_ep = gtk.CheckButton(_("Disconnect other endpoints when logging in"))
+        c_ep.set_active(self.session.config.b_papylib_disconnect_ep)
+        c_ep.connect('toggled', self._on_ep_toggled)
+        vbox.pack_start(c_ep, False, False)
+        s_entry = self.create_entry_default(_('Endpoint name'), 'epname',
+                                  'session.config.s_papylib_endpoint_name',
+                                  'emesene', _('Name for this endpoint'), has_help=False)
+        vbox.pack_start(s_entry, False, False)
+
         l_text = gtk.Label(_('If you have problems with your nickname/message/picture '
                         'just click on this button, sign in with your account '
                         'and load a picture in your Live Profile. '
@@ -1187,9 +1315,16 @@ class MSNPapylib(BaseTable):
         worker.keepalive_conversations = widget.get_active()
         self.session.config.b_papylib_keepalive = widget.get_active()
 
+    def _on_ep_toggled(self, widget):
+        ''' enable/disable conversation's keepalives in papyon '''
+        worker = self.session.get_worker()
+        worker.disconnect_ep = widget.get_active()
+        self.session.config.b_papylib_disconnect_ep = widget.get_active()
+
     def _on_live_profile_clicked(self, arg):
         ''' called when live profile button is clicked '''
-        gui.base.Desktop.open("http://profile.live.com/details/Edit/Pic")
+        profile_url = self.session.get_worker().profile_url
+        gui.base.Desktop.open(profile_url)
 
 class Facebook(BaseTable):
     """ This panel contains some facebook specific settings """
@@ -1212,18 +1347,32 @@ class Facebook(BaseTable):
                           'session.config.b_fb_status_write')
         self.append_check(_('Automatically download profile photo'),
                           'session.config.b_fb_picture_download')
+
         # box with help message
-        eventBox = gtk.EventBox()
-        eventBox.modify_bg(gtk.STATE_NORMAL, gtk.gdk.Color('#EDDE5C'))
+        if hasattr(gtk, "InfoBar"):
+            eventBox = gtk.InfoBar()
+            eventBox.set_message_type(gtk.MESSAGE_INFO)
+            box = eventBox.get_content_area ()
+        else:
+            eventBox = gtk.EventBox()
+            eventBox.modify_bg(gtk.STATE_NORMAL, gtk.gdk.Color('#EDDE5C'))
+            # icon
+            box = gtk.HBox()
+            eventBox.add(box)
+
+        # icon
+        image = gtk.image_new_from_stock(gtk.STOCK_DIALOG_INFO,
+                                         gtk.ICON_SIZE_LARGE_TOOLBAR)
+        box.pack_start(image)
 
         markup = '<span foreground="black"> %s </span>'
         noticelabel = gtk.Label()
         text = _("<b>WARNING: This will reset your facebook token."
                  "\nemesene will ask you to login into facebook on"
                  " next login</b>")
-
         noticelabel.set_markup(markup % text)
-        eventBox.add(noticelabel)
+        box.pack_start(noticelabel)
+
         self.append_row(eventBox, None)
 
         self.add_button(_('Reset Facebook settings'), 0, 7,
@@ -1246,16 +1395,22 @@ class PrivacySettings(gtk.VBox):
         self.session = session
 
         # box with help message
-        eventBox = gtk.EventBox()
-        eventBox.modify_bg(gtk.STATE_NORMAL, gtk.gdk.Color('#EDDE5C'))
+        if hasattr(gtk, "InfoBar"):
+            eventBox = gtk.InfoBar()
+            eventBox.set_message_type(gtk.MESSAGE_INFO)
+            box = eventBox.get_content_area ()
+        else:
+            eventBox = gtk.EventBox()
+            eventBox.modify_bg(gtk.STATE_NORMAL, gtk.gdk.Color('#EDDE5C'))
+            # icon
+            box = gtk.HBox()
+            eventBox.add(box)
         self.pack_start(eventBox, False, False)
 
         # icon
-        box = gtk.HBox()
         image = gtk.image_new_from_stock(gtk.STOCK_DIALOG_INFO,
                                          gtk.ICON_SIZE_LARGE_TOOLBAR)
         box.pack_start(image)
-        eventBox.add(box)
 
         # tooltip labels
         labels_box = gtk.VBox()
@@ -1445,7 +1600,7 @@ class PrivacySettings(gtk.VBox):
         contact = self.block_model.get_value(iter, 0)
         self.session.unblock(contact)
 
-    def _render_lists(self, column, render, model, iter):
+    def _render_lists(self, column, render, model, iter, *args):
         ''' changes the cell background according to the contact condition '''
         contact = model.get_value(iter, 0)
 
