@@ -102,9 +102,6 @@ class CellRendererFunction(gtk.GenericCellRenderer):
         if prop.name not in self.property_names:
             raise TypeError('No property named %s' % (prop.name,))
 
-        if prop == 'markup': #plus formatting
-            value = Plus.msnplus_to_dict
-
         self.__dict__[prop.name] = value
 
     def on_render(self, win, widget, bgnd_area, cell_area, expose_area, flags):
@@ -182,8 +179,6 @@ class CellRendererPlus(CellRendererFunction):
 
 extension.implements(CellRendererPlus, 'nick renderer')
 
-gobject.type_register(CellRendererPlus)
-
 class CellRendererNoPlus(CellRendererFunction):
     '''Nick renderer that "strip" MSN+ markup, not showing any effect/color,
     but improving the readability'''
@@ -193,7 +188,6 @@ class CellRendererNoPlus(CellRendererFunction):
     AUTHOR = 'Mariano Guerra'
     WEBSITE = 'www.emesene.org'
 
-
     def __init__(self):
         global plus_or_noplus
         plus_or_noplus = 0
@@ -201,8 +195,6 @@ class CellRendererNoPlus(CellRendererFunction):
 
 extension.implements(CellRendererNoPlus, 'nick renderer')
 
-gobject.type_register(CellRendererNoPlus)
-
 class SmileyLayout(pango.Layout):
     '''a pango layout to draw smilies'''
 
@@ -251,6 +243,7 @@ class SmileyLayout(pango.Layout):
 
     def set_markup(self, markup):
         ''' Same as set_text() '''
+        markup = msnplus_to_list(markup)
         self.set_element_list(markup)
 
     def set_width(self, width):
@@ -328,13 +321,10 @@ class SmileyLayout(pango.Layout):
 
         pango.Layout.set_text(self, text)
 
-        if hasattr(pango, 'find_base_dir'):
-            for line in text.splitlines():
-                if (pango.find_base_dir(line, -1) == pango.DIRECTION_RTL):
-                    self._is_rtl = True
-                    break
-        else:
-            self._is_rtl = False
+        for line in text.splitlines():
+            if pango.find_base_dir(line, -1) == pango.DIRECTION_RTL:
+                self._is_rtl = True
+                break
 
         logical = self.get_line(0).get_pixel_extents()[1]
         ascent = pango.ASCENT(logical)
@@ -546,11 +536,19 @@ class SmileyLabel(gtk.Label):
         gtk.Widget.__init__(self)
         self.angle = 0
         self._text = ['']
+        self.markup = ''
         self._ellipsize = True
         self._wrap = True
         self._smiley_layout = None
         self.set_flags(self.flags() | gtk.NO_WINDOW)
         self._smiley_layout = SmileyLayout(self.create_pango_context())
+        extension.subscribe(self._on_nick_renderer_changed, 'nick renderer')
+
+    def __del__(self):
+        extension.unsubscribe(self._on_nick_renderer_changed, 'nick renderer')
+
+    def _on_nick_renderer_changed(self, new_extension):
+        self.set_markup(self.markup)
 
     def set_angle(self, angle):
         self.angle = angle
@@ -567,7 +565,9 @@ class SmileyLabel(gtk.Label):
         self.queue_resize()
 
     def set_markup(self, text=['']):
-        self.set_text(text)
+        markup = msnplus_to_list(text)
+        self.markup = text
+        self.set_text(markup)
 
     def set_text(self, text=['']):
         ''' Sets widget text '''
@@ -622,8 +622,6 @@ class SmileyLabel(gtk.Label):
         ctx = event.window.cairo_create()
         self._smiley_layout.draw(ctx, area)
 
-gobject.type_register(SmileyLabel)
-
 #from emesene1 by mariano guerra adapted by cando
 #animation support by cando
 #TODO add transformation field in configuration
@@ -750,6 +748,4 @@ class AvatarRenderer(gtk.GenericCellRenderer, AvatarManager):
 
         if avatar:
             self.draw_avatar(ctx, avatar, width - dim, ypad, dim,
-                gtk.ANCHOR_CENTER, self._radius_factor, alpha)
-
-gobject.type_register(AvatarRenderer)
+                gtk.gdk.GRAVITY_CENTER, self._radius_factor, alpha)
