@@ -22,6 +22,7 @@ import os
 
 import e3
 import gui
+from gui.gtkui import check_gtk3
 import utils
 import RichBuffer
 
@@ -33,6 +34,11 @@ log = logging.getLogger('gtkui.Textbox')
 class TextBox(gtk.ScrolledWindow):
     '''a text box inside a scroll that provides methods to get and set the
     text in the widget'''
+    __gsignals__ = {
+            "search_request": (gobject.SIGNAL_RUN_FIRST,
+                gobject.TYPE_NONE,
+                (gobject.TYPE_PYOBJECT,))
+            }
 
     def __init__(self, config, on_drag_data_received=None):
         '''constructor'''
@@ -176,11 +182,8 @@ class TextBox(gtk.ScrolledWindow):
 
     text = property(fget=_get_text, fset=_set_text)
 
-gobject.type_register(TextBox)
-gobject.signal_new("search_request", TextBox, gobject.SIGNAL_RUN_FIRST,
-                   gobject.TYPE_NONE, (gobject.TYPE_PYOBJECT,))
-
 class InputView(gtk.TextView):
+
     __gsignals__ = {
         'message-send':(gobject.SIGNAL_RUN_LAST|gobject.SIGNAL_ACTION,
                         gobject.TYPE_NONE, ())
@@ -189,10 +192,16 @@ class InputView(gtk.TextView):
     def __init__(self):
         gobject.GObject.__init__(self)
         gtk.TextView.__init__(self)
-        gtk.binding_entry_add_signal(self, gtk.keysyms.KP_Enter, 0, 'message-send')
-        gtk.binding_entry_add_signal(self, gtk.keysyms.Return, 0, 'message-send')
+        self.connect('key-press-event', self.on_key_press_event)
 
-gobject.type_register(InputView)
+    def on_key_press_event(self, widget, event):
+        if event.keyval in [gtk.keysyms.Return, gtk.keysyms.KP_Enter] and \
+            not event.state & gtk.gdk.SHIFT_MASK:
+
+            if not check_im_context_filter_keypress(self, event):
+                self.emit('message-send')
+            return True
+        return False
 
 class InputText(TextBox):
     '''a widget that is used to insert the messages to send'''
@@ -222,7 +231,8 @@ class InputText(TextBox):
 
         self.spell_checker = None
 
-        if self.config.b_enable_spell_check:
+        #FIXME: gtk2 only as there isn't gtk3 package yet
+        if not check_gtk3() and self.config.b_enable_spell_check:
             try:
                 import gtkspell
                 spell_lang = self.config.get_or_set("spell_lang", "en")
@@ -254,17 +264,20 @@ class InputText(TextBox):
         self.changed = True
         self.apply_tag()
 
-        if event.state == gtk.gdk.CONTROL_MASK and \
-                ((event.keyval < 256 and chr(event.keyval) == "p") or \
-                    event.keyval == gtk.keysyms.Up):
+        if ( event.state & gtk.gdk.CONTROL_MASK ) and \
+                 ( event.keyval == gtk.keysyms.p or \
+                    event.keyval == gtk.keysyms.Up ):
 
-            self.on_cycle_history()
+            if not check_im_context_filter_keypress(self._textbox, event):
+                self.on_cycle_history()
+            return True
+        elif ( event.state & gtk.gdk.CONTROL_MASK ) and \
+                ( event.keyval == gtk.keysyms.n or \
+                    event.keyval == gtk.keysyms.Down ):
 
-        elif event.state == gtk.gdk.CONTROL_MASK and \
-                ((event.keyval < 256 and chr(event.keyval) == "n") or \
-                    event.keyval == gtk.keysyms.Down):
-
-            self.on_cycle_history(1)
+            if not check_im_context_filter_keypress(self._textbox, event):
+                self.on_cycle_history(1)
+            return True
         else:
             if self.typing_timeout is None:
                 self.send_typing_notification()
@@ -447,3 +460,10 @@ class OutputText(TextBox):
                     path, tip = obj
                     if path == what[2]:
                         self.add_image_at_anchor(anchor, path, tip)
+
+def check_im_context_filter_keypress(target, event):
+    ''' return True if the event is handled by Input Method '''
+    if hasattr(target, 'im_context_filter_keypress') and \
+       target.im_context_filter_keypress(event):
+        return True
+    return False
