@@ -20,7 +20,12 @@ import gtk
 import pango
 import gobject
 
-import Renderers
+from gui.gtkui import check_gtk3
+
+if check_gtk3():
+    import RenderersNew as Renderers
+else:
+    import Renderers
 
 class TextField(gtk.VBox):
     '''this class represent a widget that is a button and when clicked
@@ -119,8 +124,8 @@ class TextField(gtk.VBox):
     def _set_text(self, value):
         '''set the value of text'''
         self._text = value
-        self.label.set_markup(Renderers.msnplus_to_list(
-            gobject.markup_escape_text(self._text or self.empty_text)))
+        self.label.set_markup(
+            gobject.markup_escape_text(self._text or self.empty_text))
         self.entry.set_text(self._text)
 
     text = property(fget=_get_text, fset=_set_text)
