@@ -33,24 +33,27 @@ class Conversation (gui.base.Conversation, QtGui.QWidget):
         gui.base.Conversation.__init__(self, session, conv_id, None, members)
         QtGui.QWidget.__init__(self, parent)
         
-        self._session = session
-        self._conv_id = conv_id
-        self._members = members
-        self.avatar_box_is_hidden = False
         self._on_typing_timer = QtCore.QTimer()
         
         # a widget dic to avoid proliferation of instance variables:
         self._widget_d = {}
-        # an action dict to avoid proliferation of instance variables:
-        self._action_dict = {}
         
         self._setup_ui()
-        
+
+        #update information
+        self._widget_d['my_display_pic'].set_display_pic_of_account()
+        if self.members:
+            account = self.members[0]
+            contact = self.session.contacts.safe_get(account)
+            self._widget_d['his_display_pic'].set_display_pic_of_account(
+                                                                account)
+            self.set_sensitive(not contact.blocked, True)
+
         # emesene's
         self.tab_index = 0
         self.input = self._widget_d['chat_input']
         self.output = self._widget_d['chat_output']
-        
+
         #FIXME: move this to base class
         self._load_style()
         self._widget_d['chat_input'].e3_style = self.cstyle
@@ -62,69 +65,45 @@ class Conversation (gui.base.Conversation, QtGui.QWidget):
         self._on_typing_timer.timeout.connect(
             lambda *args: self._widget_d['info_panel'].set_icon(
                 gui.theme.image_theme.status_icons[
-                    self._session.contacts.get(
-                        self._members[0]).status]))
+                    self.session.contacts.get(
+                        self.members[0]).status]))
         self.subscribe_signals()
-        
-        
+
     def __del__(self):
         log.debug('conversation adieeeeeeeuuuu ;______;')
-        
-        
+
     def _setup_ui(self):
         '''Instantiates the widgets, and sets the layout'''
         widget_d = self._widget_d
-        action_dict = self._action_dict
         
         # Classes
         conv_output_cls     = extension.get_default('conversation output')
         smiley_chooser_cls  = extension.get_default('smiley chooser')
         avatar_cls          = extension.get_default('avatar')
         info_panel_cls      = extension.get_default('info panel')
-        
-        # Actions
-        icon_path = gui.theme.emote_theme.emote_to_path(':)')[6:]
-        action_dict['add_smiley']   = QtGui.QAction(
-                            QtGui.QIcon(icon_path), tr('Add Smiley'), self)
-        icon_path = gui.theme.emote_theme.emote_to_path(':S')[6:]
-        action_dict['send_nudge']   = QtGui.QAction(
-                            QtGui.QIcon(icon_path), tr('Send Nudge'), self)
-        action_dict['change_font']  = QtGui.QAction(
-                            QtGui.QIcon.fromTheme("preferences-desktop-font"),  tr('Change Font'), self)
-        action_dict['change_color'] = QtGui.QAction(
-                            QtGui.QIcon(""), tr('Change Color'), self) 
-        action_dict['toggle_avatars'] = QtGui.QAction(
-                            QtGui.QIcon.fromTheme("go-next"), tr('Toggle Avatars'), self) 
-   
+        conv_toolbar_cls      = extension.get_default('conversation toolbar')
         
         # TOP LEFT
-        widget_d['chat_output'] = conv_output_cls()
+        widget_d['chat_output'] = conv_output_cls(self.session.config)
         top_left_lay = QtGui.QHBoxLayout()
         top_left_lay.addWidget(widget_d['chat_output'])
-        
-        
+
         # BOTTOM LEFT
-        widget_d['toolbar'] = QtGui.QToolBar(self)
+        self.toolbar_handler = gui.base.ConversationToolbarHandler(self.session, gui.theme, self)
+        widget_d['toolbar'] = conv_toolbar_cls(self.toolbar_handler, self.session)
+        widget_d['toolbar'].redraw_ublock_button(self._get_first_contact().blocked)
+        widget_d['toolbar'].update_toggle_avatar_icon(self.session.config.b_show_info)
+
         widget_d['smiley_chooser'] = smiley_chooser_cls()
         widget_d['chat_input'] = Widgets.ChatInput()
-        
+
         text_edit_lay = QtGui.QHBoxLayout()
         text_edit_lay.addWidget(widget_d['chat_input'])
         
         bottom_left_lay = QtGui.QVBoxLayout()
         bottom_left_lay.addWidget(widget_d['toolbar'])
         bottom_left_lay.addLayout(text_edit_lay)
-        
-        toolbar = widget_d['toolbar']
-        toolbar.setToolButtonStyle(Qt.ToolButtonIconOnly)
-        toolbar.addAction(action_dict['change_font'])
-        toolbar.addAction(action_dict['change_color'])
-        toolbar.addSeparator()
-        toolbar.addAction(action_dict['add_smiley'])
-        toolbar.addAction(action_dict['send_nudge'])
-        toolbar.addSeparator()
-        toolbar.addAction(action_dict['toggle_avatars'])
-        
+
         widget_d['chat_input'].set_smiley_dict(gui.theme.emote_theme.emotes)
 
         widget_d['smiley_chooser'].emoticon_selected.connect(
@@ -135,18 +114,6 @@ class Conversation (gui.base.Conversation, QtGui.QWidget):
                             self._on_new_style_selected)
 
         dialog = extension.get_default('dialog')
-        self.toolbar_handler = gui.base.ConversationToolbarHandler(self._session,
-            dialog, gui.theme, self)
-        action_dict['add_smiley'].triggered.connect(
-                            self._on_show_smiley_chooser)
-        action_dict['send_nudge'].triggered.connect(
-                            self.toolbar_handler.on_notify_attention_selected)
-        action_dict['change_font'].triggered.connect(
-                            widget_d['chat_input'].show_font_chooser)
-        action_dict['change_color'].triggered.connect(
-                            widget_d['chat_input'].show_color_chooser)
-        action_dict['toggle_avatars'].triggered.connect(
-                            self.toolbar_handler.on_toggle_avatar_selected)
         
         # LEFT (TOP & BOTTOM)
         left_widget = QtGui.QSplitter(Qt.Vertical)
@@ -163,20 +130,14 @@ class Conversation (gui.base.Conversation, QtGui.QWidget):
         left_widget.moveSplitter(splitter_pos, 1)
 
         # RIGHT
-        widget_d['his_display_pic'] = avatar_cls(self._session)
-        widget_d['my_display_pic'] = avatar_cls(self._session)
+        widget_d['his_display_pic'] = avatar_cls(self.session)
+        widget_d['my_display_pic'] = avatar_cls(self.session)
         
         right_lay = QtGui.QVBoxLayout()
         right_lay.addWidget(widget_d['his_display_pic'])
         right_lay.addStretch()
         right_lay.addWidget(widget_d['my_display_pic'])
         
-        widget_d['my_display_pic'].set_display_pic_of_account()
-        if self._members:
-            his_email = self._members[0]
-            widget_d['his_display_pic'].set_display_pic_of_account(
-                                                                his_email)
-
         # LEFT & RIGHT
         widget_d['info_panel'] = info_panel_cls()
         
@@ -186,14 +147,16 @@ class Conversation (gui.base.Conversation, QtGui.QWidget):
         lay = QtGui.QVBoxLayout()
         lay.addWidget(widget_d['info_panel'])
         lay.addLayout(lay_no_info)
-        
-        
+
         self.setLayout(lay)
-        
-        
-    
+
     # emesene's
-    
+
+    def on_font_selected(self, style):
+        '''called when a new font is selected'''
+        gui.base.Conversation.on_font_selected(self, style)
+        self._widget_d['chat_input'].e3_style = style
+
     def input_grab_focus(self):
         '''sets the focus on the input widget'''
         self._widget_d['chat_input'].setFocus(Qt.OtherFocusReason)
@@ -225,7 +188,8 @@ class Conversation (gui.base.Conversation, QtGui.QWidget):
 
     def _on_show_info_changed(self, value):
         '''callback called when config.b_show_info changes'''
-        self.on_toggle_avatar()
+        self.set_image_visible(value)
+        self._widget_d['toolbar'].update_toggle_avatar_icon(value)
 
     def _on_show_avatar_onleft(self,value):
         '''callback called when config.b_avatar_on_left changes'''
@@ -253,7 +217,7 @@ class Conversation (gui.base.Conversation, QtGui.QWidget):
         # account is a string containing the email
         # does this have to update the picture too?
         log.debug('UPSingInfo Start')
-        status = self._session.contacts.get(account).status
+        status = self.session.contacts.get(account).status
         log.debug('UpSingInfo: [%s], [%s], [%s], [%s]' % (status, nick, message, account))
         self._widget_d['info_panel'].set_all(Utils.unescape(message), account)
         log.debug('UPSingInfo Stop')
@@ -266,20 +230,18 @@ class Conversation (gui.base.Conversation, QtGui.QWidget):
         '''Slot called when the user clicks ok in the color chooser or the 
         font chooser'''
         self.cstyle = self._widget_d['chat_input'].e3_style
-   
+
     def _on_show_smiley_chooser(self):
         '''Slot called when the user clicks the smiley button.
         Show the smiley chooser panel'''
         self._widget_d['smiley_chooser'].show()
 
-
     def _on_smiley_selected(self, shortcut):
         '''Slot called when the user selects a smiley in the smiley 
         chooser panel. Inserts the smiley in the chat edit'''
         # handles cursor position
         self._widget_d['chat_input'].insert_text_after_cursor(shortcut)
-        
-    
+
     def _on_send_btn_clicked(self):
         '''Slot called when the user clicks the send button or presses Enter in
         the chat line editor. Sends the message'''
@@ -296,20 +258,20 @@ class Conversation (gui.base.Conversation, QtGui.QWidget):
     def on_contact_attr_changed_succeed(self, account, what, old,
             do_notify=True):
         ''' called when contacts change their attributes'''
-        if account in self._members:
+        if account in self.members:
             if what  == 'status':
                 # FIXME: self.icon doesn't beahave correctly at the 
                 # moment. That's probably because 'set message
                 # waiting' stuff is still not implemented (in fact
                 # self.icon returns that icon.)
                 #self._widget_d['info_panel'].set_icon(self.icon)
-                status = self._session.contacts.get(account).status
+                status = self.session.contacts.get(account).status
                 icon =  gui.theme.image_theme.status_icons[status]
                 self._widget_d['info_panel'].set_icon(icon)
             elif what == 'nick':
                 self._widget_d['info_panel'].set_nick(self.text)
             elif what == 'message':
-                message = self._session.contacts.get(account).message
+                message = self.session.contacts.get(account).message
                 self._widget_d['info_panel'].set_message(message)
                 
     def on_user_typing(self, account):
@@ -327,13 +289,19 @@ class Conversation (gui.base.Conversation, QtGui.QWidget):
         pass
 
 
-    def set_sensitive(self, is_sensitive):
+    def set_sensitive(self, is_sensitive, force_sensitive_block_button=False):
         """
         used to make the conversation insensitive while the conversation
         is still open while the user is disconnected and to set it back to
         sensitive when the user is reconnected
         """
-        pass
+        self._widget_d['toolbar'].set_sensitive(is_sensitive, force_sensitive_block_button)
+        self._widget_d['chat_input'].setEnabled(is_sensitive)
+        self._widget_d['info_panel'].setEnabled(is_sensitive or force_sensitive_block_button)
+
+        # redraws block button to its corresponding icon and tooltip
+        contact_unblocked = not force_sensitive_block_button or is_sensitive
+        self._widget_d['toolbar'].redraw_ublock_button(contact_unblocked)
 
     def set_image_visible(self, is_visible):
         """
@@ -341,12 +309,8 @@ class Conversation (gui.base.Conversation, QtGui.QWidget):
 
         is_visible -- boolean that says if the widget should be shown or hidden
         """
-        if is_visible:
-            self._widget_d['his_display_pic'].show()
-            self._widget_d['my_display_pic'].show()
-        else:
-            self._widget_d['his_display_pic'].hide()
-            self._widget_d['my_display_pic'].hide()
+        self._widget_d['his_display_pic'].setVisible(is_visible)
+        self._widget_d['my_display_pic'].setVisible(is_visible)
 
     def set_header_visible(self, is_visible):
         '''
@@ -354,10 +318,7 @@ class Conversation (gui.base.Conversation, QtGui.QWidget):
 
         is_visible -- boolean that says if the widget should be shown or hidden
         '''
-        if is_visible:
-            self._widget_d['info_panel'].show()
-        else:
-            self._widget_d['info_panel'].hide()
+        self._widget_d['info_panel'].setVisible(is_visible)
 
     def set_toolbar_visible(self, is_visible):
         '''
@@ -365,23 +326,20 @@ class Conversation (gui.base.Conversation, QtGui.QWidget):
 
         is_visible -- boolean that says if the widget should be shown or hidden
         '''
-        if is_visible:
-            self._widget_d['toolbar'].show()
-        else:
-            self._widget_d['toolbar'].hide()
+        self._widget_d['toolbar'].setVisible(is_visible)
+
+    def _get_first_contact(self):
+        account = self.members[0]
+        contact = self.session.contacts.contacts[account]
+        return contact
 
-    def on_toggle_avatar(self):
-        '''hide or show the avatar bar'''
-        self.set_image_visible(self.avatar_box_is_hidden)
-        self.avatar_box_is_hidden = not self.avatar_box_is_hidden
-        
     def on_filetransfer_invitation(self, transfer, conv_id):
         ''' called when a new file transfer is issued '''
-        if self._conv_id != conv_id:
+        if self.icid != conv_id:
             return
             
         self.transferw = extension.get_and_instantiate('filetransfer widget', 
-                                                       self._session, transfer)
+                                                       self.session, transfer)
         self.transferw.show()
 
     def on_filetransfer_accepted(self, transfer):
