@@ -8,6 +8,7 @@ from PyQt4  import QtCore
 from PyQt4  import QtGui
 from PyQt4.QtCore  import Qt
 
+from gui.qt4ui import Utils
 from gui.qt4ui.Utils import tr
 
 import gui
@@ -23,7 +24,7 @@ class Dialog(object):
     WEBSITE = ''
 
     @classmethod
-    def broken_profile(cls, close_cb):
+    def broken_profile(cls, close_cb, url):
         '''a dialog that asks you to fix your profile'''
         message = _('''\
 Your live profile seems to be broken,
@@ -39,7 +40,7 @@ To fix your profile, emesene must be closed.
 Clicking Yes will close emesene.
 
 Do you want to fix your profile now?''')
-        gui.base.Desktop.open("http://profile.live.com/details/Edit/Pic")
+        gui.base.Desktop.open(url)
 
         reply = QtGui.QMessageBox.warning(None, _("You have a broken profile"),
                                           message, QtGui.QMessageBox.Yes |
@@ -438,8 +439,6 @@ Do you want to fix your profile now?''')
         '''show a confirm dialog displaying a question and two buttons:
         Yes and No, return the response as stock.YES or stock.NO or
         stock.CLOSE if the user closes the window'''
-        print response_cb
-        print args
         dialog  = YesNoDialog('')
         icon    = QtGui.QLabel()
         message = QtGui.QLabel(unicode(message))
@@ -455,6 +454,39 @@ Do you want to fix your profile now?''')
 
         response_cb(response, *args)
 
+    @classmethod
+    def select_font(cls, style, callback):
+        '''select font and if available size and style, receives a
+        e3.Message.Style object with the current style
+        the callback receives a new style object with the new selection
+        '''
+        new_qt_font, result = QtGui.QFontDialog.getFont()
+        if result:
+            estyle = Utils.qfont_to_style(new_qt_font, style.color)
+            callback(estyle)
+
+    @classmethod
+    def select_emote(cls, session, theme, callback, max_width=16):
+        '''select an emoticon, receives a gui.Theme object with the theme
+        settings the callback receives the response and a string representing
+        the selected emoticon
+        '''
+        smiley_chooser_cls = extension.get_default('smiley chooser')
+        smiley_chooser = smiley_chooser_cls()
+        smiley_chooser.emoticon_selected.connect(callback)
+        smiley_chooser.show()
+
+    @classmethod
+    def select_color(cls, color, callback):
+        '''select color, receives a e3.Message.Color with the current
+        color the callback receives a new color object woth the new selection
+        '''
+        qt_color = Utils.e3_color_to_qcolor(color)
+        new_qt_color = QtGui.QColorDialog.getColor(qt_color)
+        if new_qt_color.isValid() and not new_qt_color == qt_color:
+            color = Utils.qcolor_to_e3_color(new_qt_color)
+            callback(color)
+
 class StandardButtonDialog (QtGui.QDialog):
     '''Skeleton for a dialog window with standard buttons'''
     def __init__(self, title, expanding=False, parent=None):
@@ -674,4 +706,4 @@ class EntryDialog (OkCancelDialog):
         self.setLayout(lay)
 
     def text(self):
-        return unicode(self.edit.text())
\ No newline at end of file
+        return unicode(self.edit.text())
