@@ -97,7 +97,6 @@ class Preferences(QtGui.QWidget):
                                                         self._on_row_activated)
 
 
-
     def _on_row_activated(self, new_idx, old_idx):
         '''Callback executed when the user clicks an elemet in the list 
         on the left. Shows the matching preference page in the righ part 
@@ -107,15 +106,16 @@ class Preferences(QtGui.QWidget):
         self.widget_stack.currentWidget().on_update()
         
     
-    def _present(self):
+    def remove_subscriptions(self):
+        '''RemovesNothing...'''
+        pass
+    
+    def present(self):
         '''Does Nothing...'''
         pass
 
 
 
-
-
-
 class QListViewMod (QtGui.QListView):
     '''This is a QListView with a modified sizeHint'''
     def __init__(self, parent=None):
@@ -146,9 +146,6 @@ class QListViewMod (QtGui.QListView):
 
 
 
-
-
-
 class BaseTable(QtGui.QWidget):
     '''This is the base class to create a table to edit preferences'''
     def __init__(self, rows, columns, homogeneous=False, parent=None):
@@ -717,4 +714,6 @@ class MSNPapylib(BaseTable):
 
     def _on_live_profile_clicked(self, arg):
         ''' called when live profile button is clicked '''
-        gui.base.Desktop.open("http://profile.live.com/details/Edit/Pic")
+        profile_url = self.session.get_worker().profile_url
+        gui.base.Desktop.open(profile_url)
+
