@@ -64,20 +64,6 @@ class TopLevelWindow (QtGui.QMainWindow):
         position = self.pos()
         return size.width(), size.height(), position.x(), position.y()
     
-    def get_screen(self):
-        ''' Mimics Gtk's built-in method. Necessary because emesene.py relies
-        on this.
-        '''
-        # TODO: change emesene.py or update base class
-        class Screen(object):
-            def __init__(self):
-                pass
-            def get_width(self):
-                return QtGui.QApplication.instance().desktop().availableGeometry().width()
-            def get_height(self):
-                return QtGui.QApplication.instance().desktop().availableGeometry().height()
-        return Screen()
-
     def go_connect(self, on_cancel_login, avatar_path, config):
         '''Adds a 'connecting' page to the top level window and shows it'''
         log.debug('GO CONNECT! ^_^')
@@ -115,20 +101,18 @@ class TopLevelWindow (QtGui.QMainWindow):
             self._switch_to_page(login_page)
         self.setMenuBar(None)
 
-    def go_main(self, session, on_new_conversation,
-            on_close, on_disconnect, quit_on_close=False):
+    def go_main(self, session, on_new_conversation, quit_on_close=False):
         '''Adds a main page (the one with the contact list) to the top
         level window and shows it'''
         log.debug('GO MAIN! ^_^')
         # TODO: handle quit_on_close (??) [consider creating a base class and 
         # moving code there.] 
         main_window_cls = extension.get_default('main window')
-        main_page = main_window_cls(session, on_new_conversation, on_close, 
-                                    on_disconnect, self.setMenuBar)
+        main_page = main_window_cls(session, on_new_conversation, self.setMenuBar)
         self._content_type = 'main'
         self._switch_to_page(main_page)
-        self._setup_main_menu(session, main_page.contact_list, 
-                              on_close, on_disconnect)
+        self._setup_main_menu(session, main_page.contact_list)
+
         if quit_on_close:
             self._cb_on_close = self._given_cb_on_close
         else:
@@ -140,7 +124,7 @@ class TopLevelWindow (QtGui.QMainWindow):
     
     def maximize(self):
         '''Tries to maximize the window'''
-        QtGui.QMainWindow.showMaximized()
+        QtGui.QMainWindow.showMaximized(self)
         
     def on_disconnect(self, cb_on_close):
         '''called when the user is disconnected'''
@@ -165,7 +149,7 @@ class TopLevelWindow (QtGui.QMainWindow):
         self._cb_on_close(self._content)
         self.hide()
         
-    def _setup_main_menu(self, session, contact_list, on_close, on_disconnect):
+    def _setup_main_menu(self, session, contact_list):
         '''build all the menus used on the client'''
         
         # retrieving classes:
@@ -175,13 +159,10 @@ class TopLevelWindow (QtGui.QMainWindow):
         avatar_manager = None #avatar_manager_cls(self._session)
         
         # create menu handlers
-        menu_hnd = gui.base.MenuHandler(session, dialog_cls, contact_list,  
-                                        on_disconnect, on_close)
+        menu_hnd = gui.base.MenuHandler(session, contact_list)
 
-        contact_hnd = gui.base.ContactHandler(session, dialog_cls,
-                                              contact_list)
-        group_hnd = gui.base.GroupHandler(session, dialog_cls,
-                                          contact_list)
+        contact_hnd = gui.base.ContactHandler(session, contact_list)
+        group_hnd = gui.base.GroupHandler(session, contact_list)
         # retrieve menu classes
         main_menu_cls = extension.get_default('main menu')
         contact_menu_cls = extension.get_default('menu contact')
