@@ -94,13 +94,13 @@ class LoginPage(QtGui.QWidget, gui.LoginBase):
         account_img = QtGui.QLabel()
         account_img.setPixmap(QtGui.QPixmap(gui.theme.image_theme.user))
         widget_d['account_img'] = account_img
-        widget_d['forgive_me_btn']  = QtGui.QToolButton()
-        widget_d['forgive_me_btn'].setAutoRaise(True)
-        widget_d['forgive_me_btn'].setIcon(
+        widget_d['forget_me_btn']  = QtGui.QToolButton()
+        widget_d['forget_me_btn'].setAutoRaise(True)
+        widget_d['forget_me_btn'].setIcon(
                                     QtGui.QIcon.fromTheme('edit-delete'))
         account_box.addWidget(widget_d['account_img'])
         account_box.addWidget(widget_d['account_combo'])
-        account_box.addWidget(widget_d['forgive_me_btn'])
+        account_box.addWidget(widget_d['forget_me_btn'])
 
 
         password_box = QtGui.QHBoxLayout()
@@ -174,8 +174,8 @@ class LoginPage(QtGui.QWidget, gui.LoginBase):
                                     self._on_checkbox_state_refresh)
         widget_d['advanced_btn'].clicked.connect(
                                     self._on_connection_preferences_clicked)
-        widget_d['forgive_me_btn'].clicked.connect(
-                                    self._on_forgive_me_clicked)
+        widget_d['forget_me_btn'].clicked.connect(
+                                    self._on_forget_me_clicked)
         widget_d['login_btn'].clicked.connect(
                                     self._on_start_login)
 
@@ -303,14 +303,14 @@ class LoginPage(QtGui.QWidget, gui.LoginBase):
                                         self.server_port, new_preferences_cb, 
                                         self.config.b_use_http, self.proxy)
 
-    def _on_forgive_me_clicked(self):
+    def _on_forget_me_clicked(self):
         ''''''
         def _yes_no_cb(response):
             '''callback from the confirmation dialog'''
             if (response == 36): #Accepted
                 account = str(widget_dic['account_combo'].currentText())
                 service = self.config.d_user_service.get(account, self.config.service)
-                self.forgive_user(account, service)
+                self.forget_user(account, service)
                 #FIXME: remove fron account_list and combo
                 widget_dic['account_combo'].setCurrentIndex(0)
 
@@ -390,6 +390,27 @@ class LoginPage(QtGui.QWidget, gui.LoginBase):
         else:
             login_btn.setEnabled(False)
 
+    def clear_all(self, clear_pic=False):
+        '''
+        clear all login fields and checkbox
+        '''
+        widget_dic = self._widget_d
+        if clear_pic:
+            widget_dic['display_pic'].set_default_pic()
+        widget_dic['password_edit'].clear()
+        widget_dic['status_combo'].set_status(e3.status.ONLINE)
+        # _on_checkbox_state_changed enables them:
+        widget_dic['auto_login_chk'].setChecked(False)
+        widget_dic['save_password_chk'].setChecked(False)
+        widget_dic['save_account_chk'].setChecked(False)
+
+    def show_error(self, reason):
+        #implement nicebar in qt4
+        '''
+        show an error on the top of the window using nicebar
+        '''
+        pass
+
     # -------------------- QT_OVERRIDE
 
     def eventFilter(self, obj, event):
