@@ -2,7 +2,6 @@
 
 '''This module contains the AdiumChatOutput class'''
 import base64
-import webbrowser
 
 from PyQt4      import QtGui
 from PyQt4      import QtCore
@@ -27,10 +26,11 @@ class AdiumChatOutput (QtGui.QScrollArea):
     # pylint: enable=W0612
     
     
-    def __init__(self, parent=None):
+    def __init__(self, config, parent=None):
         QtGui.QScrollArea.__init__(self, parent)
         
         self.theme = gui.theme.conv_theme
+        self.config = config
         self._qwebview = QtWebKit.QWebView(self)
         
         self.setWidget(self._qwebview)
@@ -44,35 +44,34 @@ class AdiumChatOutput (QtGui.QScrollArea):
         self._qwebview.setHtml(body)
         
         self._qwebview.linkClicked.connect(
-                        lambda qt_url: webbrowser.open(qt_url.toString()) )
-                            
-    def _append_message(self, msg):
+                        lambda qt_url: gui.base.Desktop.open(qt_url.toString()) )
+
+    def clear(self, source="", target="", target_display="",
+            source_img="", target_img=""):
+        '''clear the content'''
+        body = self.theme.get_body(source, target, target_display, source_img,
+                target_img)
+        self._qwebview.setHtml(body)
+
+    def _append_message(self, msg, scroll=True):
         '''add a message to the conversation'''
 
-        html = self.theme.format(msg)
+        html = self.theme.format(msg, scroll)
 
-        if msg.type == "status":
-            msg.first = True
-
-        if msg.first:
-            function = "appendMessage('" + html + "')"
-        else:
-            function = "appendNextMessage('" + html + "')"
-
-        self._qwebview.page().mainFrame().evaluateJavaScript(function)
+        self._qwebview.page().mainFrame().evaluateJavaScript(html)
 
     def send_message(self, formatter, msg):
         '''add a message to the widget'''
-        self._append_message(msg)
+        self._append_message(msg, self.config.b_allow_auto_scroll)
 
     def receive_message(self, formatter, msg):
         '''add a message to the widget'''
-        self._append_message(msg)
+        self._append_message(msg, self.config.b_allow_auto_scroll)
 
     def information(self, formatter, msg):
         '''add an information message to the widget'''
         msg.message = Plus.msnplus_strip(msg.message)
-        self._append_message(msg)
+        self._append_message(msg, self.config.b_allow_auto_scroll)
 
     def update_p2p(self, account, _type, *what):
         ''' new p2p data has been received (custom emoticons) '''
